package provide DS9 1.0

proc CircleDialog {frame id} {
    global marker

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	raise $w
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuCircleDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordCircleCB

    # file menu items

    $mb.file add command -label "Apply" \
	-command "ApplyCircleDialog $frame $id"
    $mb.file add separator
    $mb.file add command -label "Close" \
	-command "CloseCircleDialog $frame $id"

    # other circle menu items

    set marker($frame,$id,radiusformat) physical
    DistMenuMarkerDialog $frame $mb $id CoordCircleCB radiusformat
    $mb add cascade -label Radius -menu $mb.radiusformat

    # Init circle variables

    EditCircleCB $frame $id

    # circle specific callbacks

    $frame marker $id callback edit EditCircleCB $frame
    $frame marker $id callback delete DeleteCircleDialog $frame

    # Dialog

    CommonMarkerDialog $frame $id
    wm protocol $w WM_DELETE_WINDOW "CloseCircleDialog $frame $id"

    # Radius

    label $w.ref.f.radiusTitle -text "Radius"
    entry $w.ref.f.radius -textvariable marker($frame,$id,radius) \
	-width 13 -relief sunken -bd 2
    label $w.ref.f.radiusformat -textvariable marker($frame,$id,radiusformat) \
	-relief groove -bd 2 -width 9 -padx 4

    grid $w.ref.f.radiusTitle $w.ref.f.radius x $w.ref.f.radiusformat \
	-padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyCircleDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseCircleDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyCircleDialog $frame $id"
}

proc PropMenuCircleDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
    $mb.properties add checkbutton -label "Fixed in Size" \
	-variable marker($frame,$id,fixed) \
	-command "PropertyMarkerDialog $frame $id fixed"
    $mb.properties add separator
    $mb.properties add radiobutton -label Include \
	-variable marker($frame,$id,include) -value 1 \
	-command "PropertyMarkerDialog $frame $id include"
    $mb.properties add radiobutton -label Exclude \
	-variable marker($frame,$id,include) -value 0 \
	-command "PropertyMarkerDialog $frame $id include"
    $mb.properties add separator
    $mb.properties add radiobutton -label Source \
	-variable marker($frame,$id,source) -value 1 \
	-command "PropertyMarkerDialog $frame $id source"
    $mb.properties add radiobutton -label Background \
	-variable marker($frame,$id,source) -value 0 \
	-command "PropertyMarkerDialog $frame $id source"
}

proc ApplyCircleDialog {frame id} {
    EditCircleDialog $frame $id
    ApplyMarkerDialog $frame $id
}

proc CloseCircleDialog {frame id} {
    global marker

    $frame marker $id delete callback edit EditCircleCB
    $frame marker $id delete callback delete DeleteCircleDialog
    DeleteMarkerCBs $frame $id

    DeleteCircleDialog $frame $id
}

proc DeleteCircleDialog {frame id} {
    global marker

    DeleteMarkerDialog $frame $id

    unset marker($frame,$id,radius)
    unset marker($frame,$id,radiusformat)
}

proc EditCircleDialog {frame id} {
    global marker

    $frame marker $id circle radius $marker($frame,$id,radius) \
	$marker($frame,$id,radiusformat)
}

proc EditCircleCB {frame id} {
    global marker

    set marker($frame,$id,radius) \
	[$frame get marker $id circle radius $marker($frame,$id,radiusformat)]
}

proc CoordCircleCB {frame id} {
    MoveMarkerCB $frame $id
    EditCircleCB $frame $id
}

proc CircleDefaultDialog {} {
    global circle
    global ed

    set w ".circle"

    set ed(ok) 0
    set ed(radius) $circle(radius)

    DialogCreate $w "Default Circle" -borderwidth 2
    frame $w.ed  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.title -text "Radius"
    entry $w.ed.radius -textvariable ed(radius) -width 10 -relief sunken -bd 2
    label $w.ed.unit -text "image" -relief groove -width 8
    
    grid $w.ed.title $w.ed.radius $w.ed.unit -padx 4 -sticky w

    button $w.buttons.ok -text "OK" -default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.radius select range 0 end
    DialogWait $w ed(ok) $w.ed.radius
    DialogDismiss $w

    if {$ed(ok)} {
	set circle(radius) $ed(radius)
    }

    unset ed
}
