package provide DS9 1.0

proc ProcessHelp {} {
    global argc
    global argv

    if {$argc > 0 && ([lindex $argv 0] == "-?")} {
	DumpHelp
	exit
    }
}

proc ProcessColorCommandLine {} {
    global argc
    global argv
    global colorbar
    global ds9

    set i 0
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -private {set colorbar(private) true}
	}
	incr i
    }
}

proc BadVisualError {} {
    global ds9

    tk_messageBox -parent $ds9(top) -icon info -type ok -message \
	"Sorry, DS9 requires a Pseudocolor8, Truecolor8, Truecolor16, Truecolor24 visual be available."
    exit
}

proc ProcessCommandLine {} {
    global argc
    global argv

    global ds9
    global about
    global current
    global contour
    global colorbar
    global debug
    global wcs
    global info
    global pan
    global mag
    global grid
    global graph
    global graphHorz
    global graphVert
    global bin
    global scale
    global iis
    global marker
    global minmax
    global tcl_platform
    global http

    set frameCount 0
    set load 0
    set file(mode) file
    set i 0

    # Note: help is processed previously
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -private {}
	    
	    -about {
		if {$tcl_platform(platform)!= "windows"} {
		    puts "$about(message)"
		}
	    }
	    -align {set wcs(align) 1; AlignWCSFrame}
	    -analysis {
		incr i
		set item [lindex $argv $i]
		ProcessAnalysisFile $item
		FileLast analysisfbox $item
		LayoutFrames
	    }
	    -array {set file(mode) array}
	    -blink {set ds9(displaymode) blink;	DisplayMode}
	    -bin {incr i; ProcessBinCmd argv i}
	    -cmap {incr i; ProcessCmapCmd argv i}
	    -colorbar {set colorbar(view) 1; UpdateView}
	    -nocolorbar {set colorbar(view) 0; UpdateView}
	    -contour {incr i; ProcessContourCmd argv i}
	    -nocontour {set contour(view) 0; ContourUpdate}
	    -coords {
		set item [lindex $argv [expr $i+1]]
		set value [FromYesNo [lindex $argv [expr $i+2]]]
		switch -- $item {
		    wcs {set info(wcs) $value}
		    detector {set info(detector) $value}
		    amplifier {set info(amplifier) $value}
		    physical {set info(physical) $value}
		    image {set info(image) $value}
		    default {}
		}
		UpdateInfoPanel

		incr i 2
	    }
	    -debug {
		DebugMenu
		incr i
		switch -- [lindex $argv $i] {
		    events {set debug(tcl,events) 1}
		    update {set debug(tcl,update) 1}
		    layout {set debug(tcl,layout) 1}
		    info {set debug(tcl,info) 1}
		    watch {set debug(tcl,watch) 1}

		    ast {
			set debug(saotk,ast) 1
			Debug ast debug(saotk,ast)
		    }
		    mosaic {
			set debug(saotk,mosaic) 1
			Debug mosaic debug(saotk,mosaic)
		    }
		    parser {
			set debug(saotk,parser) 1
			Debug parser debug(saotk,parser)
		    }
		    perf {
			set debug(saotk,perf) 1
			Debug perf debug(saotk,perf)
		    }
		    wcs {
			set debug(saotk,wcs) 1
			Debug wcs debug(saotk,wcs)
		    }
		    gz {
			set debug(saotk,gz) 1
			Debug gz debug(saotk,gz)
		    }
		    iis {
			set debug(iis) 1
			IISDebug
		    }
		    default {incr i -1}
		}
	    }
	    -dss {incr i; ProcessDSSCmd argv i 1 frameCount}
	    -exit {QuitDS9}
	    -quit {QuitDS9}
	    -grid {incr i; ProcessGridCmd argv i}
	    -nogrid {set grid(view) 0; GridUpdate}
	    -fifo {
		incr i
		set fifo [lindex $argv $i]
		if {$fifo!="none"} {
		    set iis(ififo) ${fifo}i
		    set iis(ofifo) ${fifo}o
		} else {
		    set iis(ififo) none
		    set iis(ofifo) none
		}
	    }
	    -fifo_only {set iis(port) 0; set iis(unix) none}
	    -file {set file(mode) file}
	    -histequ {set scale(type) histequ; ChangeScale}
	    -horzgraph {set graphHorz(view) 1; UpdateView}
	    -nohorzgraph {set graphHorz(view) 0; UpdateView}
	    -info {set info(view) 1; UpdateView}
	    -noinfo {set info(view) 0; UpdateView}
	    -iconify {wm iconify $ds9(top)}
	    -invert {set colorbar(invert) 1; InvertColorbar}
	    -linear {set scale(type) linear; ChangeScale}
	    -log {set scale(type) log; ChangeScale}
	    -lower {lower $ds9(top)}
	    -magnifier {set mag(view) 1; UpdateView}
	    -nomagnifier {set mag(view) 0; UpdateView}
	    -minmax {incr i; ProcessMinMaxCmd argv i}
	    -minmaxmode {incr i; set minmax(mode) [lindex $argv $i]}
	    -minmaxsample {incr i; set minmax(sample) [lindex $argv $i]}
	    -mode {incr i; set ds9(mode) [lindex $argv $i]; ChangeMode}
	    -mosaic {set file(mode) mosaic}
	    -mosaicimage {set file(mode) mosaicimage}
	    -nameserver {incr i; ProcessNSVRCmd argv i}
	    -orient {incr i; ProcessOrientCmd argv i}
	    -pagesetup {incr i; ProcessPageSetup argv i}
	    -pan {incr i; ProcessPanCmd argv i}
	    -panner {set pan(view) 1; UpdateView}
	    -nopanner {set pan(view) 0;	UpdateView}
	    -pixeltable {PixelTableDisplay}
	    -nopixeltable {PixelTableDestroy}
	    -port {incr i; set iis(port) [lindex $argv $i]}
	    -inet_only -
	    -port_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(unix) none
	    }
	    -print {incr i; ProcessPrintCmd argv i}
	    -raise {raise $ds9(top)}
	    -region -
	    -regionfile {
		RealizeDS9

		incr i
		LoadMarker [lindex $argv $i]
		FileLast markerfbox [lindex $argv $i]
	    }
	    -rotate {incr i; ProcessRotateCmd argv i}
	    -saveas {incr i; ProcessSaveAsCmd argv i}
	    -scale -
	    -ztrans {incr i; ProcessScaleCmd argv i}
	    -scalelims -
	    -scalelimits {
		incr i
		set scale(usermin) [lindex $argv $i]
		incr i
		set scale(usermax) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -scalemode {
		incr i
		set scale(mode) [string tolower [lindex $argv $i]]
		ChangeScaleMode
	    }
	    -scalescope {
		incr i
		set scale(scope) [string tolower [lindex $argv $i]]
		ChangeScaleScope
	    }
	    -shm {incr i; ProcessShmCmd argv i 1 frameCount}
	    -single {set ds9(displaymode) single; DisplayMode}
	    -squared {set scale(type) squared; ChangeScale}
	    -sqrt {set scale(type) sqrt; ChangeScale}
	    -source {
		incr i
		uplevel #0 "source \{[lindex $argv $i]\}"
	    }
	    -tile {incr i; ProcessTileCmd argv i}
	    -tilemode {
		incr i
		set tile(mode) [lindex $argv $i]
		DisplayMode
	    }
	    -title {
		incr i
		set ds9(title) [lindex $argv $i]
		set t "SAOImage $ds9(title)"
		wm title $ds9(top) "$t"
		wm iconname $ds9(top) "$t"
	    }
	    -url {set file(mode) url}
	    -unix {incr i; set iis(unix) [lindex $argv $i]}
	    -unix_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(port) 0
	    }
	    -view {incr i; ProcessViewCmd argv i}
	    -version {
		if {$tcl_platform(platform)!= "windows"} {
		    puts "$ds9(version)"
		}
	    }
	    -vertgraph {set graphVert(view) 1; UpdateView}
	    -novertgraph {set graphVert(view) 0; UpdateView}
	    -wcs {incr i; set wcs(system) [lindex $argv $i]}
	    -wcsformat {incr i; set wcs(format) [lindex $argv $i]}
	    -z1 {
		incr i
		set scale(usermin) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -z2 {
		incr i
		set scale(usermax) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -zmax {set scale(mode) zmax; ChangeScaleMode}
	    -zscale {set scale(mode) zscale; ChangeScaleMode}
	    -zoom {incr i; ProcessZoomCmd argv i}

	    default {
		

		if {([string length $item] == 1) || \
		        ([string range $item 0 1] == {-[}) || \
			([string range $item 0 0] != "-")} {

		    if {$load == 0} {
		        StartLoad
		        incr load
		    }

		    switch $file(mode) {
			array {
			    MultiLoad frameCount
			    LoadArray $item
			    FileLast arrayfbox $item
			}
			file {
			    MultiLoad frameCount
			    LoadFits $item
			    FileLast fitsfbox $item
			}
			mosaic {
			    LoadMosaicFits $item
			    FileLast fitsfbox $item
			}
			mosaicimage {
			    MultiLoad frameCount
			    LoadMosaicImageFits $item
			    FileLast fitsfbox $item
			}
			url {
			    MultiLoad frameCount
			    LoadURL $item
			    http(last) $item
			}
		    }
		    GridUpdate
		    ContourUpdate

		} else {
		    if {$tcl_platform(platform)!= "windows"} {
			puts "Unknown command line option: $item"
			DumpHelp
			exit
		    }
		}
	    }
	}
	incr i
    }

    if {$load != 0} {
        FinishLoad
    }
}

