package provide DS9 1.0

# Open

proc OpenFits {} {
    StartLoad
    LoadFits [OpenFileDialog fitsfbox]
    FinishLoad
}

proc OpenMosaicFits {} {
    StartLoad
    LoadMosaicFits [OpenFileDialog fitsfbox]
    FinishLoad
}

proc OpenMosaicImageFits {} {
    StartLoad
    LoadMosaicImageFits [OpenFileDialog fitsfbox]
    FinishLoad
}

proc OpenArray {} {
    set fn [OpenFileDialog arrayfbox]

    # do we have an array spec tag'd on
    if {[lindex $fn 0] != {} && \
	    ![regexp -nocase {(.*)(\[.*\])} $fn foo base ext]} {
	set ext [ArrayDialog]
	if {$ext != {}} {
	    set fns $fn
	    set fn {}
	    foreach f $fns {
		lappend fn "$f$ext "
	    }
	} else {
	    return
	}
    }

    StartLoad
    LoadArray $fn
    FinishLoad
}

proc OpenURL {} {
    global http

    StartLoad
    LoadURL [URLDialog http(last)]
    FinishLoad
}

proc OpenVar {} {
    set fn [OpenFileDialog $ds9(top) fitsvarfbox]
    if {$fn != {}} {
	if {![catch {set ch [open "$fn"]}]} {
	    fconfigure $ch -translation binary
	    global vardata
	    set vardata [read -nonewline $ch]
	    close $ch

	    StartLoad
	    LoadVar vardata $fn {}
	    FinishLoad
	}
    }
}

# Load

proc LoadFits {filenames} {
    global current
    global ds9
    global loadParam

    set fc 0
    foreach f $filenames {
	MultiLoad fc

	set type mmap
	set loadParam(load,type) mmap
	set loadParam(file,type) fits
	set loadParam(mosaic,mode) {}
	set loadParam(mosaic,type) {}
	ConvertFile $f
	ProcessLoad

	# now autoload markers
	if {$ds9(automarker)} {
	    # now, load fits[REGION] if present
	    set id [string first "\[" $f]
	    if {$id > 0} {
		set base [string range $f 0 [expr $id-1]]
	    } else {
		set base $f
	    }

	    set reg "${base}\[REGION\]"
	    if {[$current(frame) fitsy has ext "\"$reg\""]} {
		RealizeDS9

		catch {$current(frame) marker load fits "\"$reg\""}
	    }
	}
    }
}

proc LoadMosaicFits {filenames} {
    global current
    global mosaic
    global loadParam

    foreach f $filenames {
	set loadParam(load,type) mmap
	set loadParam(file,type) fits
	set loadParam(mosaic,mode) mosaic
	set loadParam(mosaic,type) $mosaic(type)
	ConvertFile $f
	ProcessLoad
    }
}

proc LoadMosaicImageFits {filenames} {
    global loadParam
    global mosaic

    set fc 0
    foreach f $filenames {
	MultiLoad fc

	set loadParam(load,type) mmap
	set loadParam(file,type) fits
	set loadParam(mosaic,mode) {mosaic image}
	set loadParam(mosaic,type) $mosaic(type)
	ConvertFile $f
	ProcessLoad
    }
}

proc LoadArray {filenames} {
    global loadParam

    set fc 0
    foreach f $filenames {
	MultiLoad fc

	set loadParam(load,type) mmap
	set loadParam(file,type) array
	set loadParam(mosaic,mode) {}
	set loadParam(mosaic,type) {}
	ConvertFile $f
	ProcessLoad
    }
}

proc LoadVar {varname filename mode} {
    global mosaic
    global loadParam

    if {$filename != ""} {
	set loadParam(load,type) var
	set loadParam(file,type) fits
	set loadParam(file,name) "$filename"
	set loadParam(mosaic,mode) $mode
	if {$mode != ""} {
	    set loadParam(mosaic,type) $mosaic(type)
	} else {
	    set loadParam(mosaic,type) {}
	}
	set loadParam(var,name) $varname
	ProcessLoad
    }
}

proc LoadShared {filetype mode idtype id size fn} {
    global loadParam
    global mosaic

    set loadParam(load,type) shared
    set loadParam(file,type) $filetype
    set loadParam(file,name) $fn
    set loadParam(mosaic,mode) $mode
    if {$mode != ""} {
	set loadParam(mosaic,type) $mosaic(type)
    } else {
	set loadParam(mosaic,type) {}
    }
    set loadParam(shared,idtype) $idtype
    set loadParam(shared,id) $id
    set loadParam(shared,size) $size
    ProcessLoad
}

proc LoadURL {url} {
    global ds9


    if {$url != ""} {
	if {![regexp -nocase {^([A-Za-z]+)?(://)?([^/:]+)(:([0-9]+))?(/.*)?$} \
		  $url x proto dummy host y port srvurl]} {
	    Error "Unable to parse URL $url"
	    return
	}

	switch $proto {
	    http {LoadHTTP $url "$ds9(tmpdir)/[file tail $url]"}
	    ftp {LoadFTP $url "$ds9(tmpdir)/[file tail $url]"}
	    file {LoadFits "$srvurl"}
	    default {LoadFits $url}
	}
    }
}

proc LoadFTP {url fn} {
    global loadParam

    regexp -nocase {^([A-Za-z]+)?(://)?([^/:]+)(:([0-9]+))?(/.*)?$} \
	$url x proto dummy host y port srvurl

    if [FTP::Open $host "ftp" "ds9@"] {
	FTP::Type binary
	if [FTP::Get $srvurl $fn] {
	    # alloc it because we are going to delete it after load
	    set loadParam(load,type) {alloc gz}
	    set loadParam(file,type) fits
	    set loadParam(mosaic,mode) {}
	    set loadParam(mosaic,type) {}
	    ConvertFile $fn
	    ProcessLoad
	}

	FTP::Close

	if [file exists $fn] {
	    catch {file delete -force $fn}
	}
    }
}

proc LoadHTTP {url fn} {
    global http
    global loadParam

    set ch [open $fn w]

    set cmd "http::geturl $url -channel $ch"
    set token [eval $cmd]

    HTTPLog $token

    # alloc it because we are going to delete it after load
    set loadParam(load,type) {alloc gz}
    set loadParam(file,type) fits
    set loadParam(mosaic,mode) {}
    set loadParam(mosaic,type) {}
    ConvertFile $fn
    ProcessLoad

    catch {close $ch}

    if {[file exists $fn]} {
	catch {file delete -force $fn}
    }
}

proc DisplayLog {item} {
    SimpleTextDialog ftplog "Mesage Log" 80 40 append bottom $item
}

# Save

proc SaveFits {} {
    global current
    global ed

    SaveFitsFormatDialog
    if {$ed(ok)} {
	set fn [SaveFileDialog fitsavfbox]
	if {$fn != {}} {
	    $current(frame) save fits $ed(format) file "\{$fn\}"
	}
    }
    unset ed
}

proc SaveFitsFormatDialog {} {
    global ed

    set w ".savfits"

    set ed(ok) 0
    set ed(format) image

    DialogCreate $w "Save Fits" -borderwidth 2
    frame $w.format  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.format $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.format.title -text "Save Fits:"
    radiobutton $w.format.image -text "Fits Image data" \
	-variable ed(format) \
	-value image
    radiobutton $w.format.resample \
	-text "Resample data with current zoom/pan/rotation" \
	-variable ed(format) \
	-value resample
    grid $w.format.title -padx 4 -sticky w
    grid $w.format.image -padx 4 -sticky w
    grid $w.format.resample -padx 4 -sticky w

    button $w.buttons.ok -text "OK" -default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok) $w.format.image
    DialogDismiss $w
}

# Support

proc MultiLoad {fc} {
    global ds9
    upvar $fc frameCount

    # create a new frame
    if {$frameCount != 0} {
	CreateFrame  
    }

    # go into tile mode if more than one
    if {$frameCount >= 1 && $ds9(displaymode) != "tile"} {
	set ds9(displaymode) tile
	DisplayMode
    }
    incr frameCount
}

proc BreakUp {str} {
    set r ""
    set l [string length $str]
    for {set i 0} {$i < $l} {incr i} {
	set c [string index $str $i]
	append r $c
	if {$c=="\}"} {
	    append r "\n"
	}
    }
    return $r
}

proc URLDialog {prevURL} {
    upvar $prevURL result

    if {[EntryDialog "URL" "Enter World Wide Web Location (URL):" 80 result]} {
	return $result
    } else {
	return ""
    }
}

# Header

proc DisplayHeaderMenu {} {
    global current

    set prim {}
    set cnt [$current(frame) get fits count]

    if {$cnt > 0} {
	global slb

	set slb(count) 0
	set slb(result) 0

	for {set i 1} {$i <= $cnt} {incr i} {
	    if {[$current(frame) has fits ext $i]} {
		set fn [$current(frame) get fits file name number $i]
		set b [string first {[} $fn]
		if {$b > 0} {
		    set fn [string range $fn 0 [expr $b-1]]
		}
		       
		if {$prim != $fn} {
		    set prim $fn

		    incr slb(count)
		    set slb($slb(count),item) $prim
		    set slb($slb(count),value) "-$i"
		}
	    }

	    incr slb(count)
	    set slb($slb(count),item) \
		[$current(frame) get fits file name number $i]
	    set slb($slb(count),value) $i
	}

	if {$slb(count) == 1} {
	    set slb(result) 1
	} elseif {$slb(count) > 1} {
	    SimpleListBox ".headerSelect" "Select Header"
	}

	if {$slb(result) != 0} {
	    DisplayHeader $slb(result)
	}

	unset slb
    }
}

proc DisplayHeader {which} {
    global current
    global st

    set tt "hd[string range $current(frame) end end]"

    SimpleTextDialog $tt $current(frame) 80 40 insert top \
	[$current(frame) get fits header $which]

    # create a special text tag for keywords

    $st($tt,text) tag configure keyword -font {courier} -foreground blue
    
    # color tag keywords

    set stop [$st($tt,text) index end]
    for {set i 1.0} {$i<$stop} {set i [expr $i+1]} {
	$st($tt,text) tag add keyword $i "$i +8 chars"
    }
}

proc UpdateDisplayHeader {which} {
    global st

    set tt "hd[string range $which end end]"

    if {[info exists st($tt,top)] && [winfo exist $st($tt,top)]} {
	DisplayHeaderMenu
    }
}

proc DestroyHeader {which} {
    global st

    set tt "hd[string range $which end end]"

    if {[info exists st($tt,top)] && [winfo exist $st($tt,top)]} {
	SimpleTextDestroy $tt
    }
}

proc UpdateFitsMenu {} {
    global current
    global buttons
    global crosshair
    global ds9
 
    global debug
    if {$debug(tcl,update)} {
	puts "UpdateFitsMenu"
    }

    UpdateCoordMenu2 $ds9(mb).frame.match 0 0
    # not implemented yet
    $ds9(mb).frame.match entryconfig "Detector" -state disabled
    $ds9(mb).frame.match entryconfig "Amplifier" -state disabled
    $ds9(mb).frame.match entryconfig "Physical" -state disabled

    UpdateCoordMenu2 $ds9(mb).frame.crosshair 0 0
    AdjustCoordSystem crosshair(lock)

    if {$current(frame) != ""} {
	$ds9(mb).file entryconfig "Open..." -state normal
	$ds9(mb).file entryconfig "Open Other" -state normal
	$ds9(mb).file entryconfig "Save as" -state disabled
	$ds9(mb).file entryconfig "Display Fits Header" -state disabled
	$ds9(mb).file entryconfig "Print..." -state normal
	$ds9(mb).frame entryconfig "Match Frames" -state normal
	$ds9(mb).frame entryconfig "Lock Crosshairs" -state normal
	$ds9(mb).frame entryconfig "Data Cube..." -state normal
	$ds9(mb).frame entryconfig "Clear Frame" -state disabled

	$buttons(name).file.open configure -state normal
	$buttons(name).file.save configure -state disabled
	$buttons(name).file.header configure -state disabled
	$buttons(name).file.print configure -state normal
	$buttons(name).frame.clear configure -state disabled

	if {[$current(frame) has fits]} {
	    $ds9(mb).file entryconfig "Save as" -state normal

	    global tcl_platform
	    global env
	    if {$tcl_platform(platform) == "windows"} {
		if {[info exists env(TERM)]} {
		    if {$env(TERM) != "cygwin"} {
			$ds9(mb).file.save entryconfig "Image..." \
			    -state disabled
		    }
		} else {
		    $ds9(mb).file.save entryconfig "Image..." -state disabled
		}
	    }

	    $ds9(mb).file entryconfig "Display Fits Header" -state normal
	    $ds9(mb).frame entryconfig "Clear Frame" -state normal

	    $buttons(name).file.save configure -state normal
	    $buttons(name).file.header configure -state normal
	    $buttons(name).frame.clear configure -state normal
	}
    } else {
	$ds9(mb).file entryconfig "Open..." -state disabled
	$ds9(mb).file entryconfig "Open Other" -state disabled
	$ds9(mb).file entryconfig "Save as" -state disabled
	$ds9(mb).file entryconfig "Display Fits Header" -state disabled
	$ds9(mb).file entryconfig "Print..." -state disabled
	$ds9(mb).frame entryconfig "Clear Frame" -state disabled
	$ds9(mb).frame entryconfig "Match Frames" -state disabled
	$ds9(mb).frame entryconfig "Lock Crosshairs" -state disabled
	$ds9(mb).frame entryconfig "Data Cube..." -state disabled

	$buttons(name).file.open configure -state disabled
	$buttons(name).file.save configure -state disabled
	$buttons(name).file.header configure -state disabled
	$buttons(name).file.print configure -state disabled
	$buttons(name).frame.clear configure -state disabled
    }
}

proc ProcessLoad {} {
    global current
    global loadParam

    switch $loadParam(load,type) {
	mmap {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(mosaic,mode) \
			   $loadParam(load,type) \
			   \"$loadParam(file,name)\" \
			   $loadParam(mosaic,type)}] {
		Error "Unable to load $loadParam(mosaic,mode) $loadParam(file,type) image $loadParam(file,name)"
	    }
	}
	alloc {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(mosaic,mode) \
			   $loadParam(load,type) \
			   \"$loadParam(file,name)\" \
			   $loadParam(mosaic,type)}] {
		Error "Unable to load $loadParam(mosaic,mode) $loadParam(file,type) image $loadParam(file,name)"
	    }
	}
	{alloc gz} {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(mosaic,mode) \
			   $loadParam(load,type) \
			   \"$loadParam(file,name)\" \
			   $loadParam(mosaic,type)}] {
		Error "Unable to load $loadParam(mosaic,mode) $loadParam(file,type) image $loadParam(file,name)"
	    }
	}
	shared {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(mosaic,mode) \
			   $loadParam(load,type) \
			   $loadParam(shared,idtype) \
			   $loadParam(shared,id) \
			   $loadParam(shared,size) \
			   \"$loadParam(file,name)\" \
			   $loadParam(mosaic,type)}] {
		Error "Unable to load $loadParam(mosaic,mode) $loadParam(file,type) image $loadParam(file,name)"
	    }
	}
	channel {
	    fconfigure $loadParam(channel,name) -translation binary
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(mosaic,mode) \
			   $loadParam(load,type) \
			   $loadParam(channel,name) \
			   \"$loadParam(file,name)\" \
			   $loadParam(mosaic,type)}] {
		Error "Unable to load $loadParam(mosaic,mode) $loadParam(file,type) image $loadParam(file,name)"
	    }
	}
	var {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(mosaic,mode) \
			   $loadParam(load,type) \
			   $loadParam(var,name) \
			   \"$loadParam(file,name)\" \
			   $loadParam(mosaic,type)}] {
		Error "Unable to load $loadParam(mosaic,mode) $loadParam(file,type) image $loadParam(file,name)"
	    }
	}
    }

    unset loadParam
}

proc StartLoad {} {
    SetWatchCursor
}

proc FinishLoad {} {
    global loadParam
    global current

    GridUpdate
    ContourUpdate
    UpdateInfoBoxFileName $current(frame)
    # do after updategrid and updatecontour
    UpdateDS9
    UnsetWatchCursor

    UpdateDisplayHeader $current(frame)
}

proc ConvertFile {filename} {
    global ds9
    global loadParam

    # default

    set loadParam(file,name) $filename

    foreach t $ds9(fileProcs) {
	if [$t $filename] {
	    return
	}
    }
}

# File Types

proc Stdin {filename} {
    global loadParam

    if {[string index $filename 0] == "-"} {
	set loadParam(load,type) {alloc gz}
	set loadParam(file,name) "stdin[string range $filename 1 end]"
	return 1
    } elseif {[string range $filename 0 4] == "stdin" || 
	      [string range $filename 0 4] == "STDIN"} {
	set loadParam(load,type) {alloc gz}
	set loadParam(file,name) "stdin[string range $filename 5 end]"
	return 1
    }

    return 0
}

proc CompressFits {filename} {
    global loadParam

    if { [regexp {(.*)\.Z($|\[)} $filename matched root] } {
	if [catch {set ch [open "| uncompress < $root.Z " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(file,name) "$filename"
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc PackFits {filename} {
    global loadParam

    if { [regexp {(.*)\.z($|\[)} $filename matched root] } {
	if [catch {set ch [open "| pcat $root.z " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(file,name) "$filename"
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc GzipFits {filename} {
    global loadParam

    if { [regexp {(.*)\.gz($|\[)} $filename matched root] } {
#	if [catch {set ch [open "| gunzip < $root.gz " r]}] {
#	    return 0
#	}
#	set loadParam(load,type) channel
#	set loadParam(channel,name) $ch

	set loadParam(load,type) {alloc gz}
	set loadParam(file,name) "$filename"
	return 1
    }

    return 0
}

proc FTZFits {filename} {
    global loadParam

    if { [regexp -nocase {(.*)\.(ftz)($|\[)} $filename matched root ext] } {
#	if [catch {set ch [open "| gunzip < $root.$ext " r]}] {
#	    return 0
#	}
#	set loadParam(load,type) channel
#	set loadParam(channel,name) $ch

	set loadParam(load,type) {alloc gz}
	set loadParam(file,name) "$filename"
	return 1
    }

    return 0
}

proc ExternalFits {filename} {
    global externalType
    global externalCmd
    global loadParam

    foreach a [array names externalType] {
	if { [regexp "(.*)\.$externalType($a)(.*)" $filename matched] } {
	    regsub -all {\$filename} "$externalCmd($a)" "$matched" result
	    set cmd "| $result"
	    if [catch {set ch [open "$cmd" r]}] {
		return 0
	    }
	    set loadParam(load,type) channel
	    set loadParam(file,name) "$filename"
	    set loadParam(channel,name) $ch
	    return 1
	}
    }
    return 0
}

proc ArrayDialog {} {
    global env
    global ed

    set w ".arr"
    set length 300

    set ed(ok) 0
    set ed(x) 512
    set ed(y) 512
    set ed(bitpix) -32
    set ed(skip) 0
    set ed(arch) bigendian

    if {[info exists env(DS9_ARRAY)]} {
	if {[regexp {.*(dims.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	    set ed(y) $item
	}
	if {[regexp {.*(dim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	    set ed(y) $item
	}
	if {[regexp {.*(xdim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	}
	if {[regexp {.*(ydim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(y) $item
	}
	if {[regexp {.*(bitpix.?=)(-?[0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(bitpix) $item
	}
	if {[regexp {.*(skip.?=)(-?[0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(skip) $item
	}
	if {[regexp {.*arch.?=bigendian} $env(DS9_ARRAY) foo item]} {
	    set ed(arch) $item
	}
	if {[regexp {.*arch.?=littleendian} $env(DS9_ARRAY) foo item]} {
	    set ed(arch) $item
	}
    }

    DialogCreate $w "Load Array" -borderwidth 2
    frame $w.arr
    frame $w.arr.dim -relief groove -borderwidth 2
    frame $w.arr.bitpix -relief groove -borderwidth 2
    frame $w.arr.skip -relief groove -borderwidth 2
    frame $w.arr.arch -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.arr.dim $w.arr.bitpix $w.arr.skip $w.arr.arch -side top \
	-ipadx 4 -ipady 4 -fill x -expand true
    pack $w.arr -fill x -expand true
    pack $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    label $w.arr.dim.title -text "Dimension"
    label $w.arr.dim.dummy -width 4
    entry $w.arr.dim.x -textvariable ed(x) -width 6
    entry $w.arr.dim.y -textvariable ed(y) -width 6
    grid $w.arr.dim.title $w.arr.dim.x $w.arr.dim.y \
	$w.arr.dim.dummy -padx 4 -sticky w

    label $w.arr.bitpix.title -text "Pixel Size"
    label $w.arr.bitpix.dummy -width 9
    radiobutton $w.arr.bitpix.char -text Char -variable ed(bitpix) -value 8
    radiobutton $w.arr.bitpix.short -text Short -variable ed(bitpix) -value 16
    radiobutton $w.arr.bitpix.ushort -text UShort -variable ed(bitpix) \
	-value -16
    radiobutton $w.arr.bitpix.long -text Long -variable ed(bitpix) -value 32
    radiobutton $w.arr.bitpix.float -text Float -variable ed(bitpix) -value -32
    radiobutton $w.arr.bitpix.double -text Double -variable ed(bitpix) \
	-value -64
    grid $w.arr.bitpix.title $w.arr.bitpix.char \
	$w.arr.bitpix.dummy -padx 4 -sticky w
    grid x $w.arr.bitpix.short -padx 4 -sticky w
    grid x $w.arr.bitpix.ushort -padx 4 -sticky w
    grid x $w.arr.bitpix.long -padx 4 -sticky w
    grid x $w.arr.bitpix.float -padx 4 -sticky w
    grid x $w.arr.bitpix.double -padx 4 -sticky w

    label $w.arr.skip.t1 -text "Skip First"
    label $w.arr.skip.t2 -text "bytes"
    label $w.arr.skip.dummy -width 4
    entry $w.arr.skip.skip -textvariable ed(skip) -width 6
    grid $w.arr.skip.t1 $w.arr.skip.skip $w.arr.skip.t2 \
	$w.arr.skip.dummy -padx 4 -sticky w

    label $w.arr.arch.title -text "Architecture"
    label $w.arr.arch.dummy -width 0
    radiobutton $w.arr.arch.big -text "Big-Endian" \
	-variable ed(arch) -value bigendian
    radiobutton $w.arr.arch.little -text "Little-Endian" \
	-variable ed(arch) -value littleendian
    grid $w.arr.arch.title $w.arr.arch.big $w.arr.arch.dummy -padx 4 -sticky w
    grid x $w.arr.arch.little -padx 4 -sticky w

    button $w.buttons.ok -text "OK" -default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    set r {}
    if {$ed(ok)} {
	set r "\[xdim=$ed(x),ydim=$ed(y),bitpix=$ed(bitpix),skip=$ed(skip),arch=$ed(arch)\]"
    }

    # we want to destroy this window

    destroy $w 

    unset ed
    return $r
}

proc ProcessShmCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    StartLoad
    switch -- [lindex $var $i] {
	array {
	    if {$flag} {
		MultiLoad fileCount
	    }
	    LoadShared array {} \
		[lindex $var [expr $i+1]] \
		[lindex $var [expr $i+2]] \
		[lindex $var [expr $i+3]] \
		[lindex $var [expr $i+4]]
	    incr i 4
	}
	mosaic {
	    LoadShared fits mosaic \
		[lindex $var [expr $i+1]] \
		[lindex $var [expr $i+2]] \
		[lindex $var [expr $i+3]] \
		[lindex $var [expr $i+4]]
	    incr i 4
	}
	mosaicimage {
	    if {$flag} {
		MultiLoad fc
	    }
	    LoadShared fits {mosaic image} \
		[lindex $var [expr $i+1]] \
		[lindex $var [expr $i+2]] \
		[lindex $var [expr $i+3]] \
		[lindex $var [expr $i+4]]
	    incr i 4
	}
	key -
	shmid {
	    if {$flag} {
		MultiLoad fc
	    }
	    LoadShared fits {} \
		[lindex $var $i] \
		[lindex $var [expr $i+1]] \
		[lindex $var [expr $i+2]] \
		[lindex $var [expr $i+3]]
	    incr i 3
	}
	default {
	    if {$flag} {
		MultiLoad fc
	    }
	    LoadShared fits {} key \
		[lindex $var $i] \
		[lindex $var [expr $i+1]] \
		[lindex $var [expr $i+2]]
	    incr i 2
	}
    }
    FinishLoad
}
