package provide DS9 1.0

# Public Procedures

proc InitCanvas {} {
    global canvas

    # Bindings

    bind $canvas(name) <Configure> [list LayoutFrames]
    bind $canvas(name) <Tab> [list NextFrame]
    bind $canvas(name) <Shift-Tab> [list PrevFrame]
    bind $canvas(name) <Enter> [list focus $canvas(name)]
    bind $canvas(name) <Leave> [list focus {}]

    # backward compatible bindings

    bind $canvas(name) <Button-3> {Button3Canvas %x %y}
    bind $canvas(name) <B3-Motion> {Motion3Canvas %x %y}
    bind $canvas(name) <ButtonRelease-3> {Release3Canvas %x %y}

    # freeze

    bind $canvas(name) <f> {ToggleFreeze}
}

proc Button3Canvas {x y} {
    global ds9

    set ds9(b3) 1
    ButtonColormap $x $y
}

proc Motion3Canvas {x y} {
    MotionColormap $x $y
}

proc Release3Canvas {x y} {
    global ds9

    set ds9(b3) 0
    ReleaseColormap $x $y
}

proc FreezeCanvas {} {
    global ds9

    foreach f $ds9(active) {
	UnBindEventsFrame $f
    }
}

proc UnFreezeCanvas {} {
    global ds9
    global current

    switch $ds9(displaymode) {
	single {
	    BindEventsFrame $current(frame)
	}
	tile {
	    foreach f $ds9(active) {
		BindEventsFrame $f
	    }
	}
	blink {
	    BlinkEventsFrame $current(frame)
	}
    }
}

proc CreateFrame {} {
    CreateNamedFrame Frame
}

proc CreateNamedFrame {name} {
    global ds9

    # find the first open slot

    set num 1
    while {1} {
	if {[lsearch $ds9(frames) "$name$num"]==-1} {
	    CreateNameNumberFrame "$name$num"
	    return
	}
	incr num
    }
}

proc CreateGotoFrame {num} {
    global ds9
    global active

    set which "Frame$num"
    if {[lsearch $ds9(frames) $which]==-1} {
	CreateNameNumberFrame $which
    } else {
	if {$active($which)==0} {
	    set active($which) 1
	    UpdateActiveFrames
	}
	set ds9(next) $which
	GotoFrame
    }
}

proc CreateNameNumberFrame {which} {
    global ds9
    global active
    global current
    global canvas
    global pan
    global mag
    global bin
    global contour
    global scale
    global wcs
    global colorbar
    global minmax
    global mosaic

    set ds9(next) $which

    # update frame lists
    lappend ds9(frames) $ds9(next)
    set ds9(frames) [lsort -dictionary $ds9(frames)]
    lappend ds9(active) $ds9(next)
    set ds9(active) [lsort -dictionary $ds9(active)]
    set active($ds9(next)) 1

    # and update frame list menus
    UpdateFrameMenuItems

    # and create the frame
    $canvas(name) create frame$ds9(visual)$ds9(depth) \
	-x 0 -y 0 -anchor nw -command $ds9(next) -tag $ds9(next)

    $ds9(next) colormap [colorbar get colormap]
    $ds9(next) set name \"$ds9(next)\"
    $ds9(next) panner 'panner' $pan(size) $pan(size)
    $ds9(next) magnifier 'magnifier' $mag(size) $mag(size)
    $ds9(next) magnifier mode $mag(mode)

    $ds9(next) bin function $bin(function)
    $ds9(next) bin factor to $bin(factor)
    $ds9(next) bin buffer size $bin(buffersize)

    $ds9(next) zoom to $current(zoom)
    $ds9(next) rotate to $current(rotate)
    $ds9(next) orient $current(orient)
    $ds9(next) wcs align $wcs(align)

    $ds9(next) datasec $scale(datasec)

    $ds9(next) clip scope $scale(scope)
    $ds9(next) clip mode $scale(mode)
    $ds9(next) clip user $scale(usermin) $scale(usermax)

    $ds9(next) clip minmax mode $minmax(mode)
    $ds9(next) clip minmax sample $minmax(sample)

    $ds9(next) colorscale $scale(type)
    $ds9(next) colormap area $colorbar(area) $colorbar(mode)

    $ds9(next) mosaic fast $mosaic(fast)

    if {$ds9(mode) == "crosshair"} {
	$ds9(next) crosshair on
    }

    set current(frame) $ds9(next)

    SetWatchCursor
    LayoutFrames
    UnsetWatchCursor
}

proc DeleteFrameMenu {} {
    global buttons

    if {[tk_messageBox -type okcancel -default cancel \
	     -message "Delete this Frame?" -icon question] == "ok"} {
	DeleteCurrentFrame
    }
}

proc DeleteCurrentFrame {} {
    global current

    if {$current(frame) != ""} {
	DeleteFrame $current(frame)
    }
}

proc DeleteAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	DeleteFrame $f
    }
}

proc DeleteFrame {which} {
    global ds9
    global active
    global header
    global current
    global canvas
    global contour

    # contour copy
    if {$contour(copy) == $which} {
	set contour(copy) {}
    }
    
    # kill the header dialog (if needed)
    DestroyHeader $which

    # delete any slave frames
    DeleteExamine $which

    # delete canvas widget
    $canvas(name) delete $which

    # delete it from the frame list
    set i [lsearch $ds9(frames) $which]
    set ds9(frames) [lreplace $ds9(frames) $i $i]
    unset active($which)

    # delete menu entries
    # NOTE: we can't use 'menuBar.frame delete' since there is a bug
    UpdateFrameMenuItems

    # and update layout
    UpdateActiveFrames

    # clear infobox
    ClearInfoBox
}

proc UpdateCurrentFrame {} {
    global current

    UpdateFrame $current(frame)
}

proc UpdateAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	UpdateFrame $f
    }
}

proc UpdateFrame {which} {
    SetWatchCursor
    if {$which != ""} {
	$which update
    }
    UnsetWatchCursor
}

proc ChangeMosaicFast {} {
    global ds9
    global mosaic

    foreach f $ds9(frames) {
	$f mosaic fast $mosaic(fast)
    }
}

# Event Processing

proc BindEventsFrame {which} {
    global canvas
    global analysis

    $canvas(name) bind $which <Motion> [list MotionFrame $which %x %y]
    $canvas(name) bind $which <Shift-Motion> \
	[list ShiftMotionFrame $which %x %y]
    $canvas(name) bind $which <Control-Motion> \
	[list ControlMotionFrame $which %x %y]

    $canvas(name) bind $which <Enter> [list EnterFrame $which %x %y]
    $canvas(name) bind $which <Leave> [list LeaveFrame $which]

    $canvas(name) bind $which <Button-1> [list Button1Frame $which %x %y]
    $canvas(name) bind $which <Shift-Button-1> \
	[list ShiftButton1Frame $which %x %y]
    $canvas(name) bind $which <Control-Button-1> \
	[list ControlButton1Frame $which %x %y]
    $canvas(name) bind $which <Control-Shift-Button-1> \
	[list ControlShiftButton1Frame $which %x %y]
    $canvas(name) bind $which <B1-Motion> [list Motion1Frame $which %x %y]
    $canvas(name) bind $which <ButtonRelease-1> \
	[list Release1Frame $which %x %y]

    $canvas(name) bind $which <Double-1> [list Double1Frame $which %x %y]
    $canvas(name) bind $which <Double-ButtonRelease-1> \
	[list DoubleRelease1Frame $which %x %y]

    $canvas(name) bind $which <Button-2> [list Button2Frame $which %x %y]
    $canvas(name) bind $which <Shift-Button-2> \
	[list ShiftButton2Frame $which %x %y]
    $canvas(name) bind $which <B2-Motion> [list Motion2Frame $which %x %y]
    $canvas(name) bind $which <ButtonRelease-2> \
	[list Release2Frame $which %x %y]

    $canvas(name) bind $which <Delete> [list DeleteKeyFrame $which %x %y]
    $canvas(name) bind $which <BackSpace> [list DeleteKeyFrame $which %x %y]

    $canvas(name) bind $which <Up> [list ArrowKeyFrame $which 0 -1]
    $canvas(name) bind $which <Down> [list ArrowKeyFrame $which 0 1]
    $canvas(name) bind $which <Left> [list ArrowKeyFrame $which -1 0]
    $canvas(name) bind $which <Right> [list ArrowKeyFrame $which 1 0]

    $canvas(name) bind $which <c> [list DisplayCoordDialog $which %x %y]
    $canvas(name) bind $which <i> [list $which marker property include 1 %x %y]
    $canvas(name) bind $which <e> [list $which marker property include 0 %x %y]
    $canvas(name) bind $which <s> [list $which marker property source 1 %x %y]
    $canvas(name) bind $which <b> [list $which marker property source 0 %x %y]

    $canvas(name) bind $which <plus> [list CubeNext]
    $canvas(name) bind $which <minus> [list CubePrev]

    # analysis

    for {set i 0} {$i<$analysis(bind,count)} {incr i} {
	$canvas(name) bind $which "$analysis(bind,$i,item)" \
	    "AnalysisTask $i bind $which %x %y"
    }
}

proc BlinkEventsFrame {which} {
    global canvas

    $canvas(name) bind $which <Motion> [list BlinkMotionFrame $which %x %y]
    $canvas(name) bind $which <Enter> [list BlinkEnterFrame $which %x %y]
    $canvas(name) bind $which <Leave> [list BlinkLeaveFrame $which]
}

proc UnBindEventsFrame {which} {
    global canvas
    global analysis

    $canvas(name) bind $which <Motion> {}
    $canvas(name) bind $which <Shift-Motion> {}
    $canvas(name) bind $which <Control-Motion> {}

    $canvas(name) bind $which <Enter> {}
    $canvas(name) bind $which <Leave> {}

    $canvas(name) bind $which <Button-1> {}
    $canvas(name) bind $which <Shift-Button-1> {}
    $canvas(name) bind $which <Control-Button-1> {}
    $canvas(name) bind $which <Control-Shift-Button-1> {}
    $canvas(name) bind $which <B1-Motion> {}
    $canvas(name) bind $which <ButtonRelease-1> {}

    $canvas(name) bind $which <Double-1> {}
    $canvas(name) bind $which <Double-ButtonRelease-1> {}

    $canvas(name) bind $which <Button-2> {}
    $canvas(name) bind $which <Shift-Button-2> {}
    $canvas(name) bind $which <B2-Motion> {}
    $canvas(name) bind $which <ButtonRelease-2> {}

    $canvas(name) bind $which <Delete> {}
    $canvas(name) bind $which <BackSpace> {}

    $canvas(name) bind $which <Up> {}
    $canvas(name) bind $which <Down> {}
    $canvas(name) bind $which <Left> {}
    $canvas(name) bind $which <Right> {}

    $canvas(name) bind $which <c> {}
    $canvas(name) bind $which <i> {}
    $canvas(name) bind $which <e> {}
    $canvas(name) bind $which <s> {}
    $canvas(name) bind $which <b> {}

    $canvas(name) bind $which <+> {}
    $canvas(name) bind $which <-> {}

    # analysis

    for {set i 0} {$i<$analysis(bind,count)} {incr i} {
	$canvas(name) bind $which "$analysis(bind,$i,item)" {}
    }
}

proc EnterFrame {which x y} {
    global ds9
    global canvas
    global info
    global mag

    global debug
    if {$debug(tcl,events)} {
	puts "EnterFrame"
    }

    # check to see if this event was generated while processing other events

    if {$ds9(b1) || $ds9(sb1) || $ds9(cb1) || $ds9(csb1) || $ds9(b2) || $ds9(b3)} {
	return
    }

    $canvas(name) focus $which

    # Display Filename and Coord info

    switch $ds9(mode) {
	crosshair {EnterFrameCrosshair $which}
	pointer {
	    UpdateInfoBox $which $x $y canvas
	    UpdatePixelTable $which $x $y canvas
	    UpdateMagnifier $which $x $y
	    UpdateGraph $which $x $y
	}
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate {}
    }


    # Display Frame Info

    set info(pFrame) $which
    set info(pZoom) [$which get zoom fixed]
    set info(pAngle) [$which get rotate fixed]

    # and Other stuff

    ShowGraphData

    if {$mag(view)} {
	$which magnifier on
    }

    UpdateEditMenu
}

proc LeaveFrame {which} {
    global ds9
    global canvas
    global info
    global mag

    global debug
    if {$debug(tcl,events)} {
	puts "LeaveFrame"
    }

    # check to see if this event was generated while processing other events

    if {$ds9(b1) || $ds9(sb1) || $ds9(cb1) || $ds9(csb1) || $ds9(b2) || $ds9(b3)} {
	return
    }

    $canvas(name) focus {}

    switch $ds9(mode) {
	crosshair {}
	pointer -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate {
	    ClearInfoBox
	    ClearGraphData
	    ClearPixelTable
	}
    }

    switch $ds9(displaymode) {
	single {}
	tile {
	    set info(pFrame) Frame
	    set info(pZoom) ""
	    set info(pAngle) ""
	}
	blink {}
    }

    $which magnifier off
    magnifier clear
}

proc MotionFrame {which x y} {
    DoMotion $which $x $y sizing fleur 1 MotionFrame
}

proc ShiftMotionFrame {which x y} {
    DoMotion $which $x $y exchange fleur 0 ShiftMotionFrame
}

proc ControlMotionFrame {which x y} {
    DoMotion $which $x $y sizing draped_box 0 ControlMotionFrame
}

proc DoMotion {which x y cursor1 cursor2 doPixelTable name} {
    global ds9
    global current
    global pan

    global debug
    if {$debug(tcl,events)} {
	puts "$name"
    }

    # if button 3 is down, ignore this event, we are doing something already

    if {$ds9(b3) || $ds9(b2)} {
	return
    }

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		CursorMarker $which $x $y $cursor1 $cursor2
	    }
	    UpdateInfoBox $which $x $y canvas
	    UpdateGraph $which $x $y
	    if {$doPixelTable} {
		UpdatePixelTable $which $x $y canvas
	    }
	}
	crosshair {}
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate
	{
	    UpdateInfoBox $which $x $y canvas
	    UpdateGraph $which $x $y
	}
    }

    UpdateMagnifier $which $x $y
}

proc Button1Frame {which x y} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,events)} {
	puts "Button1Frame"
    }

    # let others know that the mouse is down

    set ds9(b1) 1

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		ButtonMarker $which $x $y
	    } else {
		# we need this cause MotionMarker maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1

		set ds9(next) $which
		GotoFrame
	    }
	}
	crosshair {ButtonCrosshair $which $x $y}
	colorbar {}
	pan {ButtonPan $which $x $y}
	zoom {ButtonZoom $which $x $y}
	rotate {ButtonRotate $which $x $y}
	examine {ButtonExamine $which $x $y}
	illustrate {}
    }
}

proc ShiftButton1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "ShiftButton1Frame"
    }

    # let others know that the mouse is down

    set ds9(sb1) 1

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		ShiftMarker $which $x $y
	    }
	}
	crosshair {}
	colorbar {}
	pan {}
	zoom {ShiftZoom $which}
	rotate {}
	examine {}
	illustrate {}
    }
}

proc ControlButton1Frame {which x y} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,events)} {
	puts "ControlButton1Frame"
    }

    # let others know that the mouse is down

    set ds9(cb1) 1

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		ControlMarker $which $x $y
	    } else {
		# we need this cause MotionMarker maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1
	    }
	}
	crosshair {}
	colorbar {}
	pan {}
	zoom {}
	rotate {}
	examine {}
	illustrate {}
    }
}

proc ControlShiftButton1Frame {which x y} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,events)} {
	puts "ControlShiftButton1Frame"
    }

    # let others know that the mouse is down

    set ds9(csb1) 1

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		ControlShiftMarker $which $x $y
	    } else {
		# we need this cause MotionMarker maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1
	    }
	}
	crosshair {}
	colorbar {}
	pan {}
	zoom {}
	rotate {}
	examine {}
	illustrate {}
    }
}

proc Motion1Frame {which x y} {
    global ds9
    global current
    global mag

    global debug
    if {$debug(tcl,events)} {
	puts "Motion1Frame"
    }

    # abort if we are here by accident (such as a double click)

    if {($ds9(b1) == 0) && ($ds9(sb1) == 0) && \
	    ($ds9(cb1) == 0) && ($ds9(csb1) == 0)} {
	return
    }

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		MotionMarker $which $x $y
	    }
	    UpdateInfoBox $which $x $y canvas
	    UpdateGraph $which $x $y
	}
	crosshair {MotionCrosshair $which $x $y}
	colorbar {}
	pan {MotionPan $which $x $y}
	zoom {}
	rotate {MotionRotate $which $x $y}
	examine {}
	illustrate {}
    }

    UpdateMagnifier $which $x $y
}

proc Release1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "Release1Frame"
    }

    # abort if we are here by accident (such as a double click)

    if {($ds9(b1) == 0) && ($ds9(sb1) == 0) && \
	    ($ds9(cb1) == 0) && ($ds9(csb1) == 0)} {
	return
    }

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		ReleaseMarker $which $x $y
	    }
	}
	crosshair {ReleaseCrosshair $which $x $y}
	colorbar {}
	pan {ReleasePan $which $x $y}
	zoom {}
	rotate {ReleaseRotate $which $x $y}
	examine {}
	illustrate {}
    }

    # let others know that the mouse is up

    set ds9(b1) 0
    set ds9(sb1) 0
    set ds9(cb1) 0
    set ds9(csb1) 0

    UpdateEditMenu
}

proc Double1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "Double1Frame"
    }

    switch $ds9(mode) {
	pointer {
	    if {$which == $current(frame)} {
		DoubleMarker $which $x $y
	    }
	}
	crosshair {}
	colorbar {}
	pan {}
	zoom {}
	rotate {}
	examine {}
	illustrate {}
    }
}

proc DoubleRelease1Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "DoubleRelease1Frame"
    }

    switch $ds9(mode) {
	pointer {}
	crosshair {}
	colorbar {}
	pan {}
	zoom {}
	rotate {}
	examine {}
	illustrate {}
    }

    UpdateEditMenu
}

proc Button2Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "Button2Frame"
    }

    set ds9(b2) 1
    ButtonPan $which $x $y
}

proc ShiftButton2Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "ShiftButton2Frame"
    }

    set ds9(sb2) 1
}

proc Motion2Frame {which x y} {
    global debug
    if {$debug(tcl,events)} {
	puts "Motion2Frame"
    }

    MotionPan $which $x $y
}

proc Release2Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "Release2Frame"
    }

    ReleasePan $which $x $y

    # let others know that the mouse is up

    set ds9(b2) 0
    set ds9(sb2) 0
}

proc DeleteKeyFrame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "DeleteKeyFrame"
    }

    switch $ds9(mode) {
	pointer {DeleteMarker $which $x $y}
	crosshair {}
	colorbar {}
	pan {}
	zoom {}
	rotate {}
	examine {}
	illustrate {}
    }

    UpdateEditMenu
}

proc ArrowKeyFrame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "ArrowKeyFrame"
    }

    switch $ds9(mode) {
	pointer {
	    $which warp $x $y
	    $which marker move $x $y
	}
	pan {Pan [expr -$x] [expr -$y] canvas}
	zoom -
	rotate {}
	crosshair {ArrowKeyCrosshair $which $x $y}
	colorbar {}
	examine {}
	illustrate {}
    }

    UpdateEditMenu
}

proc BlinkEnterFrame {which x y} {
    global ds9
    global canvas
    global info
    global mag
    global blink

    global debug
    if {$debug(tcl,events)} {
	puts "BlinkEnterFrame"
    }

    $canvas(name) focus $which

    # Display Filename and Coord info

    UpdateInfoBox $which $x $y canvas
    UpdatePixelTable $which $x $y canvas
    UpdateMagnifier $which $x $y
    UpdateGraph $which $x $y

    # Display Frame Info

    set info(pFrame) $which
    set info(pZoom) [$which get zoom fixed]
    set info(pAngle) [$which get rotate fixed]

    # and if this is a true enterframe event

    if {!$blink(state)} {
	ShowGraphData

	if {$mag(view)} {
	    foreach f $ds9(frames) {
		$f magnifier on
	    }
	}
	set blink(state) 1
    }
}

proc BlinkLeaveFrame {which} {
    global ds9
    global canvas
    global info
    global mag
    global blink

    global debug
    if {$debug(tcl,events)} {
	puts "BlinkLeaveFrame"
    }

    $canvas(name) focus {}

    ClearInfoBox
    ClearPixelTable
    ClearGraphData

    set info(pFrame) Frame
    set info(pZoom) ""
    set info(pAngle) ""

    foreach f $ds9(frames) {
	$f magnifier off
    }

    magnifier clear

    # so enterframe knows we are really out of the frame
    set blink(state) 0
}

proc BlinkMotionFrame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "BlinkMotionFrame"
    }

    UpdateInfoBox $which $x $y canvas
    UpdatePixelTable $which $x $y canvas
    UpdateMagnifier $which $x $y
    UpdateGraph $which $x $y
}

# Other Public Routines

proc CopyFrame {} {
    global current

    if {$current(frame) != ""} {
	$current(frame) marker copy
    }
    UpdateEditMenu
}

proc CutFrame {} {
    global current

    if {$current(frame) != ""} {
	$current(frame) marker cut
    }
    UpdateEditMenu
}

proc PasteFrame {} {
    global current

    if {$current(frame) != ""} {
	$current(frame) marker paste
    }
    UpdateEditMenu
}

proc UndoFrame {} {
    global current

    if {$current(frame) != ""} {
	$current(frame) marker undo
    }
    UpdateEditMenu
}

proc FirstFrame {} {
    global ds9
    set ds9(next) [lindex $ds9(active) 0]
    GotoFrame
}

proc PrevFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(active) $current(frame)]
    if {$i>0} {
	set ds9(next) [lindex $ds9(active) [expr $i-1]]
    } else {
	set last [expr [llength $ds9(active)] -1]
	set ds9(next) [lindex $ds9(active) $last]
    }
    GotoFrame
}

proc NextFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(active) $current(frame)]
    set last [expr [llength $ds9(active)] -1]
    if {$i<$last} {
	set ds9(next) [lindex $ds9(active) [expr $i+1]]
    } else {
	set ds9(next) [lindex $ds9(active) 0]
    }
    GotoFrame
}

proc LastFrame {} {
    global ds9
    set ds9(next) [lindex $ds9(active) [expr [llength $ds9(active)]-1]]
    GotoFrame
}

proc UpdateActiveFrames {} {
    global ds9
    global active
    global current

    # reset active list

    set ds9(active) ""
    foreach f $ds9(frames) {
	if {$active($f)} {
	    lappend ds9(active) $f
	    $ds9(mb).frame entryconfig $f -state normal
	} else {
	    $ds9(mb).frame entryconfig $f -state disabled
	}

    }

    # New layout if needed

    if {[llength $ds9(active)] > 0} {
	if {[lsearch $ds9(active) $current(frame)] == -1} {
	    set current(frame) [lindex $ds9(active) 0]
	    set ds9(next) $current(frame)
	}
    }

    SetWatchCursor
    LayoutFrames
    UnsetWatchCursor
}

proc ActiveFrameAll {} {
    global ds9
    global active

    foreach f $ds9(frames) {
	set active($f) 1
    }
    UpdateActiveFrames
}

proc ActiveFrameNone {} {
    global ds9
    global active

    foreach f $ds9(frames) {
	set active($f) 0
    }
    UpdateActiveFrames
}

proc GotoFrame {} {
    global ds9
    global current
    global active

    if {$current(frame) != "" && $current(frame) != $ds9(next)} {
	$current(frame) marker unselect all
	$current(frame) highlite off
	$current(frame) panner off

	switch $ds9(displaymode) {
	    single {
		$current(frame) hide
		UnBindEventsFrame $current(frame)
	    }
	    tile {}
	    blink {
		$current(frame) hide
		UnBindEventsFrame $current(frame)
	    }
	}
    }

    if {$current(frame) != $ds9(next)} {
	set current(frame) $ds9(next)
	FrameToFront $current(frame)
    }
}

proc DisplayMode {} {
    global ds9
    global blink

    switch $ds9(displaymode) {
	single -
	tile {
	    # turn off blink if on
	    if {$blink(id)>0} {
		after cancel $blink(id)
		set blink(id) 0
		set blink(index) -1
	    }

	    SetWatchCursor
	    LayoutFrames
	    UnsetWatchCursor
	}
	blink {
	    # ignore if we are already blinking
	    if {$blink(id)==0} {
		SetWatchCursor
		LayoutFrames
		UnsetWatchCursor

		BlinkTimer

		# assume we are outside
		set blink(state) 0
	    }
	}
    }
}

proc BlinkTimer {} {
    global blink
    global ds9

    if {$ds9(active) != ""} {
	incr blink(index)
	if {$blink(index) >= [llength $ds9(active)]} {
	    set blink(index) 0
	}
	set ds9(next) [lindex $ds9(active) $blink(index)]
	GotoFrame
    }

    set blink(id) [after $blink(interval) BlinkTimer]
}

proc ResetCurrentFrame {} {
    global current

    ResetFrame $current(frame)
}

proc ResetAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	ResetFrame $f
    }
}

proc ResetFrame {which} {
    if {$which != ""} {
	SetWatchCursor
	$which reset

	ClearInfoBox
	ClearGraphData

	UnsetWatchCursor

	UpdatePanZoomMenu
	UpdatePanZoomDialog
	UpdateScaleMenu
	UpdateScaleDialog

	UpdateGraphYAxis
	UpdateGraphXAxis
    }
}

proc ClearCurrentFrame {} {
    global current

    ClearFrame $current(frame)
}

proc ClearAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	ClearFrame $f
    }
}

proc ClearFrame {which} {
    global current
    global header

    if {$which != ""} {

	# delete any slave frames
	DeleteExamine $which

	DestroyHeader $which

	SetWatchCursor
	$which clear
	UnsetWatchCursor

	ClearInfoBox
	UpdateDS9
	ClearGraphData
    }
}

proc MatchFramesImage {} {
    global ds9
    global current

    if {$current(frame) != ""} {
	SetWatchCursor

	set pan [$current(frame) get pan image scientific]

	foreach f $ds9(frames) {
	    if {$f != $current(frame)} {
		$f pan to image $pan

		$f zoom to $current(zoom)
		$f orient $current(orient)
		$f rotate to $current(rotate)
	    }
	}

	UnsetWatchCursor
    }
}

proc MatchFramesWCS {} {
    global ds9
    global current
    global wcs

    if {$current(frame) != ""} {
	SetWatchCursor

	set wcs(align) 1
	$current(frame) wcs align $wcs(align)

	set wcsZoom [$current(frame) get wcs zoom scientific]
	set wcsSystem [$current(frame) get wcs system]
	set wcsPan [$current(frame) get pan $wcsSystem scientific]

	foreach f $ds9(frames) {
	    if {$f != $current(frame)} {
		if  {[$f has wcs equatorial]} {
		    $f wcs align $wcs(align)
		    $f wcs zoom $wcsZoom

		    $f pan to $wcsSystem $wcsPan
		    $f zoom to $current(zoom)
		    $f orient $current(orient)
		    $f rotate to $current(rotate)
		}
	    }
	}

	UnsetWatchCursor
    }
}

# Private Procedures

proc FrameToFront {which} {
    global ds9
    global current
    global canvas
    global colorbar
    global pan
    global info
    global blink

    global debug
    if {$debug(tcl,layout)} {
	puts "FrameToFront"
    }

    colorbar set colormap [$which get colormap]
    set colorbar(map) [colorbar get name]
    set colorbar(invert) [colorbar get invert]

    $canvas(name) raise $which
    $which show
    switch $ds9(displaymode) {
	single {
	    if {!$ds9(freeze)} {
		BindEventsFrame $current(frame)
	    }
	}
	tile {
	    if {[llength $ds9(active)] > 1} {
		$which highlite on
	    }
	}
	blink {
	    if {!$ds9(freeze)} {
		BlinkEventsFrame $current(frame)
	    }
	}
    }

    if {$pan(view)} {
	$which panner on
    }

    UpdateDS9

    UpdateInfoBoxFileName $which
    set info(pFrame) $which
    set info(pZoom) [$which get zoom fixed]
    set info(pAngle) [$which get rotate fixed]
}

proc LayoutFrames {} {
    global ds9
    global current
    global canvas
    global tile
    global pan
    global mag

    global debug
    if {$debug(tcl,layout)} {
	puts "LayoutFrames"
    }

    set canvas(width) [expr [winfo width $canvas(name)] - 4]
    set canvas(height) [expr [winfo height $canvas(name)] - 4]

    # turn everything off

    foreach f $ds9(frames) {
	$f hide
	$f highlite off
	$f panner off
	$f magnifier off
	UnBindEventsFrame $f
    }

    set n [llength $ds9(active)]
    if {$n > 0} {
	switch $ds9(displaymode) {
	    single {TileOne}
	    tile {
		switch $tile(mode) {
		    row {TileRect 1 $n $tile(grid,gap)}
		    column {TileRect $n 1 $tile(grid,gap)}
		    grid {
			switch $tile(grid,mode) {
			    automatic {
				TileRect [expr int(sqrt($n-1))+1] \
				    [expr int(sqrt($n)+.5)] \
				    $tile(grid,gap)
			    }
			    manual {
				TileRect $tile(grid,row) \
				    $tile(grid,col) \
				    $tile(grid,gap)
			    }
			}
		    }
		}
	    }
	    blink {TileOne}
	}
    } else {
	set current(frame) ""
	set ds9(next) ""
	if {$pan(view)} {
	    panner clear
	}
	if {$mag(view)} {
	    magnifier clear
	}

	UpdateDS9
    }
}

# This procedure is called when we have only 1 frames to display
# Canvas Coordinate system is 1 to canvas(width), canvas(height)

proc TileOne {} {
    global ds9
    global current
    global canvas

    global debug
    if {$debug(tcl,layout)} {
	puts "TileOne"
    }

    set w $canvas(width)
    set h $canvas(height)

    set x [expr int($canvas(width)/2.) + 2]
    set y [expr int($canvas(height)/2.) + 2]

    foreach f $ds9(active) {
	$f configure -x $x -y $y -width $w -height $h -anchor center
    }
    # only show the current frame
    $current(frame) show
    FrameToFront $current(frame)
}

proc TileRect {numx numy gap} {
    global ds9
    global canvas

    global debug
    if {$debug(tcl,layout)} {
	puts "TileRect"
    }

    set w [expr $canvas(width) /$numx - $gap]
    set h [expr $canvas(height)/$numy - $gap]
    
    for {set i 0; set b 1} {$i < $numy} {incr i; incr b 2} {
	for {set j 0; set a 1} {$j < $numx} {incr j; incr a 2} {
	    set c [expr $i*$numx + $j]
	    set x($c) [expr int($canvas(width) *$a/(2.*$numx)) + 2]
	    set y($c) [expr int($canvas(height)*$b/(2.*$numy)) + 2]
	}
    }

    TileIt $w $h x y [expr $numx*$numy]
}

proc TileIt {w h xx yy nn} {
    upvar $xx x
    upvar $yy y
    global ds9
    global current
    global canvas

    global debug
    if {$debug(tcl,layout)} {
	puts "TileIt"
    }

    set i 0
    foreach f $ds9(active) {
	if {$i<$nn} {
	    $f configure -x $x($i) -y $y($i) -width $w -height $h \
		-anchor center
	    $f show
	    $canvas(name) raise $f
	    if {!$ds9(freeze)} {
		BindEventsFrame $f
	    }
	}
	incr i
    }

    # if manual grid, current frame could be not included
    if {[llength $ds9(active)] > $nn} {
	set current(frame) [lindex $ds9(active) 0]
    }
    FrameToFront $current(frame)
}

proc UpdateEditMenu {} {
    global ds9
    global current
    global buttons
    global debug

    if {$debug(tcl,update)} {
	puts "UpdateEditMenu"
    }

    switch $ds9(mode) {
	pointer {
	    if {$current(frame) != ""} {
		set l [$current(frame) has marker undo]
		if {$l != ""} {
		    $ds9(mb).edit entryconfig 1 \
			-label "Undo [string totitle $l]" -state normal

		    $buttons(name).edit.undo configure -state normal
		} else {
		    $ds9(mb).edit entryconfig 1 -label "Undo" -state disabled
		    $buttons(name).edit.undo configure -state disabled
		}

		if {[$current(frame) has marker select]} {
		    $ds9(mb).edit entryconfig "Cut" -state normal
		    $ds9(mb).edit entryconfig "Copy" -state normal
		    $buttons(name).edit.cut configure -state normal
		    $buttons(name).edit.copy configure -state normal
		} else {
		    $ds9(mb).edit entryconfig "Cut" -state disabled
		    $ds9(mb).edit entryconfig "Copy" -state disabled
		    $buttons(name).edit.cut configure -state disabled
		    $buttons(name).edit.copy configure -state disabled
		}

		if {[$current(frame) has marker paste]} {
		    $ds9(mb).edit entryconfig "Paste" -state normal
		    $buttons(name).edit.paste configure -state normal
		} else {
		    $ds9(mb).edit entryconfig "Paste" -state disabled
		    $buttons(name).edit.paste configure -state disabled
		}
	    } else {
		$ds9(mb).edit entryconfig 1 -label "Undo" -state disabled
		$ds9(mb).edit entryconfig "Cut" -state disabled
		$ds9(mb).edit entryconfig "Copy" -state disabled
		$ds9(mb).edit entryconfig "Paste" -state disabled

		$buttons(name).edit.undo configure -state disabled
		$buttons(name).edit.cut configure -state disabled
		$buttons(name).edit.copy configure -state disabled
		$buttons(name).edit.paste configure -state disabled
	    }
	}
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate {
	    $ds9(mb).edit entryconfig 1 -label "Undo" -state disabled
	}
    }
}

proc UpdateFrameMenu {} {
    global ds9
    global current
    global buttons

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateFrameMenu"
    }

    if {$current(frame) != ""} {
	$ds9(mb).frame entryconfig "Delete Frame" -state normal
	$ds9(mb).frame entryconfig "Reset Frame" -state normal
	$ds9(mb).frame entryconfig "Refresh Frame" -state normal
	$ds9(mb).frame entryconfig "Single Frame" -state normal
	$ds9(mb).frame entryconfig "Tile Frames" -state normal
	$ds9(mb).frame entryconfig "Blink Frames" -state normal
	$ds9(mb).frame entryconfig "Tile" -state normal
	$ds9(mb).frame entryconfig "Blink Interval" -state normal
	$ds9(mb).frame entryconfig "Match Colorbars" -state normal

	$buttons(name).frame.delete configure -state normal
	$buttons(name).frame.reset configure -state normal
	$buttons(name).frame.single configure -state normal
	$buttons(name).frame.tile configure -state normal
	$buttons(name).frame.blink configure -state normal
	$buttons(name).frame.first configure -state normal
	$buttons(name).frame.prev configure -state normal
	$buttons(name).frame.next configure -state normal
	$buttons(name).frame.last configure -state normal
    } else {
	$ds9(mb).frame entryconfig "Delete Frame" -state disabled
	$ds9(mb).frame entryconfig "Reset Frame" -state disabled
	$ds9(mb).frame entryconfig "Refresh Frame" -state disabled
	$ds9(mb).frame entryconfig "Single Frame" -state disabled
	$ds9(mb).frame entryconfig "Tile Frames" -state disabled
	$ds9(mb).frame entryconfig "Blink Frames" -state disabled
	$ds9(mb).frame entryconfig "Tile" -state disabled
	$ds9(mb).frame entryconfig "Blink Interval" -state disabled
	$ds9(mb).frame entryconfig "Match Colorbars" -state disabled

	$buttons(name).frame.delete configure -state disabled
	$buttons(name).frame.reset configure -state disabled
	$buttons(name).frame.single configure -state disabled
	$buttons(name).frame.tile configure -state disabled
	$buttons(name).frame.blink configure -state disabled
	$buttons(name).frame.first configure -state disabled
	$buttons(name).frame.prev configure -state disabled
	$buttons(name).frame.next configure -state disabled
	$buttons(name).frame.last configure -state disabled
    }
}

proc UpdateFrameMenuItems {} {
    global ds9

    if {[$ds9(mb).frame index end]>23} {
	$ds9(mb).frame delete 24 end
    }
    if {[$ds9(mb).frame.active index end]>3} {
	$ds9(mb).frame.active delete 4 end
    }

    foreach f $ds9(frames) {
	$ds9(mb).frame add radiobutton -label $f \
	    -variable ds9(next) -value $f -command GotoFrame
	$ds9(mb).frame.active add checkbutton -label $f \
	    -variable active($f) -command UpdateActiveFrames
    }
}

proc TileDialog {} {
    global tile
    global ds9
    global menu

    # see if we already have a window visible
    if [winfo exist $tile(top)] {
	raise $tile(top)
	return
    }

    set w $tile(top)
    set title "Tile Parameters"

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW TileDestroyDialog

    $w configure -menu $tile(mb)

    set tile(gd,mode) $tile(grid,mode)
    set tile(gd,row) $tile(grid,row)
    set tile(gd,col) $tile(grid,col)
    set tile(gd,gap) $tile(grid,gap)

    menu $tile(mb) -tearoff 0
    $tile(mb) add cascade -label "File" -menu $tile(mb).file

    menu $tile(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $tile(mb).file add command -label "Close" -command TileDestroyDialog

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -side top -ipadx 4 -ipady 4 -fill x -expand true

    label $w.param.t1 -text "Grid"
    radiobutton $w.param.auto -text "Automatic" \
	-variable tile(gd,mode) -value automatic
    radiobutton $w.param.manual -text "Manual" \
	-variable tile(gd,mode) -value manual

    label $w.param.t2 -text "Frame Layout"
    entry $w.param.row -textvariable tile(gd,row) -width 6
    label $w.param.t2x -text "x"
    entry $w.param.col -textvariable tile(gd,col) -width 6

    label $w.param.t3 -text "Frame Gap"
    entry $w.param.gap -textvariable tile(gd,gap) -width 6
    label $w.param.t4 -text "pixels"

    grid $w.param.t1 $w.param.auto - - $w.param.manual \
	-padx 4 -pady 2 -sticky w
    grid $w.param.t2 $w.param.row $w.param.t2x $w.param.col \
	-padx 4 -pady 2 -sticky w
    grid $w.param.t3 $w.param.gap - $w.param.t4 -padx 4 -pady 2 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" -command TileApplyDialog
    button $w.buttons.close -text "Close" -command TileDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true
}

proc TileDestroyDialog {} {
    global tile

    destroy $tile(top)
    destroy $tile(mb)

    unset tile(gd,mode)
    unset tile(gd,row)
    unset tile(gd,col)
    unset tile(gd,gap)
}

proc TileApplyDialog {} {
    global tile

    set tile(mode) grid
    set tile(grid,mode) $tile(gd,mode)
    set tile(grid,row) $tile(gd,row)
    set tile(grid,col) $tile(gd,col)
    set tile(grid,gap) $tile(gd,gap)
    
    DisplayMode
}

proc CubeDialog {} {
    global cube
    global current
    global menu
    global ds9
    global blink

    # see if we already have a window visible
    if [winfo exist $cube(top)] {
	raise $cube(top)
	return
    }

    set w $cube(top)
    set title "Data Cube"

    # create the cube window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW CubeDestroyDialog

    $w configure -menu $cube(mb)

    # for animation
    set cube(id) 0 

    menu $cube(mb) -tearoff 0
    $cube(mb) add cascade -label "File" -menu $cube(mb).file
    $cube(mb) add cascade -label "Interval" -menu $cube(mb).blink

    menu $cube(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $cube(mb).file add command -label "Close" -command CubeDestroyDialog

    menu $cube(mb).blink -tearoff 0 -selectcolor $menu(selectcolor)
    $cube(mb).blink add radiobutton -label "1/4 Second" \
	-variable blink(interval) -value 250
    $cube(mb).blink add radiobutton -label "1/2 Second" \
	-variable blink(interval) -value 500
    $cube(mb).blink add radiobutton -label "1 Second" \
	-variable blink(interval) -value 1000
    $cube(mb).blink add radiobutton -label "2 Seconds" \
	-variable blink(interval) -value 2000
    $cube(mb).blink add radiobutton -label "4 Seconds" \
	-variable blink(interval) -value 4000
    $cube(mb).blink add radiobutton -label "8 Seconds" \
	-variable blink(interval) -value 8000

    frame $w.param -relief groove -borderwidth 2
    frame $w.vcr -relief groove -borderwidth 2
    pack $w.param $w.vcr -fill x -expand true -ipadx 4 -ipady 4

    set cube(scale) [scale $w.param.slice -from 1 -to 1 -length 300 \
			 -variable cube(slice) -orient horizontal \
			 -tickinterval 1 \
			 -showvalue true -resolution 1 \
			 -command CubeApplyDialog ]
    pack $w.param.slice	-side top -ipadx 4 -ipady 4 -anchor w

    set bs 2
    button $w.vcr.first -text "First" -width $bs -height $bs \
	-command CubeFirstDialog
    button $w.vcr.prev -text "Prev" -width $bs -height $bs \
	-command CubePrev
    button $w.vcr.stop -text "Stop" -width $bs -height $bs \
	-command CubeStopDialog
    button $w.vcr.play -text "Play" -width $bs -height $bs \
	-command CubePlayDialog
    button $w.vcr.next -text "Next" -width $bs -height $bs \
	-command CubeNext
    button $w.vcr.last -text "Last" -width $bs -height $bs \
	-command CubeLastDialog

    pack $w.vcr.first $w.vcr.prev \
	$w.vcr.stop $w.vcr.play \
	$w.vcr.next $w.vcr.last -side left -expand true

    UpdateCubeDialog
}

proc CubeDestroyDialog {} {
    global cube

    CubeStopDialog

    destroy $cube(top)
    destroy $cube(mb)

    unset cube(scale)
    unset cube(slice)
    unset cube(depth)
    unset cube(id)
}

proc CubeApplyDialog {slice} {
    global cube
    global current

    CubeStopDialog

    if {$current(frame) != ""} {
	$current(frame) update fits slice $cube(slice)
    }
}

proc CubeStopDialog {} {
    global cube
    global curent

    if [winfo exist $cube(top)] {
	if {$cube(id)>0} {
	    after cancel $cube(id)
	    set cube(id) 0
	}
    }
}

proc CubePlayDialog {} {
    global cube

    if {$cube(id) == 0} {
	CubeTimer
    }
}

proc CubeTimer {} {
    global cube
    global current
    global blink

    if {$cube(slice) == $cube(depth)} {
	set cube(slice) 1
    } else {
	set cube(slice) [expr $cube(slice)+1]
    }
    if {$current(frame) != ""} {
	$current(frame) update fits slice $cube(slice)
    }
    set cube(id) [after $blink(interval) CubeTimer]
}

proc CubeFirstDialog {} {
    global cube
    global current

    CubeStopDialog

    set cube(slice) 1
    if {$current(frame) != ""} {
	$current(frame) update fits slice $cube(slice)
    }
}

proc CubePrev {} {
    global cube
    global current

    if {$current(frame) != ""} {
	CubeStopDialog

	set cube(slice) [$current(frame) get fits slice]
	set cube(depth) [$current(frame) get fits depth]

	if {$cube(slice) == 1} {
	    set cube(slice) $cube(depth)
	} else {
	    set cube(slice) [expr $cube(slice)-1]
	}

	$current(frame) update fits slice $cube(slice)
    }
}

proc CubeNext {} {
    global cube
    global current

    if {$current(frame) != ""} {
	CubeStopDialog

	set cube(slice) [$current(frame) get fits slice]
	set cube(depth) [$current(frame) get fits depth]

	if {$cube(slice) == $cube(depth)} {
	    set cube(slice) 1
	} else {
	    set cube(slice) [expr $cube(slice)+1]
	}

	$current(frame) update fits slice $cube(slice)
    }
}

proc CubeLastDialog {} {
    global cube
    global current

    CubeStopDialog

    set cube(slice) $cube(depth)
    if {$current(frame) != ""} {
	$current(frame) update fits slice $cube(slice)
    }
}

proc UpdateCubeDialog {} {
    global cube
    global current

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateCubeDialog"
    }

    if {[winfo exist $cube(top)] && $current(frame) != ""} {
	set cube(depth) [$current(frame) get fits depth]
	set cube(slice) [$current(frame) get fits slice]
	if {$cube(depth) < 10} {
	    set interval 1
	} elseif {$cube(depth) < 100} {
	    set interval 10
	} else {
	    set interval 100
	}

	$cube(scale) configure -to $cube(depth) -tickinterval $interval
    }
}

proc ProcessTileCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global tile

    switch -- [string tolower [lindex $var $i]] {
	mode {
	    incr i
	    set tile(mode) [lindex $var $i]
	}
	grid {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		mode {
		    incr i
		    set tile(grid,mode) [lindex $var $i]
		}
		layout {
		    incr i
		    set tile(grid,row) [lindex $var $i]
		    incr i
		    set tile(grid,col) [lindex $var $i]
		}
		gap {
		    incr i
		    set tile(grid,gap) [lindex $var $i]
		}
		default {
		    set tile(mode) grid
		}
	    }
	}
	column {
	    set tile(mode) column
	}
	row {
	    set tile(mode) row
	}

	yes -
	true -
	1 -
	no -
	false -
	0 {
	    if {[FromYesNo [lindex $var $i]]} {
		set ds9(displaymode) tile
	    } else {
		set ds9(displaymode) single
	    }
	}
	default {
	    set ds9(displaymode) tile
	    incr i -1
	}
    }
    DisplayMode
}
