package provide DS9 1.0

proc GridUpdate {} {
    global grid
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	if {$grid(view) && [$current(frame) has fits]} {
	    AdjustCoordSystem grid(system)
	    $current(frame) grid create $grid(system) $grid(type) \
		[GridOptions]
	} else {
	    $current(frame) grid delete
	}

	UnsetWatchCursor
    }
}	    

proc GridOptions {} {
    global grid
    global current

    set ops "\""

    # Type
    switch $grid(type) {
	1 {
	    set ops "$ops Labelling=interior,"
	    set ops "$ops NumLabGap(1)=.01,"
	    set ops "$ops NumLabGap(2)=.01,"
	}
	2 -
	3 {
	    set ops "$ops Labelling=exterior,"
	    set ops "$ops NumLabGap(1)=-.01,"
	    set ops "$ops NumLabGap(2)=-.01,"
	}
	4 {
	    set ops "$ops Labelling=exterior,"
	    set ops "$ops NumLabGap(1)=.01,"
	    set ops "$ops NumLabGap(2)=.01,"
	}
    }

    # Grid
    set ops "$ops Grid=$grid(grid),"
    set ops "$ops Colour(grid)=$grid(grid,color),"
    set ops "$ops Width(grid)=$grid(grid,width),"
    set ops "$ops Style(grid)=$grid(grid,style),"

    # Axes
    set ops "$ops DrawAxes=$grid(axes),"
    set ops "$ops Colour(axes)=$grid(axes,color),"
    set ops "$ops Width(axes)=$grid(axes,width),"
    set ops "$ops Style(axes)=$grid(axes,style),"

    # Ticks
    if {!$grid(tick)} {
	set ops "$ops MajTickLen=0,"
	set ops "$ops MinTick(1)=0,"
	set ops "$ops MinTick(2)=0,"
    }
    set ops "$ops Colour(ticks)=$grid(tick,color),"
    set ops "$ops Width(ticks)=$grid(tick,width),"
    set ops "$ops Style(ticks)=$grid(tick,style),"

    # Border
    set ops "$ops Border=$grid(border),"
    set ops "$ops Colour(border)=$grid(border,color),"
    set ops "$ops Width(border)=$grid(border,width),"
    set ops "$ops Style(border)=$grid(border,style),"

    # Rotate Text
    set ops "$ops LabelUp=$grid(labelup),"

    # NumLab
    set ops "$ops NumLab=$grid(numlab),"
    set ops \
     "$ops Font(numlab)=[GridFont2Ast $grid(numlab,font) $grid(numlab,style)],"
    set ops "$ops Size(numlab)=$grid(numlab,size),"
    set ops "$ops Colour(numlab)=$grid(numlab,color),"

    # TextLab
    set ops "$ops TextLab(1)=$grid(textlab),"
    if {!$grid(textlab,def1)} {
	set ops "$ops Label(1)=[GridStripComma $grid(label1)] ,"
    }
    set ops "$ops TextLabGap(1)=$grid(textlab,gap1),"

    set ops "$ops TextLab(2)=$grid(textlab),"
    if {!$grid(textlab,def2)} {
	set ops "$ops Label(2)=[GridStripComma $grid(label2)] ,"
    }
    set ops "$ops TextLabGap(2)=$grid(textlab,gap2),"

    set ops \
  "$ops Font(textlab)=[GridFont2Ast $grid(textlab,font) $grid(textlab,style)],"
    set ops "$ops Size(textlab)=$grid(textlab,size),"
    set ops "$ops Colour(textlab)=$grid(textlab,color),"

    # Title
    set ops "$ops DrawTitle=$grid(title),"
    if {$grid(title,def)} {
	set t [GridStripComma "[$current(frame) get fits object name]"]
	if {$t != ""} {
	    set ops "$ops Title=$t ,"
	}
    } else {
	set t [GridStripComma "$grid(title,text)"]
	if {$t != ""} {
	    set ops "$ops Title=$t ,"
	}
    }
    set ops \
	"$ops Font(title)=[GridFont2Ast $grid(title,font) $grid(title,style)],"
    set ops "$ops Size(title)=$grid(title,size),"
    set ops "$ops Colour(title)=$grid(title,color),"

    # Format
    switch $grid(system) {
	"fk4" -
	"fk5" -
	"icrs" {
	    switch $grid(format) {
		degrees {
		    set ops "$ops Format(1)=d.3,"
		    set ops "$ops Format(2)=d.3,"
		}
		sexagesimal {
		    set ops "$ops Format(1)=hms,"
		    set ops "$ops Format(2)=dms,"
		}
		hms {
		    set ops "$ops Format(1)=lhms,"
		    set ops "$ops Format(2)=ldms,"
		}
	    }
	}
	"galactic" -
	"ecliptic" {
	    switch $grid(format) {
		degrees {
		    set ops "$ops Format(1)=d.3,"
		    set ops "$ops Format(2)=d.3,"
		}
		sexagesimal {
		    set ops "$ops Format(1)=dms,"
		    set ops "$ops Format(2)=dms,"
		}
		hms {
		    set ops "$ops Format(1)=ldms,"
		    set ops "$ops Format(2)=ldms,"
		}
	    }
	}
    }

    # Grid Spacing

    if {$grid(gap1) != ""} {
	if {$grid(gapunit1) == "degrees"} {
	    set ops "$ops Gap(1)=[expr 3.14159/180.*$grid(gap1)],"
	} else {
	    set ops "$ops Gap(1)=$grid(gap1),"
	}
    }

    if {$grid(gap2) != ""} {
	if {$grid(gapunit2) == "degrees"} {
	    set ops "$ops Gap(2)=[expr 3.14159/180.*$grid(gap2)],"
	} else {
	    set ops "$ops Gap(2)=$grid(gap2),"
	}
    }

    set ops "$ops \""

    return $ops
}

proc GridParseOptions {opt} {
    global grid
    global current

    # Type
    set dummy {}
    strtok { Labelling=%[^,],} opt dummy
    strtok " NumLabGap(1)=%f," opt dummy
    strtok " NumLabGap(2)=%f," opt dummy

    # Grid
    strtok " Grid=%d," opt grid(grid)
    strtok " Colour(grid)=%d," opt grid(grid,color)
    strtok " Width(grid)=%d," opt grid(grid,width)
    strtok " Style(grid)=%d," opt grid(grid,style)

    # Axes
    strtok " DrawAxes=%d," opt grid(axes)
    strtok " Colour(axes)=%d," opt grid(axes,color)
    strtok " Width(axes)=%d," opt grid(axes,width)
    strtok " Style(axes)=%d," opt grid(axes,style)

    # Ticks
    set grid(tick) 1
    strtok " MajTickLen=%d," opt grid(tick)
    strtok " MinTick(1)=%d," opt grid(tick)
    strtok " MinTick(2)=%d," opt grid(tick)

    strtok " Colour(ticks)=%d," opt grid(tick,color)
    strtok " Width(ticks)=%d," opt  grid(tick,width)
    strtok " Style(ticks)=%d," opt grid(tick,style)

    # Border
    strtok " Border=%d," opt grid(border)
    strtok " Colour(border)=%d," opt grid(border,color)
    strtok " Width(border)=%d," opt grid(border,width)
    strtok " Style(border)=%d," opt grid(border,style)

    # Rotate Text
    strtok " LabelUp=%d," opt grid(labelup)

    # Numlab
    strtok " NumLab=%d," opt grid(numlab)

    set numlabFont 0
    strtok " Font(numlab)=%d," opt numlabFont
    GridAst2Font $numlabFont grid(numlab,font) grid(numlab,style)
    
    strtok " Size(numlab)=%d," opt grid(numlab,size)
    strtok " Colour(numlab)=%d," opt grid(numlab,color)

    # Textlab
    strtok " TextLab(1)=%d," opt grid(textlab)
    set grid(textlab,def1) [expr ![strtok { Label(1)=%[^,],} opt grid(label1)]]
    strtok " TextLabGap(1)=%f," opt grid(textlab,gap1)

    strtok " TextLab(2)=%d," opt grid(textlab)
    set grid(textlab,def2) [expr ![strtok { Label(2)=%[^,],} opt grid(label2)]]
    strtok " TextLabGap(2)=%f," opt grid(textlab,gap2)

    set textlabFont 0
    strtok " Font(textlab)=%d," opt textlabFont
    GridAst2Font $textlabFont grid(textlab,font) grid(textlab,style)

    strtok " Size(textlab)=%d," opt grid(textlab,size)
    strtok " Colour(textlab)=%d," opt grid(textlab,color)

    # Title
    strtok " DrawTitle=%d," opt grid(title)
    strtok { Title=%[^,],} opt grid(title,text)

    set titleFont 0
    strtok " Font(title)=%d," opt titleFont
    GridAst2Font $titleFont grid(title,font) grid(title,style)

    strtok " Size(title)=%d," opt grid(title,size)
    strtok " Colour(title)=%d" opt grid(title,color)

    # Format
    set format1 ""
    set format2 ""
    strtok { Format(1)=%[^,]} opt format1
    strtok { Format(2)=%[^,]} opt format2
    switch $format1 {
	"d.3" {set grid(format) degrees}
	"hms" {set grid(format) sexagesimal}
	"dms" {set grid(format) sexagesimal}
	"lhms" {set grid(format) hms}
	"ldms" {set grid(format) hms}
	default {set grid(format) sexagesimal}
    }

    # Spacing
    set gap1 ""
    strtok { Gap(1)=%[^,]} opt gap1
    if {($gap1 != "") && ($grid(gapunit1) == "degrees")} {
	set grid(gap1) [expr 180.*$gap1/3.14159]
    } else {
	set grid(gap1) $gap1
    }

    set gap2 ""
    strtok { Gap(2)=%[^,]} opt gap2
    if {($gap2 != "") && ($grid(gapunit2) == "degrees")} {
	set grid(gap2) [expr 180.*$gap2/3.14159]
    } else {
	set grid(gap2) $gap2
    }
}

proc strtok {tmp strvar varvar} {
    upvar $strvar str
    upvar $varvar var

    set r [scan $str $tmp var]
    if {$r} {
	set i [string first "," $str]
	set str [string range $str [expr $i+1] end]
#	puts "Success parse $tmp"
    } else {
#	puts "Failed to parse $tmp"
    }

    return $r
}

proc GridAst2Font {ast fnvar fsvar} {
    upvar $fnvar fn
    upvar $fsvar fs

    switch $ast {
	0 {set fn "system"; set fs "plain"}
	1 {set fn "times"; set fs "plain"}
	2 {set fn "helvetica"; set fs "plain"}
	3 {set fn "symbol"; set fs "plain"}
	4 {set fn "courier"; set fs "plain"}
	10 {set fn "system"; set fs "bold"}
	11 {set fn "times"; set fs "bold"}
	12 {set fn "helvetica"; set fs "bold"}
	13 {set fn "symbol"; set fs "bold"}
	14 {set fn "courier"; set fs "bold"}
	20 {set fn "system"; set fs "italic"}
	21 {set fn "times"; set fs "italic"}
	22 {set fn "helvetica"; set fs "italic"}
	23 {set fn "symbol"; set fs "italic"}
	24 {set fn "courier"; set fs "italic"}
	default {set fn "system"; set fs "plain"}
    }
}

proc GridFont2Ast {fn fs} {
    if {$fn == "system" && $fs == "plain"} {
	return 0;
    } elseif {$fn == "times" && $fs == "plain"} {
	return 1;
    } elseif {$fn == "helvetica" && $fs == "plain"} {
	return 2;
    } elseif {$fn == "symbol" && $fs == "plain"} {
	return 3;
    } elseif {$fn == "courier" && $fs == "plain"} {
	return 4;
    } elseif {$fn == "system" && $fs == "bold"} {
	return 10;
    } elseif {$fn == "times" && $fs == "bold"} {
	return 11;
    } elseif {$fn == "helvetica" && $fs == "bold"} {
	return 12;
    } elseif {$fn == "symbol" && $fs == "bold"} {
	return 13;
    } elseif {$fn == "courier" && $fs == "bold"} {
	return 14;
    } elseif {$fn == "system" && $fs == "italic"} {
	return 20;
    } elseif {$fn == "times" && $fs == "italic"} {
	return 21;
    } elseif {$fn == "helvetica" && $fs == "italic"} {
	return 22;
    } elseif {$fn == "symbol" && $fs == "italic"} {
	return 23;
    } elseif {$fn == "courier" && $fs == "italic"} {
	return 24;
    } else {
	return 0;
    }
}

proc GridDialog {} {
    global grid
    global current
    global menu
    global gr
    global ds9

    # see if we already have a window visible

    if [winfo exist $grid(top)] {
	raise $grid(top)
	return
    }

    set w $grid(top)
    set title "Coordinate Grid"

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW GridDestroyDialog

    $w configure -menu $grid(mb)

    menu $grid(mb) -tearoff 0

    GridFileMenu
    GridTypeMenu
    GridCoordMenu
    GridViewMenu
    GridColorMenu
    GridLineMenu
    GridFontMenu

    set length 300

    frame $w.buttons -relief groove -borderwidth 2
    frame $w.label -relief groove -borderwidth 2
    frame $w.param -relief groove -borderwidth 2
    pack $w.label $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    # Labels

    label $w.label.label -text "Image Title"
    entry $w.label.title -textvariable grid(title,text) \
	-width 23 -relief sunken -bd 2
    checkbutton $w.label.default -text "Default" -variable grid(title,def)
    label $w.label.label1 -text "Axis 1"
    entry $w.label.title1 -textvariable grid(label1) \
	-width 23 -relief sunken -bd 2
    checkbutton $w.label.default1 -text "Default" -variable grid(textlab,def1)
    label $w.label.label2 -text "Axis 2"
    entry $w.label.title2 -textvariable grid(label2) \
	-width 23 -relief sunken -bd 2
    checkbutton $w.label.default2 -text "Default" -variable grid(textlab,def2)

    grid $w.label.label -row 0 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.label.title -row 0 -column 1 -padx 4 -pady 1 -sticky w
    grid $w.label.default -row 0 -column 2 -padx 4 -pady 1 -sticky w
    grid $w.label.label1 -row 1 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.label.title1 -row 1 -column 1 -padx 4 -pady 1 -sticky w
    grid $w.label.default1 -row 1 -column 2 -padx 4 -pady 1 -sticky w
    grid $w.label.label2 -row 2 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.label.title2 -row 2 -column 1 -padx 4 -pady 1 -sticky w
    grid $w.label.default2 -row 2 -column 2 -padx 4 -pady 1 -sticky w

    # Params

    label $w.param.gap -text "Grid Gap"
    label $w.param.gapunit -text "Grid Units"
    label $w.param.textlabgap -text "Label Gap"

    label $w.param.title1 -text "Axis 1"
    entry $w.param.gap1 -textvariable grid(gap1) \
	-width 10 -relief sunken -bd 2
    label $w.param.gapunit1 -textvariable grid(gapunit1) \
	-relief groove -bd 2 -width 9 -padx 4
    entry $w.param.textlabgap1 -textvariable grid(textlab,gap1) \
	-width 10 -relief sunken -bd 2
    label $w.param.textlabgapunit1 -text "%" -padx 4

    label $w.param.title2 -text "Axis 2"
    entry $w.param.gap2 -textvariable grid(gap2) \
	-width 10 -relief sunken -bd 2
    label $w.param.gapunit2 -textvariable grid(gapunit2) \
	-relief groove -bd 2 -width 9 -padx 4
    entry $w.param.textlabgap2 -textvariable grid(textlab,gap2) \
	-width 10 -relief sunken -bd 2
    label $w.param.textlabgapunit2 -text "%" -padx 4

    grid $w.param.gap -row 0 -column 1 -padx 4 -pady 1 -sticky news
    grid $w.param.gapunit -row 0 -column 2 -padx 4 -pady 1 -sticky news
    grid $w.param.textlabgap -row 0 -column 3 -padx 4 -pady 1 -sticky news

    grid $w.param.title1 -row 1 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.param.gap1 -row 1 -column 1 -padx 4 -pady 1 -sticky w
    grid $w.param.gapunit1 -row 1 -column 2 -padx 4 -pady 1 -sticky w
    grid $w.param.textlabgap1 -row 1 -column 3 -padx 4 -pady 1 -sticky w
    grid $w.param.textlabgapunit1 -row 1 -column 4 -padx 4 -pady 1 -sticky w

    grid $w.param.title2 -row 2 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.param.gap2 -row 2 -column 1 -padx 4 -pady 1 -sticky w
    grid $w.param.gapunit2 -row 2 -column 2 -padx 4 -pady 1 -sticky w
    grid $w.param.textlabgap2 -row 2 -column 3 -padx 4 -pady 1 -sticky w
    grid $w.param.textlabgapunit2 -row 2 -column 4 -padx 4 -pady 1 -sticky w

    button $w.buttons.apply -text "Apply" -command GridApplyDialog
    button $w.buttons.close -text "Close" -command GridDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "GridApplyDialog"

    UpdateGridDialog
    GridUpdateAxesUnit
}

proc GridApplyDialog {} {
    global grid

    set grid(view) 1
    GridUpdate
}

proc GridDestroyDialog {} {
    global grid

    destroy $grid(top)
    destroy $grid(mb)
}

proc GridUpdateAxesUnit {} {
    global grid

    switch $grid(system) {
	fk4 -
	fk5 -
	icrs -
	galactic -
	ecliptic {
	    set grid(gapunit1) degrees
	    set grid(gapunit2) degrees
	}
	linear {
	    set grid(gapunit1) x
	    set grid(gapunit2) y
	}
	detector -
	amplifier -
	physical -
	image {
	    set grid(gapunit1) pixels
	    set grid(gapunit2) pixels
	}
	default {
	    set grid(gapunit1) ""
	    set grid(gapunit2) ""
	}
    }
}

proc UpdateGridMenu {} {
    global grid
    global current

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateGridMenu"
    }

    # set menu

    if {($current(frame) != "") && [$current(frame) has fits]} {
	set grid(view) [$current(frame) has grid]
    }

    # reassign system and format

    if {($current(frame) != "") && [$current(frame) has fits]} {
	if [$current(frame) has grid] {
	    set grid(system) [$current(frame) get grid system]
	    set grid(type) [$current(frame) get grid type]
	    GridParseOptions [$current(frame) get grid option]
	}
    }
}

proc UpdateGridDialog {} {
    global current
    global grid

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateGridDialog"
    }

    if {[winfo exist $grid(top)]} {

	# enable/disable menu items
	UpdateCoordMenu $grid(mb).coord 0 1 1
	if {$current(frame) != ""} {
	    if {[$current(frame) has wcs equatorial]} {
		$grid(mb).coord entryconfig "HMS/DMS" -state normal
	    } else {
		$grid(mb).coord entryconfig "HMS/DMS" -state disabled
	    }
	} else {
	    $grid(mb).coord entryconfig "HMS/DMS" -state normal
	}

	# change values if needed
	AdjustCoordSystem grid(system)
    }
}

proc GridFileMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label File -menu $grid(mb).file

    menu $grid(mb).file -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).file add command -label Apply \
	-command GridApplyDialog
    $grid(mb).file add separator
    $grid(mb).file add command -label "Load Configuration..." \
	-command GridLoadDialog
    $grid(mb).file add command -label "Save Configuration..." \
	-command GridSaveDialog
    $grid(mb).file add separator
    $grid(mb).file add command -label Close \
	-command GridDestroyDialog
}

proc GridCoordMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label Coordinate -menu $grid(mb).coord

    menu $grid(mb).coord -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).coord add radiobutton -label "Equatorial B1950" \
	-variable grid(system) -value fk4 -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "Equatorial J2000" \
	-variable grid(system) -value fk5 -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "ICRS" \
	-variable grid(system) -value icrs -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "Galactic" \
	-variable grid(system) -value galactic -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "Ecliptic" \
	-variable grid(system) -value ecliptic -command GridUpdateAxesUnit
    $grid(mb).coord add separator
    $grid(mb).coord add radiobutton -label "Linear" \
	-variable grid(system) -value linear -command GridUpdateAxesUnit
    $grid(mb).coord add separator
    $grid(mb).coord add radiobutton -label "Detector" \
	-variable grid(system) -value detector -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "Amplifier" \
	-variable grid(system) -value amplifier -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "Physical" \
	-variable grid(system) -value physical -command GridUpdateAxesUnit
    $grid(mb).coord add radiobutton -label "Image" \
	-variable grid(system) -value image -command GridUpdateAxesUnit
    $grid(mb).coord add separator
    $grid(mb).coord add radiobutton -label "Degrees" \
	-variable grid(format) -value degrees
    $grid(mb).coord add radiobutton -label "Sexagesimal" \
	-variable grid(format) -value sexagesimal
    $grid(mb).coord add radiobutton -label "HMS/DMS" \
	-variable grid(format) -value hms
}

proc GridTypeMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label Type -menu $grid(mb).type

    menu $grid(mb).type -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).type add radiobutton -label "Frame Interior Axes" \
	-variable grid(type) -value 1
    $grid(mb).type add radiobutton -label "Frame Edge Axes" \
	-variable grid(type) -value 2
    $grid(mb).type add radiobutton -label "Image Edge Axes/Interior Numerics" \
	-variable grid(type) -value 3
    $grid(mb).type add radiobutton -label "Image Edge Axes/Exterior Numerics" \
	-variable grid(type) -value 4
}

proc GridViewMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label View -menu $grid(mb).view

    menu $grid(mb).view -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).view add checkbutton -label Grid -variable grid(grid)
    $grid(mb).view add checkbutton -label Axes -variable grid(axes)
    $grid(mb).view add checkbutton -label "Axes Numbers" -variable grid(numlab)
    $grid(mb).view add checkbutton -label "Axes Tickmarks" -variable grid(tick)
    $grid(mb).view add checkbutton -label "Axes Label" -variable grid(textlab)
    $grid(mb).view add checkbutton -label Title -variable grid(title)
    $grid(mb).view add checkbutton -label Border -variable grid(border)
    $grid(mb).view add separator
    $grid(mb).view add checkbutton -label "Vertical Text" \
	-variable grid(labelup)
}

proc GridColorMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label Color -menu $grid(mb).color

    menu $grid(mb).color -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).color add cascade -label Grid -menu $grid(mb).color.grid
    $grid(mb).color add cascade -label Axes -menu $grid(mb).color.axes
    $grid(mb).color add cascade -label "Axes Numbers" \
	-menu $grid(mb).color.numlab
    $grid(mb).color add cascade -label "Axes Label" \
	-menu $grid(mb).color.textlab
    $grid(mb).color add cascade -label Title -menu $grid(mb).color.title
    $grid(mb).color add cascade -label Tickmarks -menu $grid(mb).color.tick
    $grid(mb).color add cascade -label Border -menu $grid(mb).color.border

    GridCreateColorMenu $grid(mb).color.grid grid(grid,color)
    GridCreateColorMenu $grid(mb).color.axes grid(axes,color)
    GridCreateColorMenu $grid(mb).color.title grid(title,color)
    GridCreateColorMenu $grid(mb).color.textlab grid(textlab,color)
    GridCreateColorMenu $grid(mb).color.numlab grid(numlab,color)
    GridCreateColorMenu $grid(mb).color.tick grid(tick,color)
    GridCreateColorMenu $grid(mb).color.border grid(border,color)
}

proc GridCreateColorMenu {which varname} {
    global menu
    global grid

    menu $which -tearoff 0 -selectcolor $menu(selectcolor)
    $which add radiobutton -label "Black" -variable $varname -value 0
    $which add radiobutton -label "White" -variable $varname -value 1
    $which add radiobutton -label "Red" -variable $varname -value 2
    $which add radiobutton -label "Green" -variable $varname -value 3
    $which add radiobutton -label "Blue" -variable $varname -value 4
    $which add radiobutton -label "Cyan" -variable $varname -value 5
    $which add radiobutton -label "Magenta" -variable $varname -value 6
    $which add radiobutton -label "Yellow" -variable $varname -value 7
}

proc GridLineMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label Line -menu $grid(mb).line

    menu $grid(mb).line -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).line add cascade -label Grid -menu $grid(mb).line.grid
    $grid(mb).line add cascade -label Axes -menu $grid(mb).line.axes
    $grid(mb).line add cascade -label Tickmarks -menu $grid(mb).line.tick
    $grid(mb).line add cascade -label Border -menu $grid(mb).line.border

    GridCreateLineMenu $grid(mb).line.grid grid(grid,width) grid(grid,style)
    GridCreateLineMenu $grid(mb).line.axes grid(axes,width) grid(axes,style)
    GridCreateLineMenu $grid(mb).line.tick grid(tick,width) grid(tick,style)
    GridCreateLineMenu $grid(mb).line.border \
	grid(border,width) grid(border,style)
}

proc GridCreateLineMenu {which var1 var2} {
    global grid
    global menu

    menu $which -tearoff 0 -selectcolor $menu(selectcolor)
    $which add radiobutton -label "Thin" -variable $var1 -value 0
    $which add radiobutton -label "1" -variable $var1 -value 1
    $which add radiobutton -label "2" -variable $var1 -value 2
    $which add radiobutton -label "3" -variable $var1 -value 3
    $which add radiobutton -label "4" -variable $var1 -value 4
    $which add separator
    $which add radiobutton -label Solid -variable $var2 -value 0
    $which add radiobutton -label Dash -variable $var2 -value 1
}

proc GridFontMenu {} {
    global grid
    global menu

    $grid(mb) add cascade -label Font -menu $grid(mb).font

    menu $grid(mb).font -tearoff 1 -selectcolor $menu(selectcolor)
    $grid(mb).font add cascade -label "Axes Numbers" \
	-menu $grid(mb).font.numlab
    $grid(mb).font add cascade -label "Axes Label" \
	-menu $grid(mb).font.textlab
    $grid(mb).font add cascade -label "Title" \
	-menu $grid(mb).font.title

    GridCreateFontMenu $grid(mb).font.title \
	grid(title,font) grid(title,size) grid(title,style)
    GridCreateFontMenu $grid(mb).font.textlab \
	grid(textlab,font) grid(textlab,size) grid(textlab,style)
    GridCreateFontMenu $grid(mb).font.numlab \
	grid(numlab,font) grid(numlab,size) grid(numlab,style)
}

proc GridCreateFontMenu {which var1 var2 var3} {
    global grid
    global menu

    menu $which -tearoff 0 -selectcolor $menu(selectcolor)
    $which add radiobutton -label Times -variable $var1 -value times
    $which add radiobutton -label Helvetica -variable $var1 -value helvetica
    $which add radiobutton -label Symbol -variable $var1 -value symbol
    $which add radiobutton -label Courier -variable $var1 -value courier
    $which add separator
    $which add radiobutton -label 9 -variable $var2 -value 9
    $which add radiobutton -label 10 -variable $var2 -value 10
    $which add radiobutton -label 12 -variable $var2 -value 12
    $which add radiobutton -label 14 -variable $var2 -value 14
    $which add radiobutton -label 18 -variable $var2 -value 18
    $which add radiobutton -label 24 -variable $var2 -value 24
    $which add radiobutton -label 30 -variable $var2 -value 30
    $which add radiobutton -label 36 -variable $var2 -value 36
    $which add separator
    $which add radiobutton -label Plain -variable $var3 -value plain
    $which add radiobutton -label Bold -variable $var3 -value bold
    $which add radiobutton -label Italic -variable $var3 -value italic
}

proc GridSaveDialog {} {
    GridSave [SaveFileDialog gridfbox]
}

proc GridSave {filename} {
    global grid

    if {$filename != {}} {
	set file [open $filename w]

	puts $file "global grid"

	# Type
	puts $file "set grid(type) $grid(type)"

	# Grid
	puts $file "set grid(grid) $grid(grid)"
	puts $file "set grid(grid,color) $grid(grid,color)"
	puts $file "set grid(grid,width) $grid(grid,width)"
	puts $file "set grid(grid,style) $grid(grid,style)"

	# Axes
	puts $file "set grid(axes) $grid(axes)"
	puts $file "set grid(axes,color) $grid(axes,color)"
	puts $file "set grid(axes,width) $grid(axes,width)"
	puts $file "set grid(axes,style) $grid(axes,style)"

	# Ticks
	puts $file "set grid(tick) $grid(tick)"
	puts $file "set grid(tick,color) $grid(tick,color)"
	puts $file "set grid(tick,width) $grid(tick,width)"
	puts $file "set grid(tick,style) $grid(tick,style)"

	# Border
	puts $file "set grid(border) $grid(border)"
	puts $file "set grid(border,color) $grid(border,color)"
	puts $file "set grid(border,width) $grid(border,width)"
	puts $file "set grid(border,style) $grid(border,style)"

	# Rotate Text
	puts $file "set grid(labelup) $grid(labelup)"

	# Numlab
	puts $file "set grid(numlab) $grid(numlab)"
	puts $file "set grid(numlab,font)  $grid(numlab,font)"
	puts $file "set grid(numlab,style) $grid(numlab,style)"
	puts $file "set grid(numlab,size)  $grid(numlab,size)"
	puts $file "set grid(numlab,color) $grid(numlab,color)"

	# Textlab
	puts $file "set grid(textlab) $grid(textlab)"
	puts $file "set grid(textlab,def1) $grid(textlab,def1)"
	puts $file "set grid(label1) \"$grid(label1)\" "
	puts $file "set grid(textlab,gap1) $grid(textlab,gap1)"

	puts $file "set grid(textlab,def2) $grid(textlab,def2)"
	puts $file "set grid(label2) \"$grid(label2)\" "
	puts $file "set grid(textlab,gap2) $grid(textlab,gap2)"

	puts $file "set grid(textlab,font)  $grid(textlab,font)"
	puts $file "set grid(textlab,style) $grid(textlab,style)"
	puts $file "set grid(textlab,size)  $grid(textlab,size)"
	puts $file "set grid(textlab,color) $grid(textlab,color)"

	# Title
	puts $file "set grid(title) $grid(title)"
	puts $file "set grid(title,def) $grid(title,def)"
	puts $file "set grid(title,text) \"$grid(title,text)\" "

	puts $file "set grid(title,font)  $grid(title,font)"
	puts $file "set grid(title,style) $grid(title,style)"
	puts $file "set grid(title,size)  $grid(title,size)"
	puts $file "set grid(title,color) $grid(title,color)"

	# Format
	puts $file "set grid(format) $grid(format)"

	close $file
    }
}

proc GridLoadDialog {} {
    GridLoad [OpenFileDialog gridfbox]
}

proc GridLoad {filename} {
    if {$filename != {}} {
	source $filename
    }
}

proc GridStripComma {str} {
    # strip ','
    set t ""
    regsub -all "," "$str" " " t
    return $t
}

proc ProcessGridCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global grid
    switch -- [string tolower [lindex $var $i]] {
	load {
	    incr i
	    GridLoad [lindex $var $i]
	    GridUpdate
	}
	save {
	    incr i
	    GridSave [lindex $var $i]
	}

	yes -
	true -
	1 -
	no -
	false -
	0 {
	    set grid(view) [FromYesNo [lindex $var $i]]
	    GridUpdate
	}
	default {
	    set grid(view) 1
	    GridUpdate
	    incr i -1
	}
    }
}