package provide DS9 1.0

proc CenterCurrentFrame {} {
    global current

    CenterFrame $current(frame)
}

proc CenterAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	CenterFrame $f
    }
}

proc CenterFrame {which} {
    if {$which != ""} {
	SetWatchCursor
	$which center
	UpdateGraphXAxis
	UnsetWatchCursor

	UpdatePanZoomMenu
	UpdatePanZoomDialog
    }
}

proc Pan {x y sys} {
    global current

    if {$current(frame) != ""} {
	if {$sys == ""} {
	    set sys image
	}
	SetWatchCursor
	$current(frame) pan $sys $x $y
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc PanTo {x y sys} {
    global current

    if {$current(frame) != ""} {
	if {$sys == ""} {
	    set sys image
	}
	SetWatchCursor
	$current(frame) pan to $sys $x $y
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonPan {which x y} {
    global pan

    switch $pan(mode) {
	click {}
	drag {
	    $which pan motion begin $x $y
	}
	panzoom {}
    }
}

proc MotionPan {which x y} {
    global pan

    switch $pan(mode) {
	click {}
	drag {
	    $which pan motion $x $y
	}
	panzoom {}
    }
}

proc ReleasePan {which x y} {
    global pan
    global current

    switch $pan(mode) {
	click {
	    SetWatchCursor
	    $which pan to canvas $x $y
	}
	drag {
	    $which pan motion end $x $y

	    # we need to delay setting the cursor here, since set cursor causes
	    # an update, so the display will flash

	    SetWatchCursor
	}
	panzoom {
	    if {$pan(panzoom,last) != " $x $y "} {
		set pan(panzoom,state) 1
	    }
	    SetWatchCursor
	    switch $pan(panzoom,state) {
		1 {
		    $which pan to canvas $x $y
		    $which update
		    set c [$which get cursor canvas]
		    $which warp to $c
		    set pan(panzoom,last) $c

		    set z [$current(frame) get zoom]
		    if {$z < 2} {
			set pan(panzoom,state) 2
		    } elseif {$z < 4} {
			set pan(panzoom,state) 3
		    } elseif {$z < 8} {
			set pan(panzoom,state) 4
		    } else {
			set pan(panzoom,state) 5
		    }
		}
		2 {
		    $which zoom to 2 about canvas $x $y
		    set pan(panzoom,state) 3
		}
		3 {
		    $which zoom to 4 about canvas $x $y
		    set pan(panzoom,state) 4
		}
		4 {
		    $which zoom to 8 about canvas $x $y
		    set pan(panzoom,state) 5
		}
		5 {
		    $which zoom to 1 about canvas $x $y
		    set pan(panzoom,state) 2
		}
	    }
	    if {$which == $current(frame)} {
		set current(zoom) [$current(frame) get zoom]
	    }
	}
    }
    UpdateGraphXAxis
    UpdatePanZoomDialog
    UnsetWatchCursor
}

proc ZoomToFit {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) zoom to fit
	set current(zoom) [$current(frame) get zoom]
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeZoom {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) zoom to $current(zoom)
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc Zoom {value} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) zoom $value
	set current(zoom) [$current(frame) get zoom]
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonZoom {which x y} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$which zoom 2 about canvas $x $y
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ShiftZoom {which} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$which zoom .5
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeOrient {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) orient $current(orient)
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc Rotate {value} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) rotate $value
	set current(rotate) [$current(frame) get rotate]
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeRotate {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) rotate to $current(rotate)
	UpdateGraphXAxis
	UpdatePanZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonRotate {which x y} {
    global rotate

    $which rotate motion begin
    set rotate(x) $x
}

proc MotionRotate {which x y} {
    global panZoom
    global rotate
    global cursor

    $which rotate motion [expr double($rotate(x)-$x)/$panZoom(speed) * 180.]
}

proc ReleaseRotate {which x y} {
    global current

    $which rotate motion end
    set current(rotate) [$which get rotate fixed]

    # we need to delay setting the cursor here, since set cursor causes
    # an update, so the display will flash

    SetWatchCursor
    UpdateGraphXAxis
    UpdatePanZoomDialog
    UnsetWatchCursor
}

proc PanZoomDialog {} {
    global panZoom
    global menu
    global ds9

    # see if we already have a window visible

    if [winfo exist $panZoom(top)] {
	raise $panZoom(top)
	return
    }

    set w $panZoom(top)
    set title "Pan Zoom Rotate"

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PanZoomDestroyDialog

    $w configure -menu $panZoom(mb)

    menu $panZoom(mb) -tearoff 0
    $panZoom(mb) add cascade -label File -menu $panZoom(mb).file
    $panZoom(mb) add cascade -label Edit -menu $panZoom(mb).edit
    $panZoom(mb) add cascade -label Coordinate -menu $panZoom(mb).coord
    $panZoom(mb) add cascade -label Pan -menu $panZoom(mb).pan
    $panZoom(mb) add cascade -label Zoom -menu $panZoom(mb).zoom
    $panZoom(mb) add cascade -label Orient -menu $panZoom(mb).orient
    $panZoom(mb) add cascade -label Rotate -menu $panZoom(mb).rotate

    menu $panZoom(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).file add command -label "Apply" -command PanZoomApplyDialog
    $panZoom(mb).file add separator
    $panZoom(mb).file add command -label "Close" -command PanZoomDestroyDialog

    menu $panZoom(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).edit add command -label "Copy" -state disabled
    $panZoom(mb).edit add command -label "Paste" -state disabled
    $panZoom(mb).edit add command -label "Clear" -state disabled

    menu $panZoom(mb).coord -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).coord add radiobutton -label "Equatorial B1950" \
	-variable panZoom(system) -value fk4 \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Equatorial J2000" \
	-variable panZoom(system) -value fk5 \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "ICRS" \
	-variable panZoom(system) -value icrs \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Galactic" \
	-variable panZoom(system) -value galactic \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Ecliptic" \
	-variable panZoom(system) -value ecliptic \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add separator
    $panZoom(mb).coord add radiobutton -label "Linear" \
	-variable panZoom(system) -value linear \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add separator
    $panZoom(mb).coord add radiobutton -label "Detector" \
	-variable panZoom(system) -value detector \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Amplifier" \
	-variable panZoom(system) -value Amplifier \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Physical" \
	-variable panZoom(system) -value physical \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Image" \
	-variable panZoom(system) -value image \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add separator
    $panZoom(mb).coord add radiobutton -label "Degrees" \
	-variable panZoom(format) -value degrees \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Sexagesimal" \
	-variable panZoom(format) -value sexagesimal \
	-command PanZoomDialogCoord

    menu $panZoom(mb).pan -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).pan add command -label "Center" -command CenterCurrentFrame

    menu $panZoom(mb).zoom -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).zoom add checkbutton -label "Align to Equatorial WCS" \
	-variable wcs(align) -command AlignWCSFrame
    $panZoom(mb).zoom add separator
    $panZoom(mb).zoom add command -label "Zoom In" -command "Zoom 2"
    $panZoom(mb).zoom add command -label "Zoom Out" -command "Zoom .5"
    $panZoom(mb).zoom add separator
    $panZoom(mb).zoom add command -label "Zoom to Fit Frame" \
	-command "ZoomToFit"
    $panZoom(mb).zoom add separator
    $panZoom(mb).zoom add radiobutton -label "1/16" \
	-variable current(zoom) -value 0.0625 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "1/8" \
	-variable current(zoom) -value 0.125 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "1/4" \
	-variable current(zoom) -value 0.25 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "1/2" \
	-variable current(zoom) -value 0.5 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "1" \
	-variable current(zoom) -value 1 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "2" \
	-variable current(zoom) -value 2 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "4" \
	-variable current(zoom) -value 4 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "8" \
	-variable current(zoom) -value 8 -command ChangeZoom
    $panZoom(mb).zoom add radiobutton -label "16" \
	-variable current(zoom) -value 16 -command ChangeZoom

    menu $panZoom(mb).orient -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).orient add radiobutton  -label "None" \
	-variable current(orient) -value none -command ChangeOrient
    $panZoom(mb).orient add radiobutton  -label "Invert X" \
	-variable current(orient) -value x -command ChangeOrient
    $panZoom(mb).orient add radiobutton -label "Invert Y" \
	-variable current(orient) -value y -command ChangeOrient
    $panZoom(mb).orient add radiobutton -label "Invert X&Y" \
	-variable current(orient) -value xy -command ChangeOrient

    menu $panZoom(mb).rotate -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).rotate add radiobutton -label "0 deg" \
	-variable current(rotate) -value 0 -command ChangeRotate
    $panZoom(mb).rotate add radiobutton -label "90 deg" \
	-variable current(rotate) -value 90 -command ChangeRotate
    $panZoom(mb).rotate add radiobutton -label "180 deg" \
	-variable current(rotate) -value 180 -command ChangeRotate
    $panZoom(mb).rotate add radiobutton -label "270 deg" \
	-variable current(rotate) -value 270 -command ChangeRotate

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.zoomtitle -text "Zoom"
    entry $w.param.zoomvalue -textvariable panZoom(zoom) -width 14 \
	-relief sunken -bd 2

    label $w.param.rottitle -text "Rotate"
    entry $w.param.rotvalue -textvariable panZoom(rotate) -width 14 \
	-relief sunken -bd 2
    label $w.param.rottitle2 -text "degrees"

    label $w.param.pantitle -text "Pan"
    set panZoom(xname) [label $w.param.panxtitle -text "" -width 1]
    entry $w.param.panx -textvariable panZoom(x) -width 14 \
	-relief sunken -bd 2
    set panZoom(yname) [label $w.param.panytitle -text "" -width 1]
    entry $w.param.pany -textvariable panZoom(y) -width 14 \
	-relief sunken -bd 2
    label $w.param.pansystem -textvariable panZoom(system) \
	-width 10 -relief groove -bd 2
    
    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4

    grid $w.param.zoomtitle -row 0 -column 0 -columnspan 2 -padx 4 -sticky w
    grid $w.param.zoomvalue -row 0 -column 2 -padx 4 -sticky w

    grid $w.param.rottitle -row 1 -column 0 -columnspan 2 -padx 4 -sticky w
    grid $w.param.rotvalue -row 1 -column 2 -padx 4 -sticky w
    grid $w.param.rottitle2 -row 1 -column 3 -columnspan 2 -padx 4 -sticky w

    grid $w.param.pantitle -row 2 -column 0 -padx 4 -sticky w
    grid $panZoom(xname) -row 2 -column 1 -padx 4 -sticky w
    grid $w.param.panx -row 2 -column 2 -padx 4 -sticky w
    grid $panZoom(yname) -row 2 -column 3 -padx 4 -sticky w
    grid $w.param.pany -row 2 -column 4 -padx 4 -sticky w
    grid $w.param.pansystem -row 2 -column 5 -padx 4 -sticky w

    button $w.buttons.apply -text "Apply" -command PanZoomApplyDialog
    button $w.buttons.close -text "Close" -command PanZoomDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    $w.param.zoomvalue select range 0 end

    UpdatePanZoomDialog
}

proc PanZoomApplyDialog {} {
    global current
    global panZoom

    if {$current(frame) != ""} {
	set current(zoom) $panZoom(zoom)
	set current(rotate) $panZoom(rotate)

	SetWatchCursor
	$current(frame) zoom to $current(zoom) about \
	    $panZoom(system) $panZoom(x) $panZoom(y) 
	$current(frame) rotate to $current(rotate)
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc PanZoomDestroyDialog {} {
    global panZoom

    destroy $panZoom(top)
    destroy $panZoom(mb)

    unset panZoom(x)
    unset panZoom(y)
    unset panZoom(xname)
    unset panZoom(yname)
    unset panZoom(zoom)
    unset panZoom(rotate)
}

proc PanZoomDialogCoord {} {
    global panZoom
    global current

    if {$current(frame) != ""} {
	set coord \
	    [$current(frame) get cursor $panZoom(system) $panZoom(format)]
	set panZoom(x) [lindex $coord 0]
	set panZoom(y) [lindex $coord 1]
    } else {
	set panZoom(x) {}
	set panZoom(y) {}
    }

    switch $panZoom(system) {
	fk4 -
	fk5 -
	icrs {
	    $panZoom(xname) configure -text "a" -font symbol
	    $panZoom(yname) configure -text "d" -font symbol
	}
	galactic {
	    $panZoom(xname) configure -text "l" \
		-font {-family times -slant italic}
	    $panZoom(yname) configure -text "b" \
		-font {-family times -slant italic}
	}
	ecliptic {
	    $panZoom(xname) configure -text "l" -font symbol
	    $panZoom(yname) configure -text "b" -font symbol
	}
	default {
	    $panZoom(xname) configure -text "X" -font {helvetica 9}
	    $panZoom(yname) configure -text "Y" -font {helvetica 9}
	}
    }
}

proc UpdatePanZoomDialog {} {
    global panZoom
    global current

    if {$current(frame) != ""} {
	global debug
	if {$debug(tcl,update)} {
	    puts "UpdatePanZoomDialog"
	}

	if {[winfo exist $panZoom(top)]} {
	    if {$current(frame) != ""} {
		set panZoom(zoom) [$current(frame) get zoom]
		set panZoom(rotate) [$current(frame) get rotate]
	    } else {
		set panZoom(zoom) {}
		set panZoom(rotate) {}
	    }

	    UpdateCoordMenu $panZoom(mb).coord 0 0 1
	    AdjustCoordSystem panZoom(system)
	    PanZoomDialogCoord
	}
    }
}

proc UpdatePanZoomMenu {} {
    global current
    global wcs
    global buttons
    global ds9
    
    global debug
    if {$debug(tcl,update)} {
	puts "UpdatePanZoomMenu"
    }

    if {$current(frame) != ""} {
	$ds9(mb) entryconfig Zoom -state normal
	$buttons(name).major.zoom configure -state normal

	if {[$current(frame) has fits]} {
	    $ds9(mb).zoom entryconfig "Zoom to Fit Frame" -state normal
	} else {
	    $ds9(mb).zoom entryconfig "Zoom to Fit Frame" -state disabled
	}

	set current(zoom) [$current(frame) get zoom]
	set current(rotate) [$current(frame) get rotate]
	set current(orient) [$current(frame) get orient]
	set wcs(align) [$current(frame) get wcs align]
    } else {
	$ds9(mb) entryconfig Zoom -state disabled
	$buttons(name).major.zoom configure -state disabled

	set current(zoom) 1
	set current(rotate) 0
	set current(orient) none
	set wcs(align) 0
    }
}

proc ProcessPanCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    switch -- [string tolower [lindex $var $i]] {
	to {
	    set sys [lindex $var [expr $i+3]]
	    if {$sys == ""} {
		set sys physical
	    }
	    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] $sys 
	    incr i 3
	}
	default {
	    set sys [lindex $var [expr $i+2]]
	    if {$sys == ""} {
		set sys physical
	    }
	    Pan [lindex $var $i] [lindex $var [expr $i+1]] $sys 
	    incr i 2
	}
    }
}

proc ProcessZoomCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current

    switch -- [string tolower [lindex $var $i]] {
	to {
	    switch -- [string tolower [lindex $var [expr $i+1]]] {
		fit {ZoomToFit}
		default {
		    set current(zoom) [lindex $var [expr $i+1]]
		    ChangeZoom
		}
	    }
	    incr i 1
	}
	default {Zoom [lindex $var $i]}
    }
}

proc ProcessOrientCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    set current(orient) [string tolower [lindex $var $i]]
    ChangeOrient
}

proc ProcessRotateCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    switch -- [string tolower [lindex $var $i]] {
	to {
	    set current(rotate) [lindex $var [expr $i+1]]
	    ChangeRotate
	    incr i
	}
	default {Rotate [lindex $var $i]}
    }
}

