package provide DS9 1.0

proc ClearPrefs {} {
    global prefs
    if {[tk_messageBox -type okcancel -default cancel \
	     -message "Clear Preferences?"] == "ok"} {
	catch {file delete -force $prefs(file)}
	catch {file delete -force $prefs(file,alt)}
    }
}

proc UpdatePrefsMenu {} {
    global prefs
    global ds9

    UpdateMarkerFormatMenu prefs(marker,format) prefs(marker,coord) \
	prefs(marker,coordformat) $ds9(mb).prefs.region
}

proc LoadPrefs {} {
    global prefs
    global ds9

    if {[file exists $prefs(file)]} {
	catch {source $prefs(file)}
    } elseif {[file exists $prefs(file,alt)]} {
	catch {source $prefs(file,alt)}
    }

    # now fix any backward compatibility issues

    global wcs
    if {$wcs(format)=="hms"} {
	set wcs(format) sexagesimal
    }

    global coord
    if {$coord(format)=="hms"} {
	set coord(format) sexagesimal
    }

    global rectangle
    global box
    if {[info exists rectangle(width)]} {
	set box(width) $rectangle(width)
    }
    if {[info exists rectangle(height)]} {
	set box(height) $rectangle(height)
    }

    global bin
    if {[info exists prefs(blockfunction)]} {
	set prefs(binfunction) $prefs(blockfunction)
	set bin(function) $prefs(blockfunction)
	set prefs(binfactor) $prefs(blockfactor)
	set bin(factor) $prefs(blockfactor)
	set prefs(binbuffersize) $prefs(blockbuffersize)
	set bin(buffersize) $prefs(blockbuffersize)
    }

    global current
    global scale
    if {[info exists current(scale)]} {
	set scale(type) $current(scale)
    }

    global help
    if {$help(ref)=="http://hea-www.harvard.edu/RD/ds9/doc/ref/ref.html"} {
	set help(ref) "http://hea-www.harvard.edu/RD/ds9/doc/v2.1/ref/ref.html"
    }
    if {$help(user)=="http://hea-www.harvard.edu/RD/ds9/doc/user/user.html"} {
	set help(user) \
	    "http://hea-www.harvard.edu/RD/ds9/doc/v2.1/user/user.html"
    }
}

proc SavePrefs {} {
    global prefs

    # open prefs file

    set file [open $prefs(file) w]
    if {$file == ""} {
	tk_messageBox -message \
	    "An Error as occured while saving preferences." \
	    -type ok -icon error
	return
    }

    # general prefs

    global ds9
    puts $file "global ds9"

    global prefs
    puts $file "global prefs"

    puts $file ""
    puts $file "\# PS prefs"
    global ps
    puts $file "global ps"
    puts $file "set prefs(ps,orient) $prefs(ps,orient)"
    puts $file "set ps(orient) $prefs(ps,orient)"
    puts $file "set prefs(ps,scale) $prefs(ps,scale)"
    puts $file "set ps(scale) $prefs(ps,scale)"
    puts $file "set prefs(ps,size) $prefs(ps,size)"
    puts $file "set ps(size) $prefs(ps,size)"
    puts $file "set prefs(ps,width) $prefs(ps,width)"
    puts $file "set ps(width) $prefs(ps,width)"
    puts $file "set prefs(ps,height) $prefs(ps,height)"
    puts $file "set ps(height) $prefs(ps,height)"

    puts $file ""
    puts $file "\# Edit prefs"
    puts $file "set prefs(ds9,mode) $prefs(ds9,mode)"
    puts $file "set ds9(mode) $prefs(ds9,mode)"
    global wcs
    puts $file "global wcs"
    puts $file "set prefs(wcs,system) $prefs(wcs,system)"
    puts $file "set wcs(system) $prefs(wcs,system)"
    puts $file "set prefs(wcs,format) $prefs(wcs,format)"
    puts $file "set wcs(format) $prefs(wcs,format)"

    puts $file ""
    puts $file "\# Frame prefs"
    global canvas
    puts $file "global canvas"
    puts $file "set canvas(auto) $canvas(auto)"
    puts $file "set canvas(size) $canvas(size)"
    global tile
    puts $file "global tile"
    puts $file "set prefs(tile,mode) $prefs(tile,mode)"
    puts $file "set tile(mode) $prefs(tile,mode)"
    global blink
    puts $file "global blink"
    puts $file "set prefs(blink,interval) $prefs(blink,interval)"
    puts $file "set blink(interval) $prefs(blink,interval)"

    puts $file ""
    puts $file "\# Bin prefs"
    global bin
    puts $file "global bin"
    puts $file "set prefs(binfunction) $prefs(binfunction)"
    puts $file "set bin(function) $prefs(binfunction)"
    puts $file "set prefs(binfactor) $prefs(binfactor)"
    puts $file "set bin(factor) $prefs(binfactor)"
    puts $file "set prefs(binbuffersize) $prefs(binbuffersize)"
    puts $file "set bin(buffersize) $prefs(binbuffersize)"

    puts $file ""
    puts $file "\# Zoom prefs"
    global current
    puts $file "global current"
    global wcs
    puts $file "global wcs"
    puts $file "set prefs(wcsalign) $prefs(wcsalign)"
    puts $file "set wcs(align) $prefs(wcsalign)"
    puts $file "set prefs(zoom) $prefs(zoom)"
    puts $file "set current(zoom) $prefs(zoom)"
    puts $file "set prefs(orient) $prefs(orient)"
    puts $file "set current(orient) $prefs(orient)"
    puts $file "set prefs(rotate) $prefs(rotate)"
    puts $file "set current(rotate) $prefs(rotate)"

    puts $file ""
    puts $file "\# Scale prefs"
    global current
    puts $file "global current"
    global scale
    puts $file "global scale"
    puts $file "set prefs(scale) $prefs(scale)"
    puts $file "set scale(type) $prefs(scale)"
    puts $file "set prefs(scalemode) $prefs(scalemode)"
    puts $file "set scale(mode) $prefs(scalemode)"
    puts $file "set prefs(scale,datasec) $prefs(scale,datasec)"
    puts $file "set scale(datasec) $prefs(scale,datasec)"

    puts $file ""
    puts $file "\# MinMax prefs"
    global minmax
    puts $file "global minmax"
    puts $file "set prefs(minmaxmode) $prefs(minmaxmode)"
    puts $file "set minmax(mode) $prefs(minmaxmode)"
    puts $file "set prefs(minmaxsample) $prefs(minmaxsample)"
    puts $file "set minmax(sample) $prefs(minmaxsample)"

    puts $file ""
    puts $file "\# Color prefs"
    global colorbar
    puts $file "global colorbar"
    puts $file "set prefs(colorbarmap) $prefs(colorbarmap)"
    puts $file "set colorbar(map) $prefs(colorbarmap)"
    puts $file "set prefs(colorbarinvert) $prefs(colorbarinvert)"
    puts $file "set colorbar(invert) $prefs(colorbarinvert)"

    puts $file ""
    puts $file "\# Region prefs"
    global marker
    puts $file "global marker"
    puts $file "set prefs(shape) \{$prefs(shape)\}"
    puts $file "set marker(shape) \{$prefs(shape)\}"
    global circle
    puts $file "global circle"
    puts $file "set circle(radius) $circle(radius)"
    global projection
    puts $file "global projection"
    puts $file "set projection(thick) $projection(thick)"
    global annulus
    puts $file "global annulus"
    puts $file "set annulus(inner) $annulus(inner)"
    puts $file "set annulus(outer) $annulus(outer)"
    puts $file "set annulus(annuli) $annulus(annuli)"
    global ellipse
    puts $file "global ellipse"
    puts $file "set ellipse(radius1) $ellipse(radius1)"
    puts $file "set ellipse(radius2) $ellipse(radius2)"
    global ellipseannulus
    puts $file "global ellipseannulus"
    puts $file "set ellipseannulus(radius1) $ellipseannulus(radius1)"
    puts $file "set ellipseannulus(radius2) $ellipseannulus(radius2)"
    puts $file "set ellipseannulus(radius3) $ellipseannulus(radius3)"
    puts $file "set ellipseannulus(annuli) $ellipseannulus(annuli)"
    global box
    puts $file "global box"
    puts $file "set box(width) $box(width)"
    puts $file "set box(height) $box(height)"
    global boxannulus
    puts $file "global boxannulus"
    puts $file "set boxannulus(size1) $boxannulus(size1)"
    puts $file "set boxannulus(size2) $boxannulus(size2)"
    puts $file "set boxannulus(size3) $boxannulus(size3)"
    puts $file "set boxannulus(annuli) $boxannulus(annuli)"
    global panda
    puts $file "global panda"
    puts $file "set panda(inner) $panda(inner)"
    puts $file "set panda(outer) $panda(outer)"
    puts $file "set panda(annuli) $panda(annuli)"
    puts $file "set panda(ang1) $panda(ang1)"
    puts $file "set panda(ang2) $panda(ang2)"
    puts $file "set panda(angnum) $panda(angnum)"
    global marker
    puts $file "global marker"
    puts $file "set prefs(marker,format) $prefs(marker,format)"
    puts $file "set marker(format) $prefs(marker,format)"
    puts $file "set prefs(marker,strip) $prefs(marker,strip)"
    puts $file "set marker(strip) $prefs(marker,strip)"
    puts $file "set prefs(marker,coord) $prefs(marker,coord)"
    puts $file "set marker(coord) $prefs(marker,coord)"
    puts $file "set prefs(marker,coordformat) $prefs(marker,coordformat)"
    puts $file "set marker(coordformat) $prefs(marker,coordformat)"

    puts $file ""
    puts $file "\# Analysis prefs"
    global graphHorz
    puts $file "global graphHorz"
    puts $file "set graphHorz(grid) $graphHorz(grid)"
    puts $file "set graphHorz(log) $graphHorz(log)"
    global graphVert
    puts $file "global graphVert"
    puts $file "set graphVert(grid) $graphVert(grid)"
    puts $file "set graphVert(log) $graphVert(log)"
    global contour
    puts $file "global contour"
    puts $file "set prefs(contour,method) $prefs(contour,method)"
    puts $file "set contour(method) $prefs(contour,method)"
    puts $file "set prefs(contour,color) $prefs(contour,color)"
    puts $file "set contour(color) $prefs(contour,color)"
    puts $file "set prefs(contour,width) $prefs(contour,width)"
    puts $file "set contour(width) $prefs(contour,width)"
    global dss
    puts $file "global dss"
    puts $file "set dss(server) \{$dss(server)\}"
    global nsvr
    puts $file "global nsvr"
    puts $file "set nsvr(server) \{$nsvr(server)\}"
    global vo
    puts $file "global vo"
    puts $file "set vo(server) \{$vo(server)\}"

    puts $file "set prefs(analysis,user) \{$prefs(analysis,user)\}"
    puts $file "set ds9(analysis,user)  \{$prefs(analysis,user)\}"
    puts $file "set prefs(analysis,user2)  \{$prefs(analysis,user2)\}"
    puts $file "set ds9(analysis,user2)  \{$prefs(analysis,user2)\}"
    puts $file "set prefs(analysis,user3)  \{$prefs(analysis,user3)\}"
    puts $file "set ds9(analysis,user3)  \{$prefs(analysis,user3)\}"
    puts $file "set prefs(analysis,user4)  \{$prefs(analysis,user4)\}"
    puts $file "set ds9(analysis,user4)  \{$prefs(analysis,user4)\}"

    global analysis
    puts $file "global analysis"
    puts $file "set prefs(analysis,log) $prefs(analysis,log)"
    puts $file "set analysis(log) $prefs(analysis,log)"

    puts $file ""
    puts $file "\# Analysis Plot prefs"
    puts $file "set prefs(ap,unique) $prefs(ap,unique)"

    puts $file "set prefs(ap,grid) $prefs(ap,grid)"
    puts $file "set prefs(ap,grid,log) $prefs(ap,grid,log)"

    puts $file "set prefs(ap,discrete) $prefs(ap,discrete)"
    puts $file "set prefs(ap,discrete,symbol) $prefs(ap,discrete,symbol)"
    puts $file "set prefs(ap,discrete,color) $prefs(ap,discrete,color)"

    puts $file "set prefs(ap,linear) $prefs(ap,linear)"
    puts $file "set prefs(ap,linear,width) $prefs(ap,linear,width)"
    puts $file "set prefs(ap,linear,color) $prefs(ap,linear,color)"
    puts $file "set prefs(ap,linear,dash) $prefs(ap,linear,dash)"

    puts $file "set prefs(ap,step) $prefs(ap,step)"
    puts $file "set prefs(ap,step,width) $prefs(ap,step,width)"
    puts $file "set prefs(ap,step,color) $prefs(ap,step,color)"
    puts $file "set prefs(ap,step,dash) $prefs(ap,step,dash)"

    puts $file "set prefs(ap,quadratic) $prefs(ap,quadratic)"
    puts $file "set prefs(ap,quadratic,width) $prefs(ap,quadratic,width)"
    puts $file "set prefs(ap,quadratic,color) $prefs(ap,quadratic,color)"
    puts $file "set prefs(ap,quadratic,dash) $prefs(ap,quadratic,dash)"

    puts $file "set prefs(ap,error,width) $prefs(ap,error,width)"
    puts $file "set prefs(ap,error,color) $prefs(ap,error,color)"
    puts $file "set prefs(ap,error,style) $prefs(ap,error,style)"

    puts $file "set prefs(ap,titleFont) $prefs(ap,titleFont)"
    puts $file "set prefs(ap,titleSize) $prefs(ap,titleSize)"
    puts $file "set prefs(ap,titleStyle) $prefs(ap,titleStyle)"

    puts $file "set prefs(ap,textlabFont) $prefs(ap,textlabFont)"
    puts $file "set prefs(ap,textlabSize) $prefs(ap,textlabSize)"
    puts $file "set prefs(ap,textlabStyle) $prefs(ap,textlabStyle)"

    puts $file "set prefs(ap,numlabFont) $prefs(ap,numlabFont)"
    puts $file "set prefs(ap,numlabSize) $prefs(ap,numlabSize)"
    puts $file "set prefs(ap,numlabStyle) $prefs(ap,numlabStyle)"

    puts $file ""
    puts $file "\# Help prefs"
    global help
    puts $file "global help"
    puts $file "set help(ref) \{$help(ref)\}"
    puts $file "set help(user) \{$help(user)\}"
    puts $file "set help(app) $help(app)"

    puts $file ""
    puts $file "\# view prefs"
    global info
    puts $file "global info"
    puts $file "set info(view) $info(view)"
    global pan
    puts $file "global pan"
    puts $file "set pan(view) $pan(view)"
    global mag
    puts $file "global mag"
    puts $file "set mag(view) $mag(view)"
    global buttons
    puts $file "global buttons"
    puts $file "set buttons(view) $buttons(view)"
    global colorbar
    puts $file "global colorbar"
    puts $file "set colorbar(view) $colorbar(view)"

    puts $file ""
    puts $file "\# Infomation Panel prefs"
    global info
    puts $file "global info"
    puts $file "set info(filename) $info(filename)"
    puts $file "set info(object) $info(object)"
    puts $file "set info(minmax) $info(minmax)"
    puts $file "set info(wcs) $info(wcs)"
    puts $file "set info(detector) $info(detector)"
    puts $file "set info(amplifier) $info(amplifier)"
    puts $file "set info(physical) $info(physical)"
    puts $file "set info(image) $info(image)"
    puts $file "set info(frame) $info(frame)"

    puts $file ""
    puts $file "\# Mosaic prefs"
    global mosaic
    puts $file "global mosaic"
    puts $file "set mosaic(type) $mosaic(type)"
    puts $file "set mosaic(fast) $mosaic(fast)"

    puts $file ""
    puts $file "\# Panner prefs"
    global pan
    puts $file "global pan"
    puts $file "set pan(imagecompass) $pan(imagecompass)"
    puts $file "set pan(wcscompass) $pan(wcscompass)"
    puts $file "set pan(wcstanpoint) $pan(wcstanpoint)"

    puts $file ""
    puts $file "\# Magnifier prefs"
    global mag
    puts $file "global mag"
    puts $file "set mag(mode) $mag(mode)"
    puts $file "set mag(region) $mag(region)"
    puts $file "set mag(cursor) $mag(cursor)"
    puts $file "set mag(zoom) $mag(zoom)"

    puts $file ""
    puts $file "\# Colorbar prefs"
    global colorbar
    puts $file "global colorbar"
    puts $file "set colorbar(area) $colorbar(area)"
    puts $file "set colorbar(mode) $colorbar(mode)"

    puts $file ""
    puts $file "\# Dialog prefs"
    global ds9
    puts $file "global ds9"
    puts $file "set ds9(dialog) $ds9(dialog)"

    puts $file ""
    puts $file "\# Pan prefs"
    global pan
    puts $file "global pan"
    puts $file "set pan(mode) $pan(mode)"

    puts $file ""
    puts $file "\# Display Coords prefs"
    global coord
    puts $file "global coord"
    puts $file "set coord(fk4) $coord(fk4)"
    puts $file "set coord(fk5) $coord(fk5)"
    puts $file "set coord(icrs) $coord(icrs)"
    puts $file "set coord(galactic) $coord(galactic)"
    puts $file "set coord(ecliptic) $coord(ecliptic)"
    puts $file "set coord(linear) $coord(linear)"
    puts $file "set coord(detector) $coord(detector)"
    puts $file "set coord(amplifier) $coord(amplifier)"
    puts $file "set coord(physical) $coord(physical)"
    puts $file "set coord(image) $coord(image)"
    puts $file "set coord(format) $coord(format)"
    puts $file "set coord(value) $coord(value)"

    puts $file ""
    puts $file "\# Examine prefs"
    global examine
    puts $file "global examine"
    puts $file "set examine(mode) $examine(mode)"
    puts $file "set examine(zoom) $examine(zoom)"

    puts $file ""
    puts $file "\# URL prefs"
    global http
    global ftp
    puts $file "global http"
    puts $file "global ftp"
    puts $file "set http(log) $http(log)"
    puts $file "set ftp(log) $ftp(log)"

    puts $file ""
    puts $file "\# Misc prefs"
    global ds9
    puts $file "global ds9"
    puts $file "set ds9(cursor) $ds9(cursor)"
    puts $file "set ds9(automarker) $ds9(automarker)"

    # and close

    close $file

    # make sure old prefs files are removed
    catch {file delete -force $prefs(file,alt)}
}
