package provide DS9 1.0

# Print procs

proc Print {} {
    global tcl_platform

    if {[PrintDialog ps]} {
	if {$tcl_platform(platform)!= "windows"} {
	    set w .status
	    DisplayStatusMsg $w "Printing, please wait..."
	    if [catch {PostScript} printError] {
		Error "A printing error has occured: $printError"
	    }
	    DestroyStatusMsg $w
	} else {
	    if [catch {PostScript} printError] {
		Error "A printing error has occured: $printError"
	    }
	}
    }
}

proc PostScript {} {
    global ps
    global canvas
    global ds9

    set options " "

    # Orientation

    switch $ps(orient) {
	portrait {append options " -rotate false"}
	landscape {append options " -rotate true"}
    }

    # Page size

    if {$ps(scale) == "scaled"} {
	switch $ps(size) {
	    letter {PSPageSize 4.25i 5.5i 7.5i 10.i options}
	    legal {PSPageSize 4.25i 7.i 7.5i 13.i options}
	    tabloid {PSPageSize 5.5i 8.5i 10.i 16.i options}
	    poster {PSPageSize 18.i 24.i 35.i 47.i options}
	    a4 {PSPageSize 105m 148.5m 185m 272m options}
	    other {
		if {$ps(width) != "" && $ps(height) != ""} {
		    set x [expr double($ps(width))/2.]
		    set y [expr double($ps(height))/2.]
		    set w [expr $ps(width)-1.]
		    set h [expr $ps(height)-1.]
		    PSPageSize [append x i] [append y i] \
			[append w i] [append h i] options
		}
	    }
	    othermm {
		if {$ps(width) != "" && $ps(height) != ""} {
		    set x [expr double($ps(width))/2.]
		    set y [expr double($ps(height))/2.]
		    set w [expr $ps(width)-1.]
		    set h [expr $ps(height)-1.]
		    PSPageSize [append x m] [append y m] \
			[append w m] [append h m] options
		}
	    }
	}
    }

    # Printer vs File
    
    set channel ""
    if {$ps(dest) == "file" && $ps(filename) != {}} {
	append options " -file \{$ps(filename)\}"
    } else {
	set channel [open "| $ps(cmd)" w]
	append options " -channel $channel"
    }

    # set frame specific postscript options

    foreach f $ds9(frames) {
	$f postscript level $ps(level)
	$f postscript colorspace $ps(color)
	$f postscript interpolate $ps(interpolate)
	$f postscript resolution $ps(resolution)
    }

    # now invoke canvas postscript command

    if [catch {eval $canvas(name) postscript $options} psError] {
	Error "A postscript generation error has occured: $psError"
    }

    if {$channel != ""} {
	close $channel
    }
}

proc PSPageSize {x y w h optname} {
    upvar $optname options
    global ps
    global canvas

    set width [winfo width $canvas(name)]
    set height [winfo height $canvas(name)]
    set wo [string trim $w "icmp"]
    set ho [string trim $h "icmp"]

    switch $ps(orient) {
	portrait {
	    if {[expr double($wo)/$width] < [expr double($ho)/$height]} {
		append options " -pagewidth $w"
	    } else {
		append options " -pageheight $h"
	    }
	}
	landscape {
	    if {[expr double($wo)/$width] > [expr double($ho)/$height]} {
		append options " -pageheight $w"
	    } else {
		append options " -pagewidth $h"
	    }
	}
    }

    append options " -pagex $x -pagey $y"
}

# Print Dialog procs

proc PrintDialog {which} {
    global ed
    global ps
    global menu

    set ed(ok) 0
    set ed(dest) $ps(dest)
    set ed(cmd) $ps(cmd)
    set ed(filename) $ps(filename)
    set ed(color) $ps(color)
    set ed(color2) $ps(color2)
    set ed(level) $ps(level)
    set ed(interpolate) $ps(interpolate)
    set ed(resolution) $ps(resolution)

    set w ".print"

    DialogCreate $w "Print" -borderwidth 2
    frame $w.print -relief groove -borderwidth 2
    frame $w.options -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2

    switch $which {
	ap -
	ps {pack $w.print $w.options $w.buttons -fill x -ipadx 4 -ipady 4}
	txt {pack $w.print $w.buttons -fill x -ipadx 4 -ipady 4}
    }

    # Print

    label $w.print.title1 -text "Print To:"
    radiobutton $w.print.printer -text "Printer" -variable ed(dest) \
	-value printer -selectcolor $menu(selectcolor)
    radiobutton $w.print.file -text "File" -variable ed(dest) \
	-value file -selectcolor $menu(selectcolor)

    label $w.print.title2 -text "Print Command:"
    entry $w.print.cmd -textvariable ed(cmd) -width 30 \
	-relief sunken -bd 2

    label $w.print.title3 -text "File Name:"
    entry $w.print.name -textvariable ed(filename) -width 30 \
	-relief sunken -bd 2

    button $w.print.browse -text "Browse" -command "PrintBrowse ed(filename)"

    grid rowconfigure $w.print 0 -pad 4
    grid rowconfigure $w.print 1 -pad 4
    grid rowconfigure $w.print 2 -pad 4

    global tcl_platform
    if {$tcl_platform(platform)!= "windows"} {
	grid $w.print.title1 -row 0 -column 0 -padx 4 -sticky w
	grid $w.print.printer -row 0 -column 1 -padx 4 -sticky w
	grid $w.print.file -row 0 -column 2 -padx 4 -sticky w

	grid $w.print.title2 -row 1 -column 0 -padx 4 -sticky w
	grid $w.print.cmd -row 1 -column 1 -columnspan 2 -padx 4 -sticky w
    }
    grid $w.print.title3 -row 2 -column 0 -padx 4 -sticky w
    grid $w.print.name -row 2 -column 1 -columnspan 2 -padx 4 -sticky w

    grid $w.print.browse -row 2 -column 3 -padx 4

    # Options

    switch $which {
	txt {}
	ps {
	    label $w.options.title2 -text "Print:"
	    radiobutton $w.options.rgb -text "RGB Color" \
		-variable ed(color) -value rgb \
		-selectcolor $menu(selectcolor)
	    radiobutton $w.options.cmyk -text "CMYK Color" \
		-variable ed(color) -value cmyk \
		-selectcolor $menu(selectcolor)
	    radiobutton $w.options.gray -text "Grayscale" \
		-variable ed(color) -value gray \
		-selectcolor $menu(selectcolor)

	    label $w.options.title4 -text "Postscript:"
	    radiobutton $w.options.level1 -text "Level 1" \
		-variable ed(level) -value 1 \
		-selectcolor $menu(selectcolor) -command "PSLevel $w"
	    radiobutton $w.options.level2 -text "Level 2" \
		-variable ed(level) -value 2 \
		-selectcolor $menu(selectcolor) -command "PSLevel $w"

	    checkbutton $w.options.interpolate -text "Interpolate" \
		-variable ed(interpolate) -selectcolor $menu(selectcolor)

	    label $w.options.title5 -text "Resolution:"

	    frame $w.options.resolution
	    label $w.options.resolution.title -text "dpi"
	    menubutton $w.options.resolution.button -relief raised \
		-menu $w.options.resolution.button.menu \
		-textvariable ed(resolution)
	    menu $w.options.resolution.button.menu \
		-tearoff $menu(tearoff) -selectcolor $menu(selectcolor)
	    $w.options.resolution.button.menu add radiobutton -label "53" \
		-variable ed(resolution) -value 53
	    $w.options.resolution.button.menu add radiobutton -label "72" \
		-variable ed(resolution) -value 72
	    $w.options.resolution.button.menu add radiobutton -label "75" \
		-variable ed(resolution) -value 75
	    $w.options.resolution.button.menu add radiobutton -label "150" \
		-variable ed(resolution) -value 150
	    $w.options.resolution.button.menu add radiobutton -label "300" \
		-variable ed(resolution) -value 300
	    $w.options.resolution.button.menu add radiobutton -label "600" \
		-variable ed(resolution) -value 600
	    pack $w.options.resolution.button $w.options.resolution.title \
		-side left

	    grid rowconfigure $w.options 0 -pad 4
	    grid rowconfigure $w.options 1 -pad 4
	    grid rowconfigure $w.options 2 -pad 4

	    grid $w.options.title2 $w.options.rgb $w.options.cmyk \
		$w.options.gray -padx 4 -sticky w
	    grid $w.options.title4 $w.options.level2 $w.options.level1 \
		-padx 4 -sticky w
	    grid x $w.options.interpolate -padx 4 -sticky w
	    grid $w.options.title5 $w.options.resolution -padx 4 -sticky w

	    PSLevel $w
	}
	ap {
	    label $w.options.title2 -text "Print:"
	    radiobutton $w.options.color -text "Color" \
		-variable ed(color2) -value color \
		-selectcolor $menu(selectcolor)
	    radiobutton $w.options.gray -text "Grayscale" \
		-variable ed(color2) -value gray \
		-selectcolor $menu(selectcolor)
	    radiobutton $w.options.mono -text "Black & White" \
		-variable ed(color2) -value mono \
		-selectcolor $menu(selectcolor)

	    grid rowconfigure $w.options 0 -pad 4

	    grid $w.options.title2 -row 0 -column 0 -padx 4 -sticky w
	    grid $w.options.color -row 0 -column 1 -padx 4 -sticky w
	    grid $w.options.gray -row 0 -column 2 -padx 4 -sticky w
	    grid $w.options.mono -row 0 -column 3 -padx 4 -sticky w
	}
    }


    # Buttons

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok
    DialogDismiss $w

    if {$ed(ok)} {
	set ps(dest) $ed(dest)
	set ps(cmd) $ed(cmd)
	set ps(filename) $ed(filename)
	set ps(color) $ed(color)
	set ps(color2) $ed(color2)
	set ps(level) $ed(level)
	set ps(interpolate) $ed(interpolate)
	set ps(resolution) $ed(resolution)
    }

    set r $ed(ok)
    unset ed
    return $r
}

proc PSLevel {w} {
    global ed
    switch -- $ed(level) {
	1 {
	    $w.options.interpolate configure -state disabled
	    $w.options.cmyk configure -state disabled
	    if {$ed(color)=="cmyk"} {
		set ed(color) rgb
	    }
	}
	2 {
	    $w.options.interpolate configure -state normal
	    $w.options.cmyk configure -state normal
	}
    }
}

proc PrintBrowse {varname} {
    upvar $varname var

    FileLast pssavfbox $var
    set var [SaveFileDialog pssavfbox]
}

proc PageDialog {} {
    global ps

    PageSetupDialog ps(orient) ps(scale) ps(size) ps(width) ps(height)
}

proc PrefsPageDialog {} {
    global prefs

    PageSetupDialog prefs(ps,orient) prefs(ps,scale) prefs(ps,size) \
	prefs(ps,width) prefs(ps,height)
}

proc PageSetupDialog {orientvar scalevar sizevar widthvar heightvar} {
    global ed
    global menu

    upvar $orientvar orient
    upvar $scalevar scale
    upvar $sizevar size
    upvar $widthvar width
    upvar $heightvar height

    set ed(ok) 0
    set ed(orient) $orient
    set ed(scale) $scale
    set ed(size) $size
    set ed(width) $width
    set ed(height) $height

    set w ".page"

    DialogCreate $w "Page Setup" -borderwidth 2
    frame $w.options -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.options $w.buttons -fill x -ipadx 4 -ipady 4

    # Options

    label $w.options.title1 -text "Orientation:"
    radiobutton $w.options.portrait -text "Portrait" \
	-variable ed(orient) \
	-value portrait -selectcolor $menu(selectcolor)
    radiobutton $w.options.landscape -text "Landscape" \
	-variable ed(orient) \
	-value landscape -selectcolor $menu(selectcolor)

    label $w.options.title2 -text "Page Size:"
    radiobutton $w.options.fixed -text "Fixed in Size" \
	-variable ed(scale) \
	-value fixed -selectcolor $menu(selectcolor) \
	-command "PageDialogSize $w"
    radiobutton $w.options.scaled -text "Scale to fit Page" \
	-variable ed(scale) \
	-value scaled -selectcolor $menu(selectcolor) \
	-command "PageDialogSize $w"

    frame $w.options.s -relief groove -borderwidth 2
    radiobutton $w.options.s.letter -text "Letter(8.5 x 11 in)" \
	-variable ed(size) -value letter -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.legal -text "Legal(8.5 x 14 in)" \
	-variable ed(size) \
	-value legal -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.tabloid -text "Tabloid(11 x 17 in)" \
	-variable ed(size) -value tabloid -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.poster -text "Poster(36 x 48 in)" \
	-variable ed(size) -value poster -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.a4 -text "A4(210 x 297 mm)" \
	-variable ed(size) \
	-value a4 -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.other -text "Other (inches)" \
	-variable ed(size) \
	-value other -selectcolor $menu(selectcolor)
    radiobutton $w.options.s.othermm -text "Other (mm)" \
	-variable ed(size) \
	-value othermm -selectcolor $menu(selectcolor)

    frame $w.options.s.wh
    label $w.options.s.wh.title3 -text "Width"
    entry $w.options.s.wh.width -textvariable ed(width) \
	-width 10 -relief sunken -bd 2

    label $w.options.s.wh.title4 -text "Height"
    entry $w.options.s.wh.height -textvariable ed(height) \
	-width 10 -relief sunken -bd 2

    grid rowconfigure $w.options 0 -pad 4
    grid rowconfigure $w.options 1 -pad 4
    grid rowconfigure $w.options 2 -pad 4

    grid $w.options.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.portrait -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.landscape -row 0 -column 2 -padx 4 -sticky w

    grid $w.options.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.scaled -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.fixed -row 1 -column 2 -padx 4 -sticky w

    grid $w.options.s -row 2 -column 1 -padx 4 -sticky w -columnspan 2

    grid rowconfigure $w.options.s 0 -pad 4
    grid rowconfigure $w.options.s 1 -pad 4
    grid rowconfigure $w.options.s 2 -pad 4
    grid rowconfigure $w.options.s 3 -pad 4
    grid rowconfigure $w.options.s 4 -pad 4

    grid $w.options.s.letter -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.s.legal -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.s.tabloid -row 2 -column 1 -padx 4 -sticky w
    grid $w.options.s.poster -row 3 -column 1 -padx 4 -sticky w
    grid $w.options.s.a4 -row 4 -column 1 -padx 4 -sticky w
    grid $w.options.s.other -row 5 -column 1 -padx 4 -sticky w
    grid $w.options.s.othermm -row 6 -column 1 -padx 4 -sticky w

    grid $w.options.s.wh -row 5 -column 2 -padx 4 -sticky w -rowspan 2

    grid rowconfigure $w.options.s.wh 0 -pad 4
    grid rowconfigure $w.options.s.wh 1 -pad 4

    grid $w.options.s.wh.title3 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.s.wh.width -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.s.wh.title4 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.s.wh.height -row 1 -column 1 -padx 4 -sticky w

    # Buttons

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    PageDialogSize $w
    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok 
    DialogDismiss $w

    if {$ed(ok)} {
	set orient $ed(orient)
	set scale $ed(scale)
	set size $ed(size)
	set width $ed(width)
	set height $ed(height)
    }

    set r $ed(ok)
    unset ed
    return $r
}

proc PageDialogSize {w} {
    global ed

    if {$ed(scale) == "scaled"} {
	$w.options.s.letter configure -state normal
	$w.options.s.legal configure -state normal
	$w.options.s.tabloid configure -state normal
	$w.options.s.poster configure -state normal
	$w.options.s.a4 configure -state normal
	$w.options.s.other configure -state normal
	$w.options.s.othermm configure -state normal
    } else {
	$w.options.s.letter configure -state disabled
	$w.options.s.legal configure -state disabled
	$w.options.s.tabloid configure -state disabled
	$w.options.s.poster configure -state disabled
	$w.options.s.a4 configure -state disabled
	$w.options.s.other configure -state disabled
	$w.options.s.othermm configure -state disabled
    }
}

proc ProcessPrintCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ps
    global current

    switch -- [string tolower [lindex $var $i]] {
	destination {incr i; set ps(dest) [lindex $var $i]}
	command {incr i; set ps(cmd) [lindex $var $i]}
	filename {incr i; set ps(filename) [lindex $var $i] }
	palette {incr i; set ps(color) [lindex $var $i] }
	level {incr i; set ps(level) [lindex $var $i] }
	interpolate {incr i; set ps(interpolate) [FromYesNo [lindex $var $i]]}
	resolution {incr i; set ps(resolution) [lindex $var $i] }
	{} {
	    RealizeDS9

	    PostScript
	}

	default {
	    RealizeDS9

	    incr i -1
	    PostScript
	}
    }
}

proc ProcessPageSetupCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ps

    switch -- [string tolower [lindex $var $i]] {
	orientation {incr i; set ps(orient) [lindex $var $i]}
	pagescale {incr i; set ps(scale) [lindex $var $i]}
	pagesize {incr i; set ps(size) [lindex $var $i] }
    }
}

proc SaveAsDialog {} {
    global ds9
    global tcl_platform
    global saveas

    if {![InPath gs]} {
	Error "Unable to locate Ghostscript. Required for this function"
	return
    }

    set res {}
    catch {exec gs -help} res
    if {[lindex $res 2] < 6.50} {
	Error "Ghostscript version 6.50 or higher is required"
	return
    }

    if {[FormatDialog]} {
	switch $saveas(device) {
	    jpeg {set fn "$saveas(base).jpg"}
	    tiff24nc {set fn "$saveas(base).tif"}
	    png16m {set fn "$saveas(base).png"}
	    ppm {set fn "$saveas(base).ppm"}
	}
    } else {
	return
    }

    FileLast saveasfbox $fn
    set fn [SaveFileDialog saveasfbox]
    set saveas(base) [file rootname [file tail $fn]]
    if {$fn != {}} {
	if {$tcl_platform(platform)!= "windows"} {
	    set w .status
	    DisplayStatusMsg $w "Saving image, please wait..."
	    SaveAs $fn
	    DestroyStatusMsg $w
	} else {
	    SaveAs $fn
	}
    }
}

proc SaveAs {fn} {
    global ds9
    global canvas
    global saveas

    # NOTE: does not work for pathnames with spaces...

    set w0 [expr int([winfo width $canvas(name)]*$saveas(resolution)/72.)]
    set h0 [expr int([winfo height $canvas(name)]*$saveas(resolution)/72.)]

    set options "-sDEVICE=$saveas(device)"
    switch $saveas(device) {
	jpeg {append options " -dJPEGQ=$saveas(jpeg,qf)"}
    }
    append options " -g${w0}x${h0} -r$saveas(resolution)"
    set ch [open "| gs -q $options -sOutputFile=$fn -_" w]

    foreach f $ds9(frames) {
	$f postscript level 2
	$f postscript colorspace rgb
	$f postscript resolution 72
    }

    set ww0 [winfo width $canvas(name)]
    set hh0 [winfo height $canvas(name)]
    set options "-channel $ch "
    append options "-pagewidth $ww0 -pageheight $hh0 "
    append options "-pagex [expr $ww0/2] -pagey [expr $hh0/2] "

    if [catch {eval $canvas(name) postscript $options} psError] {
	Error "A $format error has occured"
    }

    catch {close $ch}
}

proc FormatDialog {} {
    global env
    global ed
    global menu
    global saveas

    set w ".format"

    set ed(ok) 0
    set ed(device) $saveas(device)
    set ed(resolution) $saveas(resolution)
    set ed(jpeg,qf) $saveas(jpeg,qf)

    DialogCreate $w "Format" -borderwidth 2
    frame $w.format -relief groove -borderwidth 2
    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.format $w.param $w.buttons \
	-ipadx 4 -ipady 4 -fill x -expand true

    label $w.format.title -text "Format"
    radiobutton $w.format.jpeg -text JPEG -variable ed(device) -value jpeg
    radiobutton $w.format.tiff -text "TIFF RGB 24-bit no compression" \
	-variable ed(device) -value tiff24nc
    radiobutton $w.format.png -text "PNG 24-bit (Portable Network Graphics)" \
	-variable ed(device) -value png16m
    radiobutton $w.format.ppm -text "PPM RGB (Portable Pixmap)" \
	-variable ed(device) -value ppm

    grid $w.format.jpeg -ipadx 4 -sticky w
    grid $w.format.tiff -ipadx 4 -sticky w
    grid $w.format.png -ipadx 4 -sticky w
    grid $w.format.ppm -ipadx 4 -sticky w

    frame $w.param.resolution
    label $w.param.resolution.title -text "Resolution:"
    label $w.param.resolution.title2 -text "dpi"

    menubutton $w.param.resolution.button -relief raised \
	-menu $w.param.resolution.button.menu \
	-textvariable ed(resolution)
    menu $w.param.resolution.button.menu -tearoff 0
    $w.param.resolution.button.menu add radiobutton -label "53" \
	-variable ed(resolution) -value 53
    $w.param.resolution.button.menu add radiobutton -label "72" \
	-variable ed(resolution) -value 72
    $w.param.resolution.button.menu add radiobutton -label "75" \
	-variable ed(resolution) -value 75
    $w.param.resolution.button.menu add radiobutton -label "150" \
	-variable ed(resolution) -value 150
    $w.param.resolution.button.menu add radiobutton -label "300" \
	-variable ed(resolution) -value 300
    $w.param.resolution.button.menu add radiobutton -label "600" \
	-variable ed(resolution) -value 600

    pack $w.param.resolution.title $w.param.resolution.button \
	$w.param.resolution.title2 -padx 4 -side left

    scale $w.param.jpeg  -from 0 -to 100 -length 300 \
	-variable ed(jpeg,qf) -orient horizontal -label "JPEG Quality Factor" \
	-tickinterval 25 -showvalue true -resolution 1

    pack $w.param.resolution $w.param.jpeg -side top \
	-padx 4 -pady 4 -anchor w

    button $w.buttons.ok -text "OK" -default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set saveas(device) $ed(device)
	set saveas(resolution) $ed(resolution)
	set saveas(jpeg,qf) $ed(jpeg,qf)

	destroy $w 
	unset ed
	return 1
    } else {
	destroy $w 
	unset ed
	return 0
    }
}

proc ProcessSaveAsCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global help
    global saveas
    
    if {![InPath gs]} {
	Error "Unable to locate Ghostscript. Required for this function"
	return
    }

    switch -- [string tolower [lindex $var $i]] {
	jpeg {set saveas(device) jpeg}
	tiff {set saveas(device) tiff24nc}
	png {set saveas(device) png16m}
	ppm {set saveas(device) ppm}
    }
    incr i
    SaveAs [lindex $var $i]
}
