#! /bin/sh

# functions

testit () {
    echo "Test $1 $2 $3..."
    xpaset -p ds9 regions format $1
    xpaset -p ds9 regions coord $2
    xpaset -p ds9 regions coordformat $3
    xpaset -p ds9 regions file $4
    xpaset -p ds9 regions save ${4}.out
    xpaset -p ds9 regions deleteall

    o=`diff $4 ${4}.out`
    if [ "$o" = "" ]
    then
	echo "PASSED"
    else
        echo "FAILED"
	echo "$o"
    fi
    rm -f ${4}.out
}

testit2 () {
    echo "Test $1 $2 $3..."
    xpaset -p ds9 regions format $1
    xpaset -p ds9 regions coord $2
    xpaset -p ds9 regions coordformat $3
    xpaset -p ds9 regions file $4
    xpaset -p ds9 regions save ${4}.out
    xpaset -p ds9 regions deleteall

    o=`diff $5 ${4}.out`
    if [ "$o" = "" ]
    then
	echo "PASSED"
    else
        echo "FAILED"
	echo "$o"
    fi
    rm -f ${4}.out
}

echo "Starting DS9..."
if [ `xpaaccess ds9` = no ]
then
    ../ds9&

    i=1
    while [ "$i" -le 15 ]
    do
        sleep 4
        if [ `xpaaccess ds9` = yes ]
        then
	    break
        fi

        i=`expr $i + 1`
    done
fi

echo "Loading Data..."
xpaset -p ds9 scale zscale
xpaset -p ds9 file img16.fits
echo "DONE"

if [ "$1" = "ds9" -o  -z "$1" ]; then
echo
echo "Testing DS9 Format..."
testit ds9 image degrees ds9.image.reg
testit ds9 physical degrees ds9.physical.reg
testit ds9 fk4 degrees ds9.fk4.reg
testit ds9 fk4 sexagesimal ds9.fk4.hms.reg
testit ds9 fk5 degrees ds9.fk5.reg
testit ds9 fk5 sexagesimal ds9.fk5.hms.reg
testit ds9 icrs degrees ds9.icrs.reg
testit ds9 icrs sexagesimal ds9.icrs.hms.reg
testit ds9 galactic degrees ds9.galactic.reg
testit ds9 galactic sexagesimal ds9.galactic.hms.reg
testit ds9 ecliptic degrees ds9.ecliptic.reg
testit ds9 ecliptic sexagesimal ds9.ecliptic.hms.reg
fi

if [ "$1" = "fits" -o  -z "$1" ]; then
echo
echo "Testing FITS Regions Format..."
testit2 ds9 physical degrees ds9.reg.fits ds9.fits.reg
fi

if [ "$1" = "ciao" -o  -z "$1" ]; then
echo
echo "Testing CIAO Format..."
testit ciao physical degrees ciao.physical.reg
testit ciao fk5 sexagesimal ciao.fk5.reg
fi

if [ "$1" = "saotng" -o  -z "$1" ]; then
echo
echo "Testing SAOtng Format..."
testit saotng image degrees saotng.image.reg
testit saotng fk4 degrees saotng.fk4.reg
testit saotng fk4 sexagesimal saotng.fk4.hms.reg
testit saotng fk5 degrees saotng.fk5.reg
testit saotng fk5 sexagesimal saotng.fk5.hms.reg
testit saotng icrs degrees saotng.icrs.reg
testit saotng icrs sexagesimal saotng.icrs.hms.reg
testit saotng galactic degrees saotng.galactic.reg
testit saotng galactic sexagesimal saotng.galactic.hms.reg
testit saotng ecliptic degrees saotng.ecliptic.reg
testit saotng ecliptic sexagesimal saotng.ecliptic.hms.reg
fi

if [ "$1" = "saoimage" -o  -z "$1" ]; then
echo
echo "Testing SAOimage Format..."
testit saoimage image degrees saoimage.reg
fi

if [ "$1" = "pros" -o  -z "$1" ]; then
echo
echo "Testing IRAF Pros Format..."
testit pros image degrees pros.image.reg
testit pros physical degrees pros.physical.reg
testit pros fk4 degrees pros.fk4.reg
testit pros fk4 sexagesimal pros.fk4.hms.reg
testit pros fk5 degrees pros.fk5.reg
testit pros fk5 sexagesimal pros.fk5.hms.reg
testit pros galactic degrees pros.galactic.reg
testit pros galactic sexagesimal pros.galactic.hms.reg
testit pros ecliptic degrees pros.ecliptic.reg
testit pros ecliptic sexagesimal pros.ecliptic.hms.reg
fi

if [ "$1" = "xy" -o  -z "$1" ]; then
echo
echo "Testing X Y Format..."
testit xy image degrees xy.image.reg
testit xy physical degrees xy.physical.reg
testit xy fk4 degrees xy.fk4.reg
testit xy fk4 sexagesimal xy.fk4.hms.reg
testit xy fk5 degrees xy.fk5.reg
testit xy fk5 sexagesimal xy.fk5.hms.reg
testit xy icrs degrees xy.icrs.reg
testit xy icrs sexagesimal xy.icrs.hms.reg
testit xy galactic degrees xy.galactic.reg
testit xy galactic sexagesimal xy.galactic.hms.reg
testit xy ecliptic degrees xy.ecliptic.reg
testit xy ecliptic sexagesimal xy.ecliptic.hms.reg
fi

if [ "$1" = "mosaic" -o  -z "$1" ]; then
echo
echo "Loading Mosaic Data..."
xpaset -p ds9 scale zscale
xpaset -p ds9 frame clear
xpaset -p ds9 file mosaicimage mosaicimage.fits
echo "DONE"

echo
echo "Testing DS9 Mosaic Format..."
testit ds9 image degrees ds9.mosaic.image.reg
testit ds9 physical degrees ds9.mosaic.physical.reg
testit ds9 fk4 degrees ds9.mosaic.fk4.reg
testit ds9 fk4 sexagesimal ds9.mosaic.fk4.hms.reg
testit ds9 fk5 degrees ds9.mosaic.fk5.reg
testit ds9 fk5 sexagesimal ds9.mosaic.fk5.hms.reg
testit ds9 icrs degrees ds9.mosaic.icrs.reg
testit ds9 icrs sexagesimal ds9.mosaic.icrs.hms.reg
testit ds9 galactic degrees ds9.mosaic.galactic.reg
testit ds9 galactic sexagesimal ds9.mosaic.galactic.hms.reg
testit ds9 ecliptic degrees ds9.mosaic.ecliptic.reg
testit ds9 ecliptic sexagesimal ds9.mosaic.ecliptic.hms.reg
fi

echo
echo "All Finished..."

if [ -z "$1" ]; then
xpaset -p ds9 quit
fi