package provide DS9 1.0

proc VODialog {} {
    global vo
    global ds9
    global menu
    global current

    if [winfo exists $vo(top)] {
	raise $vo(top)
	return
    }

    set w $vo(top)
    set title "Virtual Observatory"

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW VODestroyDialog

    # menu

    $w configure -menu $vo(mb)
    menu $vo(mb) -tearoff 0
    $vo(mb) add cascade -label File -menu $vo(mb).file
    menu $vo(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $vo(mb).file add command -label Update -command VOUpdateDialog
    $vo(mb).file add command -label Cancel -command VOCancelDialog
    $vo(mb).file add separator
    $vo(mb).file add command -label Close -command VODestroyDialog

    frame $w.param -relief groove -borderwidth 2
    label $w.param.status  -text "Connecting to Server..."
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    button $w.buttons.close -text "Close" -command VODestroyDialog
    pack $w.buttons.close -side left -padx 10 -expand true

    if {$vo(server,host) == ""} {
	VOUpdateDialog
    } else {
	set l [llength $vo(server,host)]
	for {set ii 0} {$ii<$l} {incr ii} {
	    set b [lindex $vo(server,button) $ii]
	    checkbutton $b -text [lindex $vo(server,title) $ii] \
		-variable vo(b$ii) -command "VOCheck $ii"
	    pack $b -anchor w -padx 4

	    # update color
	    if {$vo(b$ii)} {
		$b configure -selectcolor yellow
		update idletasks
		$b configure -selectcolor green
	    } else {
		$b configure -selectcolor yellow
		$b select
		update idletasks
		$b configure -selectcolor {}
		$b deselect
	    }
	}
    }
}

proc VODestroyDialog {} {
    global vo

    if {[info exists vo(token)]} {
	set vo(state) 0
	http::reset $vo(token)
    }

    destroy $vo(top)
    destroy $vo(mb)
}

proc VOUpdateDialog {} {
    global vo
    global xpa

    set w $vo(top)
    # delete old servers
    for {set ii 0} {$ii < [llength $vo(server,button)]} {incr ii} {
	catch {xparemote $xpa [lindex $vo(server,host) $ii] - -proxy}
	catch {destroy [lindex $vo(server,button) $ii]}
	catch {unset vo(b$ii)}
    }
    set vo(server,host) {}
    set vo(server,title) {}
    set vo(server,button) {}

    pack $w.param.status -ipadx 4 -ipady 4

    if [catch {set vo(token) [http::geturl $vo(server) -command VOFinish]}] {
	Error "Unable to contact VO Server $vo(server)"
	VODestroyDialog
    } else {
	set vo(state) 1
    }
}

proc VOCancelDialog {} {
    global vo

    if {[info exists vo(token)]} {
	set vo(state) 0
	http::reset $vo(token)
    }
}

proc VOFinish {token} {
    global vo

    set w $vo(top)
    pack forget $w.param.status

    set data [string trimright [http::data $token] \n]

    HTTPLog $token
    if {$vo(state)} {
	set lines [split $data \n]
	set l [llength $lines]
	for {set ii 0} {$ii<$l} {incr ii} {
	    set line [lindex $lines $ii]
	    set b "$w.param.b$ii"

	    lappend vo(server,host) [lindex $line 0]
	    lappend vo(server,title) [lrange $line 1 end]
	    lappend vo(server,button) $b
	    checkbutton $b -text [lindex $vo(server,title) $ii] \
		-variable vo(b$ii) -command "VOCheck $ii"
	    pack $b -anchor w -padx 4
	}
    }
    set vo(state) 0

    catch {unset vo(token)}
}

proc VOCheck {ii} {
    global vo
    global xpa

    set w $vo(top)
    set b "$w.param.b$ii"

    if {$vo(b$ii)} {
	$b configure -selectcolor yellow
	update idletasks
	xparemote $xpa [lindex $vo(server,host) $ii] + -proxy
	$b configure -selectcolor green
    } else {
	$b configure -selectcolor yellow
	$b select
	update idletasks
	xparemote $xpa [lindex $vo(server,host) $ii] - -proxy
	$b configure -selectcolor {}
	$b deselect
    }
}

proc VOPref {} {
    global vo
    set result $vo(server)
    if [EntryDialog "VO Server" "Enter VO Server List URL:" 80 result] {
	set vo(server) $result
    }
}