/*
 *      Copyrights:
 *
 *    	  Copyright (c) 2000 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <tl.h>

#ifdef ANSI_FUNC
double
tlp2i (double dp, double tlmin, double binsiz, int type)
#else
double tlp2i (dp, tlmin, binsiz, type)
     double dp;
     double tlmin;
     double binsiz;
     int type;
#endif
{
  if( (binsiz == 1.0) || (binsiz <= 0.0) ){
    switch(type){
    case 'B':
    case 'I':
    case 'U':
    case 'J':
    case 'V':
    default:
      return ((dp - tlmin) + 1.0);
    case 'E':
    case 'D':
      return ((dp - tlmin) + 0.5);
    }
  }
  else{
    switch(type){
    case 'B':
    case 'I':
    case 'U':
    case 'J':
    case 'V':
    default:
      return ((dp - tlmin)/binsiz + 1.0);
    case 'E':
    case 'D':
      return ((dp - tlmin)/binsiz + 0.5);
    }
  }
}

#ifdef ANSI_FUNC
int
itlp2i (double dp, double tlmin, double binsiz, int type)
#else
int itlp2i (dp, tlmin, binsiz, type)
     double dp;
     double tlmin;
     double binsiz;
     int type;
#endif
{
  if( (binsiz == 1.0) || (binsiz <= 0.0) ){
    return (int)((dp - tlmin) + 1.0);
  }
  else{
    return (int)((dp - tlmin)/binsiz + 1.0);
  }
}

#ifdef ANSI_FUNC
double
tli2p (double di, double tlmin, double binsiz, int type)
#else
double tli2p (di, tlmin, binsiz, type)
     double di;
     double tlmin;
     double binsiz;
     int type;
#endif
{
  if( (binsiz == 1.0) || (binsiz <= 0.0) ){
    switch(type){
    case 'B':
    case 'I':
    case 'U':
    case 'J':
    case 'V':
    default:
      return ((di - 1.0) + tlmin);
    case 'E':
    case 'D':
      return ((di - 0.5) + tlmin);
    }
  }
  else{
    switch(type){
    case 'B':
    case 'I':
    case 'U':
    case 'J':
    case 'V':
    default:
      return ((di - 1.0)*binsiz + tlmin);
    case 'E':
    case 'D':
      return ((di - 0.5)*binsiz + tlmin);
    }
  }
}

#ifdef ANSI_FUNC
double
tldim (double tlmin, double tlmax, double binsiz, int type)
#else
double tldim (tlmin, tlmax, binsiz, type)
     double tlmin;
     double tlmax;
     double binsiz;
     int type;
#endif
{
  if( (binsiz == 1.0) || (binsiz <= 0.0) ){
    switch(type){
    case 'B':
    case 'I':
    case 'U':
    case 'J':
    case 'V':
    default:
      return ((tlmax-tlmin) + 1.0);
    case 'E':
    case 'D':
      return (tlmax-tlmin);
    }
  }
  else{
    switch(type){
    case 'B':
    case 'I':
    case 'U':
    case 'J':
    case 'V':
    default:
      return ((tlmax-tlmin)/binsiz + 1.0);
    case 'E':
    case 'D':
      return ((tlmax-tlmin)/binsiz);
    }
  }
}

