#include "iis.h"

FitsIIS::FitsIIS(int width, int height)
{
  // new header
  head_ = new FitsHead(width, height, 8);
  if (!head_->isValid())
    return;

  // alloc memory
  data_ = new char[width*height];
  if (!data_)
    return;

  // clear memory
  memset(data_, '\0', width*height);

  // made it this far, must be valid
  valid_ = 1;
}

FitsIIS::~FitsIIS()
{
  if (data_)
    delete (char*)data_;
}

void FitsIIS::erase()
{
  // clear memory
  FitsImageHDU* hdu = (FitsImageHDU*)head_->hdu();
  memset(data_, '\0', hdu->realbytes());
}

const char* FitsIIS::get(int x, int y)
{
  if (head_)
    return (const char*)data_+(y*head_->naxis(1)+x);
  else
    return NULL;
}

void FitsIIS::set(int x, int y, const char* ptr, int length)
{
  // we have to make sure we don't overwrite our buffer

  int start = (y*head_->naxis(1)+x);
  if (head_ && (start+length <= head_->pixbytes()))
    memcpy((char*)data_+start, ptr, length);
}

