#ifdef HAVE_SYS_SHM_H
#include <sys/shm.h>

#include "share.h"

FitsShareID::FitsShareID(int shmid, int sz, const char* filter)
{
  parse(filter);

  // Attach the memory segment

  mapsize_ = sz;

  if ((int)(mapdata_ = (char*)shmat(shmid, 0, SHM_RDONLY)) == -1) {
    cerr << "Fitsy++ Internal Error: shmat failed" << endl;
    return;
  }

  // so far so good
  valid_ = 1;
}

FitsShareID::~FitsShareID()
{
  shmdt(mapdata_);
}

FitsShareKey::FitsShareKey(int key, int sz, const char* filter)
{
  parse(filter);

  // Attach the memory segment

  mapsize_ = sz;
  int shmid;
  if ((shmid = shmget(key, sz, 0)) < 0)
    return;

  if ((int)(mapdata_ = (char*)shmat(shmid, 0, SHM_RDONLY)) == -1) {
    cerr << "Fitsy++ Internal Error: shmat failed" << endl;
    return;
  }

  // so far so good
  valid_ = 1;
}

FitsShareKey::~FitsShareKey()
{
  shmdt(mapdata_);
}
#else
#include "share.h"

FitsShareID::FitsShareID(int shmid, int sz, const char* filter)
{
  // shared memory not supported
  valid_ = 0;
}

FitsShareID::~FitsShareID() {}

FitsShareKey::FitsShareKey(int key, int sz, const char* filter)
{
  // shared memory not supported
  valid_ = 0;
}

FitsShareKey::~FitsShareKey() {}
#endif



