#include <math.h>
#include <iostream.h>

#include "contourscale.h"
#include "frame.h"

ContourScale::ContourScale(int s) : size_(s)
{
  level_ = new float[size_+1];

  for (int i=0; i<=size_; i++)
    level_[i] = 0;
}

ContourScale::ContourScale(int s, float* l) : size_(s)
{
  level_ = new float[size_+1];

  for (int i=0; i<=size_; i++)
    level_[i] = l[i];
}

ContourScale::~ContourScale()
{
  if (level_)
    delete [] level_;
}

ostream& operator<<(ostream& s, const ContourScale& c)
{
  for (int i=0; i<c.size_+1; i++)
    s << c.level_[i] << endl;
  return s;
}

// Note: this an inverse algorithm. We have y, we need x

LinearContourScale::LinearContourScale(int s, float low, float high) 
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++)
    level_[i] = (float)i/size_*diff + low;
}

LogContourScale::LogContourScale(int s, float low, float high) 
  : ContourScale(s)
{
  double diff = high-low+1;
  for (int i=0; i<=size_; i++) {
    double y = (float)i/size_;
    double x = pow(10,y*log10(diff))/diff;
    level_[i] = x*diff+low-1;
  }
}

SquaredContourScale::SquaredContourScale(int s, float low, float high)
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++)
    level_[i] = sqrt((float)i/size_)*diff+low;
}

SqrtContourScale::SqrtContourScale(int s, float low, float high) 
  : ContourScale(s)
{
  float diff = high-low;
  for (int i=0; i<=size_; i++) {
    float y = (float)i/size_;
    level_[i] = (y*y)*diff+low;
  }
}

HistEquContourScale::HistEquContourScale(int s, float low, float high, 
					 double* hist, int histsize) 
  : ContourScale(s)
{
  if (!hist) {
    float diff = high-low;
    for (int i=0; i<=size_; i++)
      level_[i] = (float)i/size_*diff + low;
  }
  else {
    float diff = high-low;
    for (int i=0; i<=size_; i++) {
      float v = float(i)/size_;
      int j=0;
      while (j<histsize-1) {
	if (hist[j]>v)
	  break;
	j++;
      }
      level_[i] = float(j)/histsize*diff + low;
    }
  }
}

