#ifndef __fitsdata_h__
#define __fitsdata_h__

#include <iostream.h>
#include <strstream.h>

#include "vector.h"
#include "frame.h"
#include "file.h"

#if defined( __alpha) || defined(__mips64)
#define BITPIX32 int
#else
#define BITPIX32 long
#endif

class ColorScale;
class FitsFile;

class FitsData {
protected:
  int width;
  int height;
  long size;
  char buf[32];

  int byteswap;

  double bscale;
  double bzero;
  int hasScaling;

  int hasBlank;
  int blank;
  float nanf;          // generated NaN
  double nand;         // generated NaN

  double high;         // bscale applied
  double low;          // bscale applied

  float zHigh;         // bscale applied
  float zLow;          // bscale applied
  double aLow;	       // bscale applied		    
  double aHigh;	       // bscale applied		    
  double uLow;	       // bscale applied		    
  double uHigh;	       // bscale applied		    

  int scanValid;
  int incr;

  float zContrast;
  int zSampleSize;
  int zSampleLine;
  int zscaleValid;

  double* autoCutHist;
  int autoCutTotal;
  int autoCutValid;
  float autoCutPer;

  Frame::ClipMode clipMode;
  Frame::MinMaxMode mmMode;

  double datamin;      // bscale applied
  double datamax;      // bscale applied
  int hasdatamin;

  double irafmin;      // bscale applied
  double irafmax;      // bscale applied
  int hasirafmin;

  Frame::ScanMode scanMode;

  // zscale constants

  enum PixelType {GOOD_PIXEL, BAD_PIXEL, REJECT_PIXEL};

  int zSubSample(float*, float*, int, int);
  int zFitLine(float*, int, float*, float*, float, int, int);
  void zFlattenData (float*, float*, float*, int npix, float, float dz);
  int zComputeSigma (float*, short*, int npix, float*, float*);
  int zRejectPixels (float*, float*, float*, short*, int, double*, 
		     double*, double*, double*, float, int);

 protected:
  void autoCut(int*);

public:
  FitsData(FitsFile*);
  virtual ~FitsData();

  virtual const char* getValue(const Vector&) =0;
  virtual float getValueFloat(const Vector&) =0;
  virtual float getValueFloat(int) =0;
  virtual double getValueDouble(const Vector&) =0;
  virtual double getValueDouble(int) =0;

  virtual const char* getMin() =0;
  virtual const char* getMax() =0;
  virtual double getMinDouble() =0;
  virtual double getMaxDouble() =0;

  const char* getLow();
  const char* getHigh();
  double getLowDouble() {return low;}
  double getHighDouble() {return high;}

  void setClip(double l, double h) {low = l; high = h;}

  void resetMinMax() {scanMode = Frame::RESET;}

  virtual void updateClip(Frame::ClipMode,
			  Frame::MinMaxMode,int,
			  float,int,int,
			  float,
			  double,double,
			  Frame::ScanMode,int*) =0;

  int hasDATAMIN() {return hasdatamin;}
  int hasIRAFMIN() {return hasirafmin;}

  virtual void bin(double*,int,double,double,int*) =0;
};

template<class T>
class FitsDatam : public FitsData {
 private:
  T* data;
  T min;               // bscale not applied
  T max;               // bscale not applied

  T Tmin();
  T Tmax();

  void updateMinMax(int*);
  void scan(int*, int);
  void output(ostrstream&, T);
  void zscale(int*);
  int zSampleImage(float**,int*);

 public:
  FitsDatam(FitsFile*);

  T swap(T*);

  const char* getValue(const Vector&);

  float getValueFloat(int i);
  float getValueFloat(const Vector&);

  double getValueDouble(int i);
  double getValueDouble(const Vector&);

  const char* getMin();
  const char* getMax();

  double getMinDouble();
  double getMaxDouble();

  void updateClip(Frame::ClipMode,
		  Frame::MinMaxMode,int,
		  float,int,int,
		  float,
		  double,double,
		  Frame::ScanMode,int*);

  void bin(double*, int, double, double, int*);
};

#endif
