#include "framepseudo8.h"
#include "colorscalepseudo8.h"
#include "fitsimage.h"
#include "nan.h"
#include "ps.h"
#include "util.h"

// Tk Canvas Widget Function Declarations

int FramePseudoColor8CreateProc(Tcl_Interp*, Tk_Canvas, Tk_Item*, int, 
				Tcl_Obj *const []);

// FramePseudoColor8 Specs

static Tk_CustomOption tagsOption = {
  Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
};

static Tk_ConfigSpec framePseudoColor8Specs[] = {

  {TK_CONFIG_STRING, "-command", NULL, NULL, "frame",
   Tk_Offset(WidgetOptions, cmdName), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-x", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, x), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-y", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, y), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-width", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, width), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-height", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, height), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_ANCHOR, "-anchor", NULL, NULL, "nw",
   Tk_Offset(WidgetOptions, anchor), 0, NULL},
  {TK_CONFIG_CUSTOM, "-tags", NULL, NULL, NULL,
   0, TK_CONFIG_NULL_OK, &tagsOption},

  {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL},
};

// Tk Static Structure

static Tk_ItemType framePseudoColor8Type = {
  "framepseudocolor8",          // name
  sizeof(WidgetOptions),        // item size
  FramePseudoColor8CreateProc,  // configProc
  framePseudoColor8Specs,       // configSpecs
  WidgetConfigProc,             // configProc
  WidgetCoordProc,              // coordProc
  WidgetDeleteProc,             // deleteProc
  WidgetDisplayProc,            // displayProc
  0,                            // alwaysRedraw
  WidgetPointProc,              // pointProc
  WidgetAreaProc,               // areaProc
  WidgetPostscriptProc,         // postscriptProc
  WidgetScaleProc,              // scaleProc
  WidgetTranslateProc,          // translateProc
  (Tk_ItemIndexProc*)NULL,      // indexProc
  WidgetICursorProc,            // icursorProc
  (Tk_ItemSelectionProc*)NULL,  // selectionProc
  (Tk_ItemInsertProc*)NULL,     // insertProc
  (Tk_ItemDCharsProc*)NULL,     // dCharsProc
  (Tk_ItemType*)NULL            // nextPtr
};

// Non-Member Functions

int FramePseudoColor8_Init(Tcl_Interp* interp)
{
  Tk_CreateItemType(&framePseudoColor8Type);
  return TCL_OK;
}

int FramePseudoColor8CreateProc(Tcl_Interp* interp, Tk_Canvas canvas, 
				Tk_Item* item, int argc, Tcl_Obj *const argv[])
{
  FramePseudoColor8* frame = new FramePseudoColor8(interp, canvas, item);

  // and set default configuration

  if (frame->configure(argc, (char**)argv, 0) != TCL_OK) {
    delete frame;
    Tcl_AppendResult(interp, " error occured while creating frame.", NULL);
    return TCL_ERROR;
  }

  return TCL_OK;
}

// FramePseudoColor8 Member Functions

FramePseudoColor8::FramePseudoColor8(Tcl_Interp* i, Tk_Canvas c, Tk_Item* item)
  : Frame(i, c, item)
{
  configSpecs = framePseudoColor8Specs;  // frame configure options
}

void FramePseudoColor8::updateColorScale()
{
  // we need colors before we can construct a scale

  if (!indexCells || !colorCells)
    return;

  if (colorScale)
    delete colorScale;

  switch (colorScaleType) {
  case LINEARSCALE:
    colorScale =
      new LinearScalePseudoColor8(colorCount, indexCells, colorCells, 
				  colorCount);
    break;
  case LOGSCALE:
    colorScale = new LogScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
					  colorCount);
    break;
  case SQUAREDSCALE:
    colorScale =
      new SquaredScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
				   colorCount);
    break;
  case SQRTSCALE:
    colorScale = 
      new SqrtScalePseudoColor8(SCALESIZE, indexCells, colorCells, colorCount);
    break;
  case IISSCALE:
    colorScale =
      new IISScalePseudoColor8(this,indexCells, colorCells, colorCount);
    break;
  case HISTEQUSCALE:
    updateHistEqu();
    colorScale =
      new HistEquScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
				   colorCount, histequ, HISTEQUSIZE);
    break;
  }
}

void FramePseudoColor8::fillXImage(XImage* xmap, 
				   int x0, int y0, int x1, int y1,
				   double* (FitsImage::*getMatrix)())
{
  // we need a colorScale before we can render
  if (!colorScale)
    return;

  if (!currentFits)
    return;

  int& width = xmap->width;
  int& height = xmap->height;
  int& bytesPerLine = xmap->bytes_per_line;

  double* m = (currentFits->*getMatrix)();
  int* params = currentFits->getDataParams(scanMode);
  int& srcw = params[0];
  int& xmin = params[1];
  int& xmax = params[2];
  int& ymin = params[3];
  int& ymax = params[4];

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->colors();

  double ll = currentFits->getLowDouble();
  double hh = currentFits->getHighDouble();
  double diff = hh - ll;

  // clear ximage

#ifndef _WIN32
  memset(xmap->data, getWhiteColor(), bytesPerLine * height);
#else
  memset(xmap->data, 10, bytesPerLine * height);
#endif

  // test as doubles, convert to int's at last minute.
  // use double to maintain precision
  // This solves a host of problems.

  for (int j=y0; j<y1; j++) {
    // the line may be padded at the end
    char* dest = xmap->data + j*bytesPerLine + x0;

    for (int i=x0; i<x1; i++,dest++) {
      double x = i*m[0] + j*m[3] + m[6];
      double y = i*m[1] + j*m[4] + m[7];

      if (x>=xmin && x<xmax && y>=ymin && y<ymax) {
	double value = currentFits->getValueDouble(((int)y)*srcw + (int)x);
	
	if (!isNaNd(value))
	  if (value <= ll)
	    *dest = table[0];
	  else if (value >= hh)
	    *dest = table[length];
	  else
	    *dest = table[(int)(((value - ll)/diff * length) + .5)];
      }
    }
  }
}

void FramePseudoColor8::fillXImageMosaic(XImage* xmap, 
					 int x0, int y0, int x1, int y1,
					 double* (FitsImage::*getMatrix)())
{
  // we need a colorScale before we can render

  if (!colorScale)
    return;

  int& width = xmap->width;
  int& height = xmap->height;
  int& bytesPerLine = xmap->bytes_per_line;
  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->colors();

  // clear ximage

#ifndef _WIN32
  memset(xmap->data, getWhiteColor(), bytesPerLine * height);
#else
  memset(xmap->data, 10, bytesPerLine * height);
#endif

  // test as doubles, convert to int's at last minute.
  // use double to maintain precision
  // This solves a host of problems.

  for (int j=y0; j<y1; j++) {
    // the line may be padded at the end
    char* dest = xmap->data + j*bytesPerLine + x0;

    for (int i=x0; i<x1; i++,dest++) {

      FitsImage* ptr = fits;
      while (ptr) {
	double* m = (ptr->*getMatrix)();
	double x = i*m[0] + j*m[3] + m[6];
	double y = i*m[1] + j*m[4] + m[7];

	int* params = ptr->getDataParams(scanMode);
	int& srcw = params[0];
	int& xmin = params[1];
	int& xmax = params[2];
	int& ymin = params[3];
	int& ymax = params[4];

	if (x>=xmin && x<xmax && y>=ymin && y<ymax) {
	  double value = ptr->getValueDouble(((int)y)*srcw + (int)x);

	  double ll = ptr->getLowDouble();
	  double hh = ptr->getHighDouble();
	  double diff = hh - ll;

	  if (!isNaNd(value))
	    if (value <= ll)
	      *dest = table[0];
	    else if (value >= hh)
	      *dest = table[length];
	    else
	      *dest = table[(int)(((value - ll)/diff * length) + .5)];

	  break;
	}
	else
	  ptr = ptr->next();
      }
    }
  }
}

void FramePseudoColor8::fillXImageMosaicFast(XImage* xmap, 
					     int x0, int y0, int x1, int y1,
					     double* (FitsImage::*getMatrix)())
{
  // we need a colorScale before we can render

  if (!colorScale)
    return;

  if (!fits)
    return;

  int& width = xmap->width;
  int& height = xmap->height;
  int& bytesPerLine = xmap->bytes_per_line;
  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->colors();

  // clear ximage

#ifndef _WIN32
  memset(xmap->data, getWhiteColor(), bytesPerLine * height);
#else
  memset(xmap->data, 10, bytesPerLine * height);
#endif

  FitsImage* ptr = NULL;
  double* m;

  int* params;
  int srcw;
  int xmin;
  int xmax;
  int ymin;
  int ymax;

  double ll;
  double hh;
  double diff;

  // test as doubles, convert to int's at last minute.
  // use double to maintain precision
  // This solves a host of problems.

  for (int j=y0; j<y1; j++) {
    // the line may be padded at the end
    char* dest = xmap->data + j*bytesPerLine + x0;

    for (int i=x0; i<x1; i++,dest++) {
      double x,y;

      if (ptr) {
	x = i*m[0] + j*m[3] + m[6];
	y = i*m[1] + j*m[4] + m[7];

	if (x>=xmin && x<xmax && y>=ymin && y<ymax) {
	  double value = ptr->getValueDouble(((int)y)*srcw + (int)x);

	  if (!isNaNd(value))
	    if (value <= ll)
	      *dest = table[0];
	    else if (value >= hh)
	      *dest = table[length];
	    else
	      *dest = table[(int)(((value - ll)/diff * length) + .5)];
	}
	else
	  ptr = NULL;
      }
      
      if (!ptr) {
	ptr = fits;
	while (ptr) {
	  m = (ptr->*getMatrix)();
	  x = i*m[0] + j*m[3] + m[6];
	  y = i*m[1] + j*m[4] + m[7];

	  params = ptr->getDataParams(scanMode);
	  srcw = params[0];
	  xmin = params[1];
	  xmax = params[2];
	  ymin = params[3];
	  ymax = params[4];

	  if (x>=xmin && x<xmax && y>=ymin && y<ymax) {
	    double value = ptr->getValueDouble(((int)y)*srcw + (int)x);

	    ll = ptr->getLowDouble();
	    hh = ptr->getHighDouble();
	    diff = hh - ll;

	    if (!isNaNd(value))
	      if (value <= ll)
		*dest = table[0];
	      else if (value >= hh)
		*dest = table[length];
	      else
		*dest = table[(int)(((value - ll)/diff * length) + .5)];

	    break;
	  }
	  else
	    ptr = ptr->next();
	}
      }
    }
  }
}

void FramePseudoColor8::updateMagnifierReplicate(const Vector& v)
{
  // v is in CANVAS Coords
  // we have a race condition here. It's possible that a magnifier command has
  // been generated right after the pixmap has been invalidated and before
  // a new XImage has been generated. So, bail, and try again.

  if (!baseXImage || !magnifierXImage)
    return;

  char* image = baseXImage->data;
  char* mag = magnifierXImage->data;
  int white = getWhiteColor();

  // map upper left of magnifier to image

  Matrix m = Translate(-(v * canvasToWidget)) * ximageToMagnifier;
  Matrix n = m.invert();

  Vector ll = Vector(0,0) * n;

  // and init image row and column counters

  int imageY = (int)ll[1];

  // loop thru each magnifier pixel

  char* row = new char[magnifierWidth];
  for (int j=0; j<=magnifierHeight; j+=(int)magnifierZoom, imageY++) {

    // Clear tmp row

#ifndef _WIN32
    memset(row, white, magnifierWidth);
#else
    memset(row, 10, magnifierWidth);
#endif

    // Build tmp row

    int imageX = (int)ll[0];
    for (int i=0; i<=magnifierWidth; i+=(int)magnifierZoom, imageX++) {

      // find pixel value

      unsigned char value;
      if (imageX>=0 && imageX<options->width && 
	  imageY>=0 && imageY<options->height)
	value = image[imageY*baseXImage->bytes_per_line + imageX];
      else
#ifndef _WIN32
	value = white;
#else
        value = 10;
#endif

      // replicate pixel

      int start = i - (int)(magnifierZoom/2.);
      if (start < 0)
	start = 0;
      int stop = i + (int)(magnifierZoom/2.);
      if (stop > magnifierWidth)
	stop = magnifierWidth;
      for (int k=start; k<stop; k++) {
	  row[k] = value;
      }
    }

    // and replicate temp row
    
    int start = j - (int)(magnifierZoom/2.);
    if (start < 0)
      start = 0;
    int stop = j + (int)(magnifierZoom/2.);
    if (stop > magnifierHeight)
      stop = magnifierHeight;
    for (int k=start; k<stop; k++)
	memcpy(mag+(k*magnifierXImage->bytes_per_line), row, magnifierWidth);
  }
  delete row;

  XPutImage(display, magnifierPixmap, gc, magnifierXImage, 0, 0, 0, 0, 
	    magnifierWidth, magnifierHeight);
}

void FramePseudoColor8::colormapCmd(int id, float b, float c, int i, 
				    unsigned short* index, 
				    unsigned char* cells, int cnt)
{
  cmapID = id;
  bias = b;
  contrast = c;
  invert = i;

  updateColorCells(index, cells, cnt);
  updateColorScale();
}

void FramePseudoColor8::rotateMotion()
{
  // Rotate from src to dest

  Vector c = Vector(options->width,options->height)/2;
  Matrix m = (Translate(-c) * 
	      Rotate(rotation-rotateRotation) * Translate(c)).invert();
  double* mm = m.mm();
  int white = getWhiteColor();

  int& width = options->width;
  int& height = options->height;
  char* src = rotateSrcXM->data;

  for (int j=0; j<height; j++) {
    // the line may be padded at the end
    char* dest = rotateDestXM->data + j*rotateDestXM->bytes_per_line;

    for (int i=0; i<width; i++, dest++) {
      double x = i*mm[0] + j*mm[3] + mm[6];
      double y = i*mm[1] + j*mm[4] + mm[7];

      if (x >= 0 && x < width && y >= 0 && y < height)
	*dest = src[((int)y)*rotateSrcXM->bytes_per_line + (int)x];
      else
#ifndef _WIN32
	*dest = white;
#else
	*dest = 10;
#endif
    }
  }

  // XImage to Pixmap

  XPutImage(display, rotatePM, gc, rotateDestXM,
	    0, 0, 0, 0, options->width, options->height);

  // Display Pixmap

  Vector d = Vector(0,0) * widgetToWindow;
  XCopyArea(display, rotatePM, Tk_WindowId(tkwin), rotateGCXOR, 0, 0, 
	    options->width, options->height, (int)d[0], (int)d[1]);
}
