#include <iostream.h>
#include <strstream.h>

#include "frame.h"
#include "fitsimage.h"

#include "alloc.h"
#include "allocgz.h"
#include "channel.h"
#include "mmap.h"
#include "share.h"
#include "socket.h"
#include "socketgz.h"
#include "var.h"

// *** Fits ***

void Frame::loadFitsAllocCmd(const char* fn)
{
  FitsImage* img = new FitsImageFitsAlloc(this, fn);
  load(ALLOC,fn,img);
}

void Frame::loadFitsAllocGZCmd(const char* fn)
{
  FitsImage* img = new FitsImageFitsAllocGZ(this, fn);
  load(ALLOCGZ,fn,img);
}

void Frame::loadFitsChannelCmd(const char* ch, const char* fn)
{
  FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn);
  load(CHANNEL,fn,img);
}

void Frame::loadFitsMMapCmd(const char* fn, LoadMethod lm)
{
  FitsImage* img = new FitsImageFitsMMap(this, fn);
  setScanModeIncr(lm);
  load(MMAP,fn,img);
}

void Frame::loadFitsShareCmd(ShmType type, int id, int size, 
			     const char* fn, LoadMethod lm)
{
  FitsImage* img = new FitsImageFitsShare(this, type, id, size, fn);
  setScanModeIncr(lm);
  load(SHARE,fn,img);
}

void Frame::loadFitsSocketCmd(int s, const char* fn)
{
  FitsImage* img = new FitsImageFitsSocket(this, s, fn);
  load(SOCKET,fn,img);
}

void Frame::loadFitsSocketGZCmd(int s, const char* fn)
{
  FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn);
  load(SOCKETGZ,fn,img);
}

void Frame::loadFitsVarCmd(const char* ch, const char* fn, LoadMethod lm)
{
  FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn);
  setScanModeIncr(lm);
  load(VAR,fn,img);
}

// *** Array ***

void Frame::loadArrAllocCmd(const char* fn)
{
  FitsImage* img = new FitsImageArrAlloc(this, fn);
  load(ALLOC,fn,img);
}

void Frame::loadArrAllocGZCmd(const char* fn)
{
  FitsImage* img = new FitsImageArrAllocGZ(this, fn);
  load(ALLOCGZ,fn,img);
}

void Frame::loadArrChannelCmd(const char* ch, const char* fn)
{
  FitsImage* img = new FitsImageArrChannel(this, interp, ch, fn);
  load(CHANNEL,fn,img);
}

void Frame::loadArrMMapCmd(const char* fn)
{
  FitsImage* img = new FitsImageArrMMap(this, fn);
  load(MMAP,fn,img);
}

void Frame::loadArrShareCmd(ShmType type, int id, int size, const char* fn)
{
  FitsImage* img = new FitsImageArrShare(this, type, id, size, fn);
  load(SHARE,fn,img);
}

void Frame::loadArrSocketCmd(int s, const char* fn)
{
  FitsImage* img = new FitsImageArrSocket(this, s, fn);
  load(SOCKET,fn,img);
}

void Frame::loadArrSocketGZCmd(int s, const char* fn)
{
  FitsImage* img = new FitsImageArrSocketGZ(this, s, fn);
  load(SOCKETGZ,fn,img);
}

void Frame::loadArrVarCmd(const char* ch, const char* fn)
{
  FitsImage* img = new FitsImageArrVar(this, interp, ch, fn);
  load(VAR,fn,img);
}

// *** Fits Slave ***

void Frame::loadFitsSlaveCmd(const char* fn, FitsFile* f)
{
  FitsImage* img = new FitsImageSlave(this,fn,f);
  load(MMAP,fn,img);
}

// *** Mosaic Image ***

void Frame::loadMosaicImageAllocCmd(const char* fn, MosaicType mt)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicAlloc(this, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	loadMosaicImage(ALLOC,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsAlloc(this, fn);
	loadWFPC2(ALLOC,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicAlloc(this, fn, mt);
      loadMosaicImage(ALLOC,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsAlloc(this, fn);
      loadWFPC2(ALLOC,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageAllocGZCmd(const char* fn, MosaicType mt)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicAllocGZ(this, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	loadMosaicImage(ALLOCGZ,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsAllocGZ(this, fn);
	loadWFPC2(ALLOCGZ,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicAllocGZ(this, fn, mt);
      loadMosaicImage(ALLOCGZ,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsAllocGZ(this, fn);
      loadWFPC2(ALLOCGZ,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageChannelCmd(const char* ch, const char* fn, 
				    MosaicType mt)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicChannel(this, interp, ch, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	loadMosaicImage(CHANNEL,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsChannel(this, interp, ch, fn);
	loadWFPC2(CHANNEL,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicChannel(this, interp, ch, fn, mt);
      loadMosaicImage(CHANNEL,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn);
      loadWFPC2(CHANNEL,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageMMapCmd(const char* fn, 
				   MosaicType mt, LoadMethod lm)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicMMap(this, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	setScanModeIncr(lm);
	loadMosaicImage(MMAP,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsMMap(this, fn);
	setScanModeIncr(lm);
	loadWFPC2(MMAP,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicMMap(this, fn, mt);
      setScanModeIncr(lm);
      loadMosaicImage(MMAP,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsMMap(this, fn);
      setScanModeIncr(lm);
      loadWFPC2(MMAP,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageShareCmd(ShmType type, int id, int size,
				  const char* fn,
				  MosaicType mt, LoadMethod lm)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicShare(this, type, id, size, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	setScanModeIncr(lm);
	loadMosaicImage(MMAP,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsShare(this, type, id, size, fn);
	setScanModeIncr(lm);
	loadWFPC2(SHARE,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicShare(this, type, id, size, fn, mt);
      setScanModeIncr(lm);
      loadMosaicImage(MMAP,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsShare(this, type, id, size, fn);
      setScanModeIncr(lm);
      loadWFPC2(SHARE,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageSocketCmd(int s, const char* fn, MosaicType mt)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicSocket(this, s, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	loadMosaicImage(SOCKET,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsSocket(this, s, fn);
	loadWFPC2(SOCKET,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicSocket(this, s, fn, mt);
      loadMosaicImage(SOCKET,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsSocket(this, s, fn);
      loadWFPC2(SOCKET,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageSocketGZCmd(int s, const char* fn, MosaicType mt)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicSocketGZ(this, s, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	loadMosaicImage(SOCKETGZ,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsSocketGZ(this, s, fn);
	loadWFPC2(SOCKETGZ,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicSocketGZ(this, s, fn, mt);
      loadMosaicImage(SOCKETGZ,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn);
      loadWFPC2(SOCKETGZ,fn,img);
    }
    break;
  }
}

void Frame::loadMosaicImageVarCmd(const char* ch, const char* fn,
				MosaicType mt, LoadMethod lm)
{
  switch (mt) {
  case AUTO:
    {
      // try IRAF,WCS
      FitsImage* img = new FitsImageMosaicVar(this, interp, ch, fn, mt);
      if (img && img->isValid() && img->isMosaic()) {
	setScanModeIncr(lm);
	loadMosaicImage(VAR,fn,img,mt);
      }
      else {
	if (img)
	  delete img;

	// now try WFPC2
	img = new FitsImageFitsVar(this, interp, ch, fn);
	setScanModeIncr(lm);
	loadWFPC2(VAR,fn,img);
      }
    }
    break;
  case IRAF:
  case WCS:
    {
      FitsImage* img = new FitsImageMosaicVar(this, interp, ch, fn, mt);
      setScanModeIncr(lm);
      loadMosaicImage(VAR,fn,img,mt);
    }
    break;
  case WFPC2:
    {
      FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn);
      setScanModeIncr(lm);
      loadWFPC2(VAR,fn,img);
    }
    break;
  }
}

// *** Mosaic ***

void Frame::loadMosaicAllocCmd(const char* fn, MosaicType mt)
{
  FitsImage* img = new FitsImageFitsAlloc(this, fn, mt, fitsCount+1);
  loadMosaic(img);
}

void Frame::loadMosaicAllocGZCmd(const char* fn, MosaicType mt)
{
  FitsImage* img = new FitsImageFitsAllocGZ(this, fn, mt, fitsCount+1);
  loadMosaic(img);
}

void Frame::loadMosaicChannelCmd(const char* ch, const char* fn, MosaicType mt)
{
  FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn, 
					    mt, fitsCount+1);
  loadMosaic(img);
}

void Frame::loadMosaicMMapCmd(const char* fn, MosaicType mt, LoadMethod lm)
{
  FitsImage* img = new FitsImageFitsMMap(this, fn, mt, fitsCount+1);
  setScanModeIncr(lm);
  loadMosaic(img);
}

void Frame::loadMosaicShareCmd(ShmType type, int id, int size, const char* fn,
			       MosaicType mt, LoadMethod lm)
{
  FitsImage* img = new FitsImageFitsShare(this, type, id, size, fn,
					  mt, fitsCount+1);
  setScanModeIncr(lm);
  loadMosaic(img);
}

void Frame::loadMosaicSocketCmd(int s, const char* fn, MosaicType mt)
{
  FitsImage* img = new FitsImageFitsSocket(this, s, fn, mt, fitsCount+1);
  loadMosaic(img);
}

void Frame::loadMosaicSocketGZCmd(int s, const char* fn, MosaicType mt)
{
  FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn, mt, fitsCount+1);
  loadMosaic(img);
}

void Frame::loadMosaicVarCmd(const char* ch, const char* fn, MosaicType mt,
			     LoadMethod lm)
{
  FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn, mt, fitsCount+1);
  setScanModeIncr(lm);
  loadMosaic(img);
}

// ***

void Frame::load(MemType which, const char* fn, FitsImage* img)
{
  unloadFits();

  if (!img || !img->isValid()) {
    if (img)
      delete img;
    
    reset();

    updateClip();
    initHistogram();
    update(MATRIX);
    
    Tcl_AppendResult(interp, "Unable to load fits file", NULL);
    result = TCL_ERROR;
    return;
  }

  // Its legal, save it

  fits = img;
  currentFits = img; // the first is our 'current'
  fitsCount = 1;
  displayMode = SINGLE;

  // get the rest

  FitsImage* ptr = fits;
  for (int i=1; i<img->depth(); i++) {
    FitsImage* next = NULL;
    switch (which) {
    case ALLOC:
      next = new FitsImageFitsNextAlloc(this, fn, ptr->getFitsFile());
      break;
    case ALLOCGZ:
      next = new FitsImageFitsNextAllocGZ(this, fn, ptr->getFitsFile());
      break;
    case CHANNEL:
      next = new FitsImageFitsNextChannel(this, fn, ptr->getFitsFile());
      break;
    case MMAP:
      next = new FitsImageFitsNextMMap(this, fn, ptr->getFitsFile());
      break;
    case SHARE:
      next = new FitsImageFitsNextShare(this, fn, ptr->getFitsFile());
      break;
    case SOCKET:
      next = new FitsImageFitsNextSocket(this, fn, ptr->getFitsFile());
      break;
    case SOCKETGZ:
      next = new FitsImageFitsNextSocketGZ(this, fn, ptr->getFitsFile());
      break;
    case VAR:
      next = new FitsImageFitsNextVar(this, fn, ptr->getFitsFile());
      break;
    }

    if (next && next->isValid()) {
	ptr->setNext(next);
	ptr = next;
	fitsCount++;
      }
    else {
      if (next)
	delete next;
      break;
    }
  }

  // reset some values
  
  centerImage();
  crosshair = cursor;
  wcsAlign();

  // update clip values

  clipScope = LOCAL;
  
  if (!currentFits->hasDATAMIN() && mmMode == DATAMIN)
    mmMode = SAMPLE;
  if (!currentFits->hasIRAFMIN() && mmMode == IRAFMIN)
    mmMode = SAMPLE;

  updateClip();
  initHistogram();
  update(MATRIX);
}

void Frame::loadMosaicImage(MemType which, const char* fn, FitsImage* img,
			    MosaicType mt)
{
  if (!img || !img->isValid() || !img->isMosaic()) {
    if (img)
      delete img;
    
    Tcl_AppendResult(interp, "Unable to load mosaic file", NULL);
    result = TCL_ERROR;
    return;
  }

  // Its legal, save it

  unloadFits();

  fits = img;
  currentFits = img; // the first is our 'current'
  fitsCount = 1;
  displayMode = MOSAIC;

  // get the rest

  FitsImage* ptr = fits;
  while (1) {
    FitsImage* next = NULL;
    int cnt = fitsCount+1;
    switch (which) {
    case ALLOC:
      next = new FitsImageMosaicNextAlloc(this, fn, ptr->getFitsFile(), 
					  mt, cnt);
      break;
    case ALLOCGZ:
      next = new FitsImageMosaicNextAllocGZ(this, fn, ptr->getFitsFile(), 
					    mt, cnt);
      break;
    case CHANNEL:
      next = new FitsImageMosaicNextChannel(this, fn, ptr->getFitsFile(), 
					    mt, cnt);
      break;
    case MMAP:
      next = new FitsImageMosaicNextMMap(this, fn, ptr->getFitsFile(), 
					 mt, cnt);
      break;
    case SHARE:
      next = new FitsImageMosaicNextShare(this, fn, ptr->getFitsFile(), 
					  mt, cnt);
      break;
    case SOCKET:
      next = new FitsImageMosaicNextSocket(this, fn, ptr->getFitsFile(), 
					   mt, cnt);
      break;
    case SOCKETGZ:
      next = new FitsImageMosaicNextSocketGZ(this, fn, ptr->getFitsFile(),
					     mt, cnt);
      break;
    case VAR:
      next = new FitsImageMosaicNextVar(this, fn, ptr->getFitsFile(), 
					mt, cnt);
      break;
    }

    if (next && next->isValid()) {
      if (next->isImage() && next->isMosaic()) {
	ptr->setNext(next);
	ptr = next;
	fitsCount++;
      }
      else
	if (next)
	  delete next;
    }
    else {
      if (next)
	delete next;
      break;
    }
  }

  // reset some values
  
  centerImage();
  crosshair = cursor;
  wcsAlign();

  // update clip values

  if (!currentFits->hasDATAMIN() && mmMode == DATAMIN)
    mmMode = SAMPLE;
  if (!currentFits->hasIRAFMIN() && mmMode == IRAFMIN)
    mmMode = SAMPLE;

  updateClip();
  initHistogram();
  update(MATRIX);
}

void Frame::loadMosaic(FitsImage* img)
{
  if (!img || !img->isValid() || !img->isMosaic()) {
    if (img)
      delete img;
    
    Tcl_AppendResult(interp, "Unable to load mosaic file", NULL);
    result = TCL_ERROR;
    return;
  }

  // Its legal, save it

  fitsCount++;
  displayMode = MOSAIC;

  if (fits) {
    FitsImage* ptr = fits;
    while (ptr && ptr->next())
      ptr = ptr->next();
    ptr->setNext(img);
  }
  else {
    fits = img;
    currentFits = img; // the first is our 'current'
  }

  // reset some values

  centerImage();
  crosshair = cursor;
  wcsAlign();

  // update clip values

  if (!fits->hasDATAMIN() && mmMode == DATAMIN)
    mmMode = SAMPLE;
  if (!fits->hasIRAFMIN() && mmMode == IRAFMIN)
    mmMode = SAMPLE;

  updateClip();
  initHistogram();
  update(MATRIX);
}

void Frame::loadWFPC2(MemType which, const char* fn, FitsImage* img)
{
  unloadFits();

  // Its legal, save it

  fits = currentFits = img; // the first is our 'current'
  fitsCount = 1;
  displayMode = MOSAIC;

  if (!img || !img->isValid() || !img->isImage())
    goto error;

  // get the rest

  {
    FitsImage* ptr = fits;
    for (int i=1; i<img->depth(); i++) {
      FitsImage* next = NULL;
      switch (which) {
      case ALLOC:
	next = new FitsImageFitsNextAlloc(this, fn, ptr->getFitsFile());
	break;
      case ALLOCGZ:
	next = new FitsImageFitsNextAllocGZ(this, fn, ptr->getFitsFile());
	break;
      case CHANNEL:
	next = new FitsImageFitsNextChannel(this, fn, ptr->getFitsFile());
	break;
      case MMAP:
	next = new FitsImageFitsNextMMap(this, fn, ptr->getFitsFile());
	break;
      case SHARE:
	next = new FitsImageFitsNextShare(this, fn, ptr->getFitsFile());
	break;
      case SOCKET:
	next = new FitsImageFitsNextSocket(this, fn, ptr->getFitsFile());
	break;
      case SOCKETGZ:
	next = new FitsImageFitsNextSocketGZ(this, fn, ptr->getFitsFile());
	break;
      case VAR:
	next = new FitsImageFitsNextVar(this, fn, ptr->getFitsFile());
	break;
      }

      if (next && next->isValid()) {
	ptr->setNext(next);
	ptr = next;
	fitsCount++;
      }
      else {
	if (next)
	  delete next;
	break;
      }
    }
  }

  // ok, do we have 4 images?

  if (fitsCount != 4)
    goto error;

  // now, find WCS table

  FitsFile* table;
  switch (which) {
  case ALLOC:
    table = new FitsMosaicNextAlloc(fits->getFitsFile());
    break;
  case ALLOCGZ:
    table = new FitsMosaicNextAllocGZ(fits->getFitsFile());
    break;
  case CHANNEL:
    table = new FitsMosaicNextChannel(fits->getFitsFile());
    break;
  case MMAP:
    table = new FitsMosaicNextMMap(fits->getFitsFile());
    break;
  case SHARE:
    table = new FitsMosaicNextShare(fits->getFitsFile());
    break;
  case SOCKET:
    table = new FitsMosaicNextSocket(fits->getFitsFile());
    break;
  case SOCKETGZ:
    table = new FitsMosaicNextSocketGZ(fits->getFitsFile());
    break;
  case VAR:
    table = new FitsMosaicNextVar(fits->getFitsFile());
    break;
  }

  if (!table || !table->isValid() || !table->isTable()) {
    if (table)
      delete table;
    goto error;
  }

  // read WCS from table

  {
    FitsHead* th = table->head();
    if (th->naxes() != 2) {
      if (table)
	delete table;
      goto error;
    }

    FitsTableHDU* thdu = (FitsTableHDU*)th->hdu();

    FitsColumn* crval1 = thdu->find("CRVAL1");
    FitsColumn* crval2 = thdu->find("CRVAL2");
    FitsColumn* crpix1 = thdu->find("CRPIX1");
    FitsColumn* crpix2 = thdu->find("CRPIX2");

    FitsColumn* cd1_1 = thdu->find("CD1_1");
    FitsColumn* cd1_2 = thdu->find("CD1_2");
    FitsColumn* cd2_1 = thdu->find("CD2_1");
    FitsColumn* cd2_2 = thdu->find("CD2_2");

    FitsColumn* ctype1 = thdu->find("CTYPE1");
    FitsColumn* ctype2 = thdu->find("CTYPE2");

    char* tptr = (char*)table->data();
    int rows = thdu->rows();
    int rowlen = thdu->width();

    if (rows != 4)
      goto error;

    FitsImage* ptr = fits;

    for (int i=0; i<rows; i++, tptr+=rowlen) {
      char buf[512];
      ostrstream ostr(buf,512);

      ostr << "CRVAL1 = " << crval1->str(tptr) << endl
	   << "CRVAL2 = " << crval2->str(tptr) << endl
	   << "CRPIX1 = " << crpix1->str(tptr) << endl
	   << "CRPIX2 = " << crpix2->str(tptr) << endl
	   << "CD1_1  = " <<  cd1_1->str(tptr) << endl
	   << "CD1_2  = " <<  cd1_2->str(tptr) << endl
	   << "CD2_1  = " <<  cd2_1->str(tptr) << endl
	   << "CD2_2  = " <<  cd2_2->str(tptr) << endl
	   << "CTYPE1 = " << '\'' << ctype1->str(tptr) << '\'' << endl
	   << "CTYPE2 = " << '\'' << ctype2->str(tptr) << '\'' << endl
	   << ends;

      istrstream istr(buf,512);
      FitsHead* r = parseWCS(istr);

      // fix fitsimage params

      ptr->setMosaicParams(i+1,WFPC2);
      ptr->replaceWCS(r);
      ptr->processKeywordsWCS();

      delete r;
      ptr = ptr->next();
    }
  }

  if (table)
    delete table;

  // reset some values
  
  centerImage();
  crosshair = cursor;
  wcsAlign();

  // update clip values

  clipScope = LOCAL;
  
  if (!currentFits->hasDATAMIN() && mmMode == DATAMIN)
    mmMode = SAMPLE;
  if (!currentFits->hasIRAFMIN() && mmMode == IRAFMIN)
    mmMode = SAMPLE;

  updateClip();
  initHistogram();
  update(MATRIX);

  return;

 error:
  unloadFits();

  reset();

  updateClip();
  initHistogram();
  update(MATRIX);
    
  Tcl_AppendResult(interp, "Unable to load mosaic file", NULL);
  result = TCL_ERROR;
  return;
}

void Frame::setScanModeIncr(LoadMethod lm)
{
  if (lm == INCR)
    switch (scanMode) {
    case NODATASEC:
    case UNODATASEC:
    case RESET:
      scanMode = UNODATASEC;
      break;
    case DATASEC:
    case UDATASEC:
      scanMode = UDATASEC;
      break;
    }
}

