#ifndef __grid_h__
#define __grid_h__

#include <tk.h>
#include "vector.h"

class Frame;
class FitsImage;
class Grid;

struct FitsStore;

class Attribute {
 public:
  enum Style {SOLID,DASH};
  enum Font {
    DEFAULT=0,TIMES,HELVETICA,SYMBOL,COURIER,
    DEFAULTBOLD=10,TIMESBOLD,HELVETICABOLD,SYMBOLBOLD,COURIERBOLD,
    DEFAULTITALIC=20,TIMESITALIC,HELVETICAITALIC,SYMBOLITALIC,COURIERITALIC
  };
  enum Colour {BLACK,WHITE,RED,GREEN,BLUE,CYAN,MAGENTA,YELLOW};

 private:
  Frame* parent;

  Style style_;
  float width_;
  int size_;
  Font font_;
  Tk_Font tkfont_;
  Colour colour_;
  char* colorName_;
  int color_;

 private:
  void initFont();

 public:
  Attribute(Frame*);
  ~Attribute();

  void setStyle(double);
  void setWidth(double);
  void setSize(double);
  void setFont(double);
  void setColour(double);

  Style style() {return style_;}
  float width() {return width_;}
  int size() {return size_;}
  Font font() {return font_;}
  Colour colour() {return colour_;}

  const char* colorName() {return colorName_;}
  Tk_Font tkfont() {return tkfont_;}
  int color() {return color_;}
};

class Grid {
 public:
  enum RenderMode {X11, PS};
  enum RenderBound{WIDGETBB,IMAGEBB,MOSAICBB};

 private:
  Frame* parent;
  FitsImage* fits;
  int type_;

  char* option_;
  RenderMode renderMode;
  int psMode;
  int system_;
  Attribute* line;
  Attribute* mark;
  Attribute* text;

 private:
  int doit(RenderMode, RenderBound);

  void x11Line(int n, const float* x, const float* y);
  void x11Text(const char*, float, float, const char*, float, float);

  void psLine(int n, const float* x, const float* y);
  void psText(const char*, float, float, const char*, float, float);

  void psColor(Attribute*);
  void psWidth(Attribute*);
  void psStyle(Attribute*);

  void drawString(const char*, Vector, const char*);
  void drawRotString(const char*, Vector, const char*, double, int);

  void* fits2ast();
  void* wcs2ast();

  void putFitsCard(void*, const char*, const char*);
  void putFitsCard(void*, const char*, double);

  double calcTextAngle(double, int*);

 public:
  Grid(Frame*, int, int, const char*);
  virtual ~Grid();

  int render();
  int ps(int);

  int system() {return system_;}
  int type() {return type_;}
  const char* option() {return option_;}

  void gFlush() {}
  void gLine(int n, const float* x, const float* y)
    {(renderMode==X11) ? x11Line(n,x,y) : psLine(n,x,y);}
  void gQch(float*, float*);
  void gMark(int, const float*, const float*, int) {}
  void gText(const char* txt, float x, float y, 
	     const char* just, float upx, float upy)
    {(renderMode==X11) ? 
       x11Text(txt,x,y,just,upx,upy) : psText(txt,x,y,just,upx,upy);}
  void gTxExt(const char*, float, float, const char*, 
	      float, float, float*, float*);
  void gAttr(int attr, double value, double* old, int prim);
};

#endif
