%{
#define YYPARSE_PARAM fr
#define YYDEBUG 1

#define FR ((Frame*)fr)
#define FITS (FR->findFits(1))

#include <math.h>
#include <string.h>

#include "frame.h"
#include "fitsimage.h"
#include "util.h"
#include "vector.h"
#include "coord.h"
#include "fvcontour.h"

extern int frlex(void);
extern void frerror(const char*);

int frsign;
int frsign2;

static CoordSystem currentSystem;
static CoordSystem currentDistSystem;
static CoordFormat currentDistFormat;

static unsigned short defaultProps = Marker::SELECT |
	Marker::EDIT | Marker::MOVE | Marker::ROTATE | 
	Marker::DELETE | Marker::INCLUDE | Marker::SOURCE;
static unsigned short currentProps;
static char currentColor[16];
static int currentWidth;
static char currentFont[32];
static char currentText[80];
static int currentID;

static unsigned short propQMask;
static unsigned short propQValue;

static void setProps(unsigned short* props, unsigned short prop, int value);

%}

%union {
  double real;
  int integer;
  char str[1024];
  double vector[3];
  void* ptr;
}

%type <real> numeric
%type <integer> yesno

%type <real> angle
%type <real> optangle
%type <integer> precision
%type <real> sexagesimal
%type <vector> coord
%type <integer> coordSystem
%type <integer> coordFormat
%type <integer> canpanSystem
%type <integer> markerProperty
%type <integer> markerCallBack
%type <integer> markerFormat
%type <integer> pscolorspace
%type <integer> scaleType
%type <integer> shmType
%type <integer> colormapMode
%type <integer> contourmethod
%type <integer> incrLoad
%type <integer> mosaicType
%type <integer> compressType
%type <real> value

%token <real> REAL
%token <integer> INT
%token <str> STRING
%token <ptr> POINTER

%token <integer> HOUR
%token <integer> MINUTE
%token <real> SECOND
%token <real> DEGREE
%token <real> ARCMINUTE
%token <real> ARCSECOND
%token <real> RADIAN

%token <str> SEXSTR

%token ABOUT_
%token ACCURATE_
%token ALLOC_
%token ALIGN_
%token ALL_
%token AMPLIFIER_
%token ANGLE_
%token ANNULUS_
%token APPEND_
%token ARCMIN_
%token ARCSEC_
%token AREA_
%token ARRAY_
%token ARROW_
%token AST_
%token AVERAGE_
%token AUTO_
%token B1950_
%token BACK_
%token BBOX_
%token BEGIN_
%token BIN_
%token BITPIX_
%token BLOCK_
%token BOX_
%token BOXCIRCLE_
%token BUFFER_
%token BW_
%token CALLBACK_
%token CANVAS_
%token CENTER_
%token CHANNEL_
%token CIRCLE_
%token CIAO_
%token CLEAR_
%token CLICK_
%token CLIP_
%token COLORMAP_
%token COLOR_
%token COLORSCALE_
%token COLORSPACE_
%token COLS_
%token COMMAND_
%token CONTOUR_
%token COORDINATES_
%token COPY_
%token COUNT_
%token CURSOR_
%token CREATE_
%token CROSS_
%token CROSSHAIR_
%token CUBE_
%token CUT_
%token CMYK_
%token DATA_
%token DATAMIN_
%token DATASEC_
%token DEBUG_
%token DEGREES_
%token DEFAULT_
%token DELETE_
%token DEPTH_
%token DETECTOR_
%token DIAMOND_
%token DISTANCE_
%token DS9_
%token EDIT_
%token ECLIPTIC_
%token ELLIPSE_
%token END_
%token EQUATORIAL_
%token ERASE_
%token EXT_
%token FACTOR_
%token FALSE_
%token FAST_
%token FILE_
%token FILTER_
%token FIT_
%token FITS_
%token FITSY_
%token FIXED_
%token FK4_
%token FK5_
%token FLIP_
%token FONT_
%token FRONT_
%token FULL_
%token FUNCTION_
%token GALACTIC_
%token GET_
%token GLOBAL_
%token GRAPHICS_
%token GRAY_
%token GRID_
%token GZ_
%token HANDLE_
%token HAS_
%token HEADER_
%token HEIGHT_
%token HIDE_
%token HIGH_
%token HIGHLITE_
%token HISTEQU_
%token HISTOGRAM_
%token HORIZONTAL_
%token ICRS_
%token ID_
%token IIS_
%token IMAGE_
%token INCLUDE_
%token INCR_
%token INFO_
%token INTEGER_
%token INTERPOLATE_
%token IRAF_
%token IRAFMIN_
%token J2000_
%token KEY_
%token KEYWORD_
%token LEVEL_
%token LINE_
%token LINEAR_
%token LIST_
%token LOAD_
%token LOCAL_
%token LOG_
%token LOW_
%token MAGNIFIER_
%token MARKER_
%token MASTER_
%token MESSAGE_
%token METHOD_
%token MINMAX_
%token MMAP_
%token MOSAIC_
%token MODE_
%token MOTION_
%token MOVE_
%token N_
%token NAME_
%token NAMES_
%token NEW_
%token NO_
%token NONE_
%token NOW_
%token NUMBER_
%token OBJECT_
%token OFF_
%token ON_
%token OPTION_
%token ORIENT_
%token PAN_
%token PANDA_
%token PANNER_
%token PARAM_
%token PARSER_
%token PASTE_
%token PERF_
%token PHYSICAL_
%token PIXELS_
%token POINT_
%token POLYGON_
%token POSTSCRIPT_
%token PROJECTION_
%token PROPERTY_
%token PROS_
%token RADIUS_
%token REGION_
%token REPLACE_
%token RESAMPLE_
%token RESET_
%token RESOLUTION_
%token RGB_
%token ROTATE_
%token RULER_
%token SAMPLE_
%token SAOIMAGE_
%token SAOTNG_
%token SAVE_
%token SCAN_
%token SCIENTIFIC_
%token SCOPE_
%token SEGMENT_
%token SELECT_
%token SET_
%token SEXAGESIMAL_
%token SHARED_
%token SHIFT_
%token SHMID_
%token SHOW_
%token SIZE_
%token SLAVE_
%token SLICE_
%token SMOOTH_
%token SOCKET_
%token SOURCE_
%token SQRT_
%token SQUARED_
%token STATUS_
%token SUM_
%token SYSTEM_
%token TABLE_
%token TEXT_
%token THICK_
%token TO_
%token TRUE_
%token TYPE_
%token UNDO_
%token UNLOAD_
%token UNSELECT_
%token UPDATE_
%token USER_
%token VALUE_
%token VAR_
%token VERSION_
%token VERTEX_
%token VERTICAL_
%token WARP_
%token WCS_
%token WFPC2_
%token WIDTH_
%token X_
%token XY_
%token Y_
%token YES_
%token ZMAX_
%token ZSCALE_
%token ZOOM_

%%

command : DEBUG_ debug

	| BIN_ bin
	| CENTER_ {FR->centerCmd();}
	| CLEAR_ {FR->clearCmd();}
	| CLIP_ clip
	| COLORMAP_ colormap
	| COLORSCALE_ scaleType {FR->colorScaleCmd((Frame::ColorScaleType)$2);}
	| CONTOUR_ contour
	| CROSSHAIR_ crosshair
	| DATASEC_ yesno {FR->DATASECCmd($2);}
	| FITSY_ fitsy
	| GET_ get
	| GRID_ grid
	| HAS_ has
	| HIDE_ {FR->hideCmd();}
	| HIGHLITE_ yesno {FR->highliteCmd($2);}
	| IIS_ iis
	| LOAD_ load
	| MAGNIFIER_ magnifier
	| MARKER_ marker
	| MOSAIC_ FAST_ yesno {FR->mosaicFastCmd($3);}
	| ORIENT_ orient
	| PAN_ pan
	| PANNER_ panner
	| POSTSCRIPT_ postscript
	| RESET_ {FR->resetCmd();}
	| REGION_ region
	| ROTATE_ rotate
	| SAVE_ save
	| SET_ set
	| SHOW_ {FR->showCmd();}
	| UNLOAD_ {FR->unloadFitsCmd();}
	| UPDATE_ update
	| VERSION_ {FR->msg("Frame 1.0");}
	| WARP_ warp
	| WCS_ wcs
	| ZOOM_ zoom
	;

numeric	: REAL {$$=$1;}
	| INT {$$=$1;}
	;

debug	: yesno {yydebug=$1;}
	| AST_ yesno {DebugAST=$2;}
	| MOSAIC_ yesno {DebugMosaic=$2;}
	| PARSER_ yesno {yydebug=$2;}
	| PERF_ yesno {DebugPerf=$2;}
	| WCS_ yesno {DebugWCS=$2;}
	| GZ_ yesno {DebugGZ=$2;}
	;

yesno	: INT {$$=($1 ? 1 : 0);}

	| YES_ {$$=1;}
	| Y_ {$$=1;}
	| ON_ {$$=1;}
	| TRUE_ {$$=1;}

	| NO_ {$$=0;}
	| N_ {$$=0;}
	| OFF_ {$$=0;}
	| FALSE_ {$$=0;}
	;

precision : DEFAULT_ {$$ = Frame::DEFAULT;}
	| FIXED_ {$$ = Frame::FIXED;}
	| SCIENTIFIC_ {$$ = Frame::SCIENTIFIC;}
	| INTEGER_ {$$ = Frame::INTEGER;}
	;

optangle : /* empty */ {$$ = 0;}
	| angle {$$ = $1;}
	;

angle	: numeric {$$ = degToRad($1);} /* assume degree */
	| DEGREE {$$ = degToRad($1);}
	| RADIAN {$$=$1;}
	;

value	: numeric
	  {$$ = FITS->mapLenToRef($1, currentSystem);}
	| DEGREE
	  {$$ = FITS->mapLenToRef($1, currentSystem, DEGREES);}
	| ARCMINUTE 
	  {$$ = FITS->mapLenToRef($1, currentSystem, ARCMIN);}
	| ARCSECOND 
	  {$$ = FITS->mapLenToRef($1, currentSystem, ARCSEC);}
	;

sexagesimal: SEXSTR {$$ = parseDMS($1);}

coord	: sexagesimal sexagesimal
	{
	  Vector r;
	  if (currentSystem == GALACTIC || currentSystem == ECLIPTIC) 
	    r = Vector($1,$2);
	  else
	    r = Vector($1*360./24.,$2);

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| HOUR MINUTE SECOND {frsign2 = frsign;} INT ARCMINUTE ARCSECOND
        {
	  Vector r = Vector(hmsToDegree(frsign2,$1,$2,$3), 
	    dmsToDegree(frsign,$5,$6,$7));

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| INT ARCMINUTE ARCSECOND {frsign2 = frsign;} INT ARCMINUTE ARCSECOND
        {
	  Vector r = Vector(dmsToDegree(frsign2,$1,$2,$3),
	    dmsToDegree(frsign,$5,$6,$7));

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| numeric numeric 
	{
	  $$[0] = $1;
	  $$[1] = $2;
	  $$[2] = 1;
	}
	;

canpanSystem : CANVAS_ {$$ = CANVAS;}
	| PANNER_ {$$ = PANNER;}
	;

coordSystem : CANVAS_ {$$ = currentSystem = CANVAS;}
	| PANNER_ {$$ = currentSystem = PANNER;}
	| IMAGE_ {$$ = currentSystem = IMAGE;}
	| PHYSICAL_ {$$ = currentSystem = PHYSICAL;}
	| DETECTOR_ {$$ = currentSystem = DETECTOR;}
	| AMPLIFIER_ {$$ = currentSystem = AMPLIFIER;}
	| FK4_ {$$ = currentSystem = EQUATORIAL_FK4;}
	| B1950_ {$$ = currentSystem = EQUATORIAL_FK4;}
	| FK5_ {$$ = currentSystem = EQUATORIAL_FK5;}
	| J2000_ {$$ = currentSystem = EQUATORIAL_FK5;}
	| ICRS_ {$$ = currentSystem = ICRS;}
	| GALACTIC_ {$$ = currentSystem = GALACTIC;}
	| ECLIPTIC_ {$$ = currentSystem = ECLIPTIC;}
	| LINEAR_ {$$ = currentSystem = LINEAR;}
	;

coordFormat : PIXELS_ {$$=PIXELS;}
	| DEGREES_ {$$=DEGREES;}
	| ARCMIN_ {$$=ARCMIN;}
	| ARCSEC_ {$$=ARCSEC;}
	| SEXAGESIMAL_ {$$=SEXAGESIMAL;}
	;

distFormat : coordFormat
	{
	  switch ((CoordFormat)$1) {
	  case PIXELS:
	    currentDistSystem = IMAGE;
	    currentDistFormat = PIXELS;
	    break;
	  case DEGREES:
	  case ARCMIN:
	  case ARCSEC:
	    currentDistSystem = EQUATORIAL_FK5;
	    currentDistFormat = (CoordFormat)$1;
	    break;
	  }
	}
	| coordSystem
	{
	  switch ((CoordSystem)$1) {
	  case IMAGE:
	  case PHYSICAL:
	  case DETECTOR:
	  case AMPLIFIER:
	    currentDistSystem = (CoordSystem)$1;
	    currentDistFormat = PIXELS;
	    break;
	  case EQUATORIAL_FK4:
	  case EQUATORIAL_FK5:
	  case ICRS:
	  case GALACTIC:
	  case ECLIPTIC:
	  case LINEAR:
	    currentDistSystem = (CoordSystem)$1;
	    currentDistFormat = DEGREES;
	    break;
	  }
	}
	;

scaleType: LINEAR_ {$$ = Frame::LINEARSCALE;}
	| LOG_ {$$ = Frame::LOGSCALE;}
	| SQUARED_ {$$ = Frame::SQUAREDSCALE;}
	| SQRT_ {$$ = Frame::SQRTSCALE;}
	| HISTEQU_ {$$ = Frame::HISTEQUSCALE;}
	;	

shmType	: /* empty */ {$$ = Frame::SHMID;}
	| SHMID_ {$$ = Frame::SHMID;}
	| KEY_ {$$ = Frame::KEY;}
	;

incrLoad: /* empty */ {$$ = Frame::LOADALL;}
	| ALL_ {$$ = Frame::LOADALL;}
	| INCR_ {$$ = Frame::INCR;}
	;

mosaicType: /* empty */ {$$ = Frame::AUTO;}
	| AUTO_ {$$ = Frame::AUTO;}
	| IRAF_ {$$ = Frame::IRAF;}
	| WCS_ {$$ = Frame::WCS;}
	| WFPC2_ {$$ = Frame::WFPC2;}
	;

compressType: /* empty */ {$$ = Frame::NOCOMPRESS;}
	| NONE_ {$$ = Frame::NOCOMPRESS;}
	| GZ_ {$$ = Frame::GZ;}
	;

bin	: ABOUT_ coordSystem coord
	  {FR->binAboutCmd(Vector($3),(CoordSystem)$2);}
	| COLS_ STRING STRING {FR->binColsCmd($2,$3);}
	| FACTOR_ binFactor
	| FUNCTION_ binFunction
	| BUFFER_ SIZE_ INT {FR->binBufferSizeCmd($3);}
	| TO_ binTo
	| FILTER_ STRING {FR->binFilterCmd($2);}
	;

binFactor : numeric {FR->binFactorCmd($1);}
	| numeric ABOUT_ coordSystem coord 
	  {FR->binFactorAboutCmd($1, Vector($4), (CoordSystem)$3);}
	| TO_ numeric {FR->binFactorToCmd($2);}
	| TO_ numeric ABOUT_ coordSystem coord 
	  {FR->binFactorToAboutCmd($2, Vector($5), (CoordSystem)$4);}
	;

binFunction : AVERAGE_ {FR->binFunctionCmd(FitsHist::AVERAGE);}
	| SUM_ {FR->binFunctionCmd(FitsHist::SUM);}
	;

binTo	: FIT_ {FR->binToFitCmd();}
	| numeric ABOUT_ CENTER_
	  {FR->binCmd($1, NULL, NULL, NULL);}
	| numeric ABOUT_ CENTER_ STRING STRING STRING
	  {FR->binCmd($1, $4, $5, $6);}
	| numeric ABOUT_ coordSystem coord
	  {FR->binCmd($1, Vector($4), (CoordSystem)$3, NULL, NULL, NULL);}
	| numeric ABOUT_ coordSystem coord STRING STRING STRING
	  {FR->binCmd($1, Vector($4), (CoordSystem)$3, $5, $6, $7);}
	;

clip	: SCOPE_ clipScope
	| MODE_ clipMode
	| MINMAX_ clipMinMax
	| USER_ clipUser
	| ZSCALE_ PARAM_ numeric INT INT {FR->clipZScaleParamCmd($3,$4,$5);}
	;

clipUser: /* empty */ numeric numeric {FR->clipUserCmd($1,$2);}
	| LOW_ numeric {FR->clipUserLowCmd($2);} 
	| HIGH_ numeric {FR->clipUserHighCmd($2);} 
	;

clipScope: GLOBAL_ {FR->clipScopeCmd(Frame::GLOBAL);}
	| LOCAL_ {FR->clipScopeCmd(Frame::LOCAL);}
	;

clipMode: numeric {FR->clipModeCmd($1);}
	| MINMAX_ {FR->clipModeCmd(Frame::MINMAX);}
	| ZSCALE_ {FR->clipModeCmd(Frame::ZSCALE);}
	| ZMAX_ {FR->clipModeCmd(Frame::ZMAX);}
	| USER_ {FR->clipModeCmd(Frame::USERCLIP);}
	;

clipMinMax : MODE_ clipMinMaxMode
	| SAMPLE_ INT {FR->clipMinMaxParamCmd($2);}
	;

clipMinMaxMode : SCAN_ {FR->clipMinMaxModeCmd(Frame::SCAN);}
	| SAMPLE_ {FR->clipMinMaxModeCmd(Frame::SAMPLE);}
	| DATAMIN_ {FR->clipMinMaxModeCmd(Frame::DATAMIN);}
	| IRAFMIN_ {FR->clipMinMaxModeCmd(Frame::IRAFMIN);}
	;

colormap : INT numeric numeric INT POINTER POINTER INT 
	  {FR->colormapCmd($1, $2, $3, $4, (unsigned short*)$5, 
	    (unsigned char*)$6, $7);}
	| AREA_ INT colormapMode 
	  {FR->colormapAreaCmd($2,(Frame::ColormapMode)$3);}
	| BEGIN_ colormapBegin
	| MOTION_ INT numeric numeric INT POINTER POINTER INT
	  {FR->colormapMotionCmd($2, $3, $4, $5, (unsigned short*)$6,
	    (unsigned char*)$7, $8);}
	| END_ INT numeric numeric INT POINTER POINTER INT
	  {FR->colormapEndCmd($2, $3, $4, $5, (unsigned short*)$6,
	    (unsigned char*)$7, $8);}
	;

colormapBegin : /* empty */ {FR->colormapBeginCmd();}
	| numeric numeric {FR->colormapBeginCmd(Vector($1,$2));}
	;

colormapMode: CENTER_ {$$ = Frame::CENTER;}
	| CLICK_ {$$ = Frame::CLICK;}
	;

contour	: COLOR_ STRING {FR->contourSetColorCmd($2);}
	| COPY_ coordSystem {FR->contourCopyCmd((CoordSystem)$2);}
	| CREATE_ contourCreate
	| DELETE_ {FR->contourDeleteCmd();}
	| LOAD_ STRING INT STRING coordSystem
	  {FR->contourLoadCmd($2, $3, $4, (CoordSystem)$5);}
	| PASTE_ STRING INT POINTER coordSystem
	  {FR->contourPasteCmd($2, $3, $4, (CoordSystem)$5);}
	| SAVE_ STRING coordSystem
	  {FR->contourSaveCmd($2, (CoordSystem)$3);}
	| WIDTH_ INT {FR->contourSetLineWidthCmd($2);}
	;

contourCreate : STRING INT INT numeric contourmethod 
	  {FR->contourCreateCmd($1,$2,$3,$4,$5);}
	| STRING INT INT numeric contourmethod scaleType
	  {FR->contourCreateCmd($1,$2,$3,$4,$5,(Frame::ColorScaleType)$6);}
	| STRING INT INT numeric contourmethod numeric numeric scaleType
	  {FR->contourCreateCmd($1,$2,$3,$4,$5,$6,$7,
	    (Frame::ColorScaleType)$8);}
	| STRING INT numeric contourmethod STRING
	  {FR->contourCreateCmd($1,$2,$3,$4,$5);}
	| POLYGON_ markerProperties
	  {FR->createContourPolygonCmd(currentColor,currentWidth, 
	    currentFont, currentText, currentProps, NULL);}
	;

contourmethod : SMOOTH_ {$$ = FVContour::SMOOTH;}
	| BLOCK_ {$$ = FVContour::BLOCK;}
	;

crosshair: BEGIN_ MOTION_ coordSystem coord 
	  {FR->crosshairBeginCmd(Vector($4), (CoordSystem)$3);}
	| MOTION_ coordSystem coord
	  {FR->crosshairMotionCmd(Vector($3), (CoordSystem)$2);}
	| coordSystem coord {FR->crosshairCmd(Vector($2), (CoordSystem)$1);}
	| yesno {FR->crosshairCmd($1);}
	| WARP_ numeric numeric {FR->crosshairWarpCmd(Vector($2,$3));}
	;

fitsy	: HAS_ EXT_ STRING {FR->fitsyHasExtCmd($3);}
	;

get	: BIN_ getBin
	| BITPIX_ {FR->getBitpixCmd();}
	| CLIP_ getClip
	| COLORMAP_ {FR->getColormapCmd();}
	| COLORSCALE_ {FR->getColorScaleCmd();}
	| CONTOUR_ getContour
	| COORDINATES_ getCoord
	| CROSSHAIR_ getCrosshair
	| CURSOR_ getCursor
	| DATA_ getData
	| DATASEC_ {FR->getDATASECCmd();}
	| FITS_ getFits
	| GRID_ getGrid
	| HEIGHT_ {FR->getHeightCmd();}
	| HISTOGRAM_ STRING STRING INT {FR->getHistogramCmd($2,$3,$4);}
	| HORIZONTAL_ CUT_ getHorz
	| IIS_ getiis
	| INFO_ getInfo
	| MINMAX_ {FR->getMinMaxCmd();}
	| MARKER_ markerGet
	| NAME_ {FR->getNameCmd();}
	| ORIENT_ {FR->getOrientCmd();}
	| PAN_ getCursor
	| PIXELS_ TABLE_ canpanSystem numeric numeric INT STRING
	  {FR->getPixelTableCmd(Vector($4,$5), (CoordSystem)$3, $6, $7);}
	| ROTATE_ getRotate
	| VALUE_ canpanSystem numeric numeric
	  {FR->getValueCmd(Vector($3,$4),(CoordSystem)$2);}
	| VERTICAL_ CUT_ getVert
	| WCS_ getWCS
	| WIDTH_ {FR->getWidthCmd();}
	| ZOOM_ getZoom
	;

getBin	: FACTOR_ {FR->getBinFactorCmd();}
	| FUNCTION_ {FR->getBinFunctionCmd();}
	| BUFFER_ SIZE_ {FR->getBinBufferSizeCmd();}
	| CURSOR_ {FR->getBinCursorCmd();}
	| FILTER_ {FR->getBinFilterCmd();}
	| COLS_ {FR->getBinColsCmd();}
	| LIST_ {FR->getBinListCmd();}
	;

getClip	: getClipMode
	| SCOPE_ {FR->getClipScopeCmd();}
	| MODE_ {FR->getClipModeCmd();}
	| MINMAX_ getClipMinMax
	| USER_ LEVEL_ {FR->getClipUserCmd();}
	| ZSCALE_ PARAM_ {FR->getClipZScaleParamCmd();}
	;

getClipMode: /* empty */ {FR->getClipCmd();}
	| numeric {FR->getClipCmd($1);}
	| MINMAX_ {FR->getClipCmd(Frame::MINMAX);}
	| ZSCALE_ {FR->getClipCmd(Frame::ZSCALE);}
	| ZMAX_ {FR->getClipCmd(Frame::ZMAX);}
	| USER_ {FR->getClipCmd(Frame::USERCLIP);}
	;

getClipMinMax : MODE_ {FR->getClipMinMaxModeCmd();}
	| SAMPLE_ {FR->getClipMinMaxSampleCmd();}
	;

getContour: COLOR_ {FR->getContourColorNameCmd();}
	| LEVEL_ getContourLevel
	| METHOD_ {FR->getContourMethodCmd();}
	| WIDTH_ {FR->getContourLineWidthCmd();}
	;

getContourLevel : /* empty */ {FR->getContourLevelCmd();}
	| INT numeric numeric scaleType 
	  {FR->getContourLevelCmd($1,$2,$3,(Frame::ColorScaleType)$4);}
	;

getCoord : canpanSystem numeric numeric coordSystem 
	  {FR->getCoordCmd(Vector($2,$3), (CoordSystem)$1, (CoordSystem)$4,
	    PIXELS, Frame::DEFAULT);}
	| canpanSystem numeric numeric coordSystem coordFormat
	  {FR->getCoordCmd(Vector($2,$3), (CoordSystem)$1, (CoordSystem)$4,
	    (CoordFormat)$5, Frame::DEFAULT);}
	;

getCrosshair: coordSystem 
	  {FR->getCrosshairCmd((CoordSystem)$1, PIXELS, Frame::DEFAULT);}
	| coordSystem precision 
	  {FR->getCrosshairCmd((CoordSystem)$1, PIXELS, (Frame::Precision)$2);}
	| coordSystem coordFormat
	  {FR->getCrosshairCmd((CoordSystem)$1, (CoordFormat)$2, 
	    Frame::DEFAULT);}
	| STATUS_ {FR->getCrosshairStatusCmd();}
	;

getCursor : coordSystem 
	  {FR->getCursorCmd((CoordSystem)$1, PIXELS, Frame::DEFAULT);}
	| coordSystem coordFormat
	  {FR->getCursorCmd((CoordSystem)$1, (CoordFormat)$2, Frame::DEFAULT);}
	| coordSystem precision 
	  {FR->getCursorCmd((CoordSystem)$1, PIXELS, (Frame::Precision)$2);}
	;

getData	: coordSystem coord value value STRING
	  {FR->getDataValuesCmd(1, Vector($2), (CoordSystem)$1,
	    Vector($3,$4), $5);}
	| INT coordSystem coord value value STRING
	  {FR->getDataValuesCmd($1, Vector($3), (CoordSystem)$2,
	    Vector($4,$5), $6);}
	;

getHorz	: STRING STRING {FR->getHorzCutCmd($1,$2);}
	| STRING STRING numeric numeric 
	  {FR->getHorzCutCmd($1,$2,Vector($3,$4));}
	;

getInfo	: STRING {FR->getInfoCmd($1);}
 	| coordSystem numeric numeric coordSystem coordFormat STRING
	  {FR->getInfoCmd(Vector($2,$3), (CoordSystem)$1, 
	    (CoordSystem)$4, (CoordFormat)$5, $6);}
	;

getiis	: POINTER INT INT INT INT {FR->iisGetCmd((char*)$1,$2,$3,$4,$5);}
	| CURSOR_ {FR->iisGetCursorCmd();}
	| WIDTH_ {FR->getWidthCmd();}
	| HEIGHT_ {FR->getHeightCmd();}
	;

getFits	: COUNT_ {FR->getFitsCountCmd();}
	| FILE_ NAME_ getFitsFileName
	| FILE_ NAMES_ {FR->getFitsFileNamesCmd(Frame::BASIC);}
	| FULL_ FILE_ NAME_ getFitsFullFileName
	| FULL_ FILE_ NAMES_ {FR->getFitsFileNamesCmd(Frame::FULL);}
	| HEADER_ INT {FR->getFitsHeaderCmd($2);}
	| HEADER_ INT KEYWORD_ STRING {FR->getFitsHeaderKeywordCmd($2,$4);}
	| OBJECT_ NAME_ getFitsObjectName
	| MASTER_ {FR->getFitsMasterCmd();}
	| SIZE_ distFormat
	  {FR->getFitsSizeCmd(currentDistSystem, currentDistFormat);}
	| DEPTH_ {FR->getFitsDepthCmd();}
	| SLICE_ {FR->getFitsSliceCmd();}
	;

getFitsFileName: /* empty */ {FR->getFitsFileNameCmd(1, Frame::BASIC);}
	| NUMBER_ INT {FR->getFitsFileNameCmd($2, Frame::BASIC);}
	| canpanSystem numeric numeric
	{FR->getFitsFileNameCmd(Vector($2,$3), (CoordSystem)$1, Frame::BASIC);}
	;

getFitsFullFileName: /* empty */ {FR->getFitsFileNameCmd(1, Frame::FULL);}
	| NUMBER_ INT {FR->getFitsFileNameCmd($2, Frame::FULL);}
	| canpanSystem numeric numeric
	{FR->getFitsFileNameCmd(Vector($2,$3), (CoordSystem)$1, Frame::FULL);}
	;

getFitsObjectName: /* empty */ {FR->getFitsObjectNameCmd(1);}
	| NUMBER_ INT {FR->getFitsObjectNameCmd($2);}
	;

getGrid : SYSTEM_ {FR->getGridSystemCmd();}
	| TYPE_ {FR->getGridTypeCmd();}
	| OPTION_ {FR->getGridOptionCmd();}
	;

getRotate : /* empty */ {FR->getRotateCmd(Frame::DEFAULT);}
	| precision {FR->getRotateCmd((Frame::Precision)$1);}
	;

getVert	: STRING STRING {FR->getVertCutCmd($1,$2);}
	| STRING STRING numeric numeric 
	  {FR->getVertCutCmd($1,$2,Vector($3,$4));}
	;

getWCS	: ALIGN_ {FR->getWCSAlignCmd();}
	| ZOOM_ getWCSZoom
	| FLIP_ {FR->getWCSFlipCmd();}
	| SYSTEM_ {FR->getWCSSystemCmd();}
	;

getWCSZoom : /* empty */ {FR->getWCSZoomCmd(Frame::DEFAULT);}
	| precision {FR->getWCSZoomCmd((Frame::Precision)$1);}
	;

getZoom	: /* empty */ {FR->getZoomCmd(Frame::DEFAULT);}
	| precision {FR->getZoomCmd((Frame::Precision)$1);}
	;

grid	: CREATE_ coordSystem INT STRING {FR->gridCmd((CoordSystem)$2,$3,$4);}
	| DELETE_ {FR->gridDeleteCmd();}
	;

has	: AMPLIFIER_ {FR->hasAmplifierCmd();}
	| BIN_ {FR->hasBinCmd();}
	| CONTOUR_ {FR->hasContourCmd();}
	| DATAMIN_ {FR->hasDATAMINCmd();}
	| DATASEC_ {FR->hasDATASECCmd();}
	| DETECTOR_ {FR->hasDetectorCmd();}
	| FITS_ hasFits
	| GRID_ {FR->hasGridCmd();}
	| IIS_ {FR->hasIISCmd();}
	| IRAFMIN_ {FR->hasIRAFMINCmd();}
	| MARKER_ hasMarker
	| PHYSICAL_ {FR->hasPhysicalCmd();}
	| WCS_ hasWCS
	;

hasFits	: {FR->hasFitsCmd();}
	| CUBE_ {FR->hasFitsCubeCmd();}
	| EXT_ INT {FR->hasFitsExtCmd($2);}
	| IMAGE_ {FR->hasFitsImageCmd();}
	| MOSAIC_ {FR->hasFitsMosaicCmd();}
	| TABLE_ {FR->hasFitsTableCmd();}
	;

hasMarker : SELECT_ {FR->hasMarkerSelectedCmd();}
	| PASTE_ {FR->hasMarkerPasteCmd();}
	| UNDO_ {FR->hasMarkerUndoCmd();}
	;

hasWCS	: /* empty */ {FR->hasWCSCmd();}
	| EQUATORIAL_ {FR->hasWCSEquCmd();}
	| LINEAR_ {FR->hasWCSLinearCmd();}
	;

iis	: NEW_ INT INT {FR->iisCmd($2,$3);}
	| ERASE_ {FR->iisEraseCmd();}
	| MESSAGE_ STRING {FR->iisMessageCmd($2);}
	| CURSOR_ iiscursor
	| SET_ POINTER INT INT INT INT
	  {FR->iisSetCmd((const char*)$2,$3,$4,$5,$6);}
	| UPDATE_ {FR->iisUpdateCmd();}
	| WCS_ numeric numeric numeric numeric 
	  numeric numeric numeric numeric INT
	    {FR->iisWCSCmd(Matrix($2,$3,$4,$5,$6,$7),Vector($8,$9),$10);}
	;

iiscursor: INT INT coordSystem
	  {FR->iisSetCursorCmd(Vector($1,$2),(CoordSystem)$3);}
	| MODE_ yesno {FR->iisCursorModeCmd($2);}
	;

load	: STRING incrLoad {FR->loadFitsMMapCmd($1,(Frame::LoadMethod)$2);}
	| ARRAY_ loadArr
	| FITS_ loadFits
	| FITS_ MOSAIC_ loadFitsMosaic
	| FITS_ MOSAIC_ IMAGE_ loadFitsMosaicImage
	| INCR_ loadIncr
	;

loadArr : STRING {FR->loadArrMMapCmd($1);}
	| ALLOC_ STRING {FR->loadArrAllocCmd($2);}
	| ALLOC_ GZ_ STRING {FR->loadArrAllocGZCmd($3);}
	| CHANNEL_ STRING {FR->loadArrChannelCmd($2, NULL);}
	| CHANNEL_ STRING STRING {FR->loadArrChannelCmd($2, $3);}
	| MMAP_ STRING {FR->loadArrMMapCmd($2);}
	| SHARED_ loadArrShm
	| SOCKET_ loadArrSocket
	| VAR_ STRING {FR->loadArrVarCmd($2, NULL);}
	| VAR_ STRING STRING {FR->loadArrVarCmd($2, $3);}
	;

loadArrShm: shmType INT INT
	  {FR->loadArrShareCmd((Frame::ShmType)$1, $2, $3, NULL);}
	| shmType INT INT STRING
	  {FR->loadArrShareCmd((Frame::ShmType)$1, $2, $3, $4);}
	;

loadArrSocket: INT {FR->loadArrSocketCmd($1, NULL);}
	| INT STRING {FR->loadArrSocketCmd($1, $2);}
	| GZ_ INT {FR->loadArrSocketGZCmd($2, NULL);}
	| GZ_ INT STRING {FR->loadArrSocketGZCmd($2, $3);}
	;

loadFits : STRING incrLoad {FR->loadFitsMMapCmd($1,(Frame::LoadMethod)$2);}
	| ALLOC_ loadFitsAlloc
	| CHANNEL_ loadFitsChannel
	| MMAP_ STRING incrLoad 
	  {FR->loadFitsMMapCmd($2,(Frame::LoadMethod)$3);}
	| SHARED_ loadFitsShm
	| SOCKET_ loadFitsSocket
	| SLAVE_ STRING POINTER {FR->loadFitsSlaveCmd($2,(FitsFile*)$3);}
	| VAR_ loadFitsVar
	;

loadFitsAlloc: STRING {FR->loadFitsAllocCmd($1);}
	| GZ_ STRING {FR->loadFitsAllocGZCmd($2);}
	;

loadFitsChannel: STRING {FR->loadFitsChannelCmd($1, NULL);}
	| STRING STRING {FR->loadFitsChannelCmd($1, $2);}
	;

loadFitsShm: shmType INT INT incrLoad
	  {FR->loadFitsShareCmd((Frame::ShmType)$1, $2, $3, NULL, 
	    (Frame::LoadMethod)$4);}
	| shmType INT INT STRING incrLoad
	  {FR->loadFitsShareCmd((Frame::ShmType)$1, $2, $3, $4, 
	    (Frame::LoadMethod)$5);}
	;

loadFitsSocket: INT {FR->loadFitsSocketCmd($1, NULL);}
	| INT STRING {FR->loadFitsSocketCmd($1, $2);}
	| GZ_ INT {FR->loadFitsSocketGZCmd($2, NULL);}
	| GZ_ INT STRING {FR->loadFitsSocketGZCmd($2, $3);}
	;

loadFitsVar: STRING incrLoad 
	  {FR->loadFitsVarCmd($1, NULL, (Frame::LoadMethod)$2);}
	| STRING STRING incrLoad 
	  {FR->loadFitsVarCmd($1, $2, (Frame::LoadMethod)$3);}
	;

loadFitsMosaic : STRING mosaicType incrLoad 
	  {FR->loadMosaicMMapCmd($1, (Frame::MosaicType)$2,
	    (Frame::LoadMethod)$3);}
	| ALLOC_ loadFitsMosaicAlloc
	| CHANNEL_ loadFitsMosaicChannel
	| MMAP_ STRING mosaicType incrLoad 
	  {FR->loadMosaicMMapCmd($2, (Frame::MosaicType)$3, 
	    (Frame::LoadMethod)$4);}
	| SHARED_ loadFitsMosaicShm
	| SOCKET_ loadFitsMosaicSocket
	| VAR_ loadFitsMosaicVar
	;

loadFitsMosaicAlloc: STRING mosaicType 
	  {FR->loadMosaicAllocCmd($1, (Frame::MosaicType)$2);}
	| GZ_ STRING mosaicType 
	  {FR->loadMosaicAllocGZCmd($2, (Frame::MosaicType)$3);}
	;

loadFitsMosaicChannel: STRING mosaicType
	  {FR->loadMosaicChannelCmd($1, NULL, (Frame::MosaicType)$2);}
	| STRING STRING mosaicType 
	  {FR->loadMosaicChannelCmd($1, $2, (Frame::MosaicType)$3);}
	;

loadFitsMosaicShm: shmType INT INT mosaicType incrLoad
	  {FR->loadMosaicShareCmd((Frame::ShmType)$1, $2, $3, NULL, 
	    (Frame::MosaicType)$4, (Frame::LoadMethod)$5);}
	| shmType INT INT STRING mosaicType incrLoad
	  {FR->loadMosaicShareCmd((Frame::ShmType)$1, $2, $3, $4, 
	    (Frame::MosaicType)$5, (Frame::LoadMethod)$6);}
	;

loadFitsMosaicSocket: INT mosaicType 
	  {FR->loadMosaicSocketCmd($1, NULL, (Frame::MosaicType)$2);}
	| INT STRING mosaicType 
	  {FR->loadMosaicSocketCmd($1, $2, (Frame::MosaicType)$3);}
	| GZ_ INT mosaicType 
	  {FR->loadMosaicSocketGZCmd($2, NULL, (Frame::MosaicType)$3);}
	| GZ_ INT STRING mosaicType 
	  {FR->loadMosaicSocketGZCmd($2, $3, (Frame::MosaicType)$4);}
	;

loadFitsMosaicVar: STRING mosaicType incrLoad 
	  {FR->loadMosaicVarCmd($1, NULL, (Frame::MosaicType)$3, 
	    (Frame::LoadMethod)$3);}
	| STRING STRING mosaicType incrLoad 
	  {FR->loadMosaicVarCmd($1, $2, (Frame::MosaicType)$3,
	    (Frame::LoadMethod)$4);} 
	;

loadFitsMosaicImage: STRING mosaicType incrLoad 
	  {FR->loadMosaicImageMMapCmd($1, (Frame::MosaicType)$2, 
	    (Frame::LoadMethod)$3);}
	| ALLOC_ loadFitsMosaicImageAlloc
	| CHANNEL_ loadFitsMosaicImageChannel
	| MMAP_ STRING mosaicType incrLoad 
	  {FR->loadMosaicImageMMapCmd($2, (Frame::MosaicType)$3,
	    (Frame::LoadMethod)$4);}
	| SHARED_ loadFitsMosaicImageShm
	| SOCKET_ loadFitsMosaicImageSocket
	| VAR_ loadFitsMosaicImageVar
	;

loadFitsMosaicImageAlloc: STRING mosaicType 
	  {FR->loadMosaicImageAllocCmd($1, (Frame::MosaicType)$2);}
	| GZ_ STRING mosaicType 
	  {FR->loadMosaicImageAllocGZCmd($2, (Frame::MosaicType)$3);}
	;

loadFitsMosaicImageChannel: STRING mosaicType
	  {FR->loadMosaicImageChannelCmd($1, NULL, (Frame::MosaicType)$2);}
	| STRING STRING mosaicType
	  {FR->loadMosaicImageChannelCmd($1, $2, (Frame::MosaicType)$3);}
	;

loadFitsMosaicImageShm: shmType INT INT mosaicType incrLoad
	  {FR->loadMosaicImageShareCmd((Frame::ShmType)$1, $2, $3, NULL,
	    (Frame::MosaicType)$4, (Frame::LoadMethod)$5);}
	| shmType INT INT STRING mosaicType incrLoad
	  {FR->loadMosaicImageShareCmd((Frame::ShmType)$1, $2, $3, $4, 
	    (Frame::MosaicType)$5, (Frame::LoadMethod)$6);}
	;

loadFitsMosaicImageSocket: INT mosaicType 
	  {FR->loadMosaicImageSocketCmd($1, NULL, (Frame::MosaicType)$2);}
	| INT STRING mosaicType
	  {FR->loadMosaicImageSocketCmd($1, $2, (Frame::MosaicType)$3);}
	| GZ_ INT mosaicType
	  {FR->loadMosaicImageSocketGZCmd($2, NULL, (Frame::MosaicType)$3);}
	| GZ_ INT STRING mosaicType
	  {FR->loadMosaicImageSocketGZCmd($2, $3, (Frame::MosaicType)$4);}
	;

loadFitsMosaicImageVar: STRING mosaicType incrLoad 
	  {FR->loadMosaicImageVarCmd($1, NULL, (Frame::MosaicType)$2,
	    (Frame::LoadMethod)$3);}
	| STRING STRING mosaicType incrLoad 
	  {FR->loadMosaicImageVarCmd($1, $2, (Frame::MosaicType)$3,
	    (Frame::LoadMethod)$4);}
	;

loadIncr: DATA_ INT INT INT INT INT {FR->loadIncrDataCmd($2,$3,$4,$5,$6);}
	| MINMAX_ INT INT INT INT INT {FR->loadIncrMinMaxCmd($2,$3,$4,$5,$6);}
	| END_ {FR->loadIncrEndCmd();}
	;

magnifier: yesno {FR->magnifierCmd($1);}
	| GRAPHICS_ yesno {FR->magnifierGraphicsCmd($2);}
	| CURSOR_ yesno {FR->magnifierCursorCmd($2);}
	| MODE_ magnifierMode
	| STRING INT INT {FR->magnifierCmd($1, $2, $3);}
	| UPDATE_ {FR->updateMagnifierCmd();}
	| UPDATE_ numeric numeric {FR->updateMagnifierCmd(Vector($2, $3));}
	| ZOOM_ numeric {FR->magnifierZoomCmd($2);}
	;

magnifierMode: FAST_ {FR->magnifierModeCmd(Frame::FAST);}
	| ACCURATE_ {FR->magnifierModeCmd(Frame::ACCURATE);}
	;

marker	: COLOR_ STRING {FR->markerColorCmd($2);}
	| COPY_ {FR->markerCopyCmd();}
	| COMMAND_ markerFormat STRING
	  {FR->markerCommandCmd((Frame::MarkerFormat)$2,$3);}
	| CREATE_ markerCreate
	| CUT_ {FR->markerCutCmd();}
	| DELETE_ {FR->markerDeleteCmd();}
	| DELETE_ ALL_ {FR->markerDeleteAllCmd();}
	| EDIT_ markerEdit
	| FONT_ STRING {FR->markerFontCmd($2);}
	| INT ANGLE_ angle coordSystem
	  {FR->markerAngleCmd($1,$3,(CoordSystem)$4);}
	| INT ANNULUS_ RADIUS_ numeric numeric INT distFormat
	  {FR->markerAnnulusRadiusCmd($1, $4, $5, $6,
	    currentDistSystem, currentDistFormat);}
	| INT ANNULUS_ RADIUS_ STRING distFormat
	  {FR->markerAnnulusRadiusCmd($1, $4, 
	    currentDistSystem, currentDistFormat);}
	| INT BOX_ ANNULUS_ SIZE_ numeric numeric numeric INT distFormat
	  {FR->markerBoxAnnulusSizeCmd($1, Vector($5, $6),
	    Vector($7, $7*$6/$5), $8,
	    currentDistSystem, currentDistFormat);}
	| INT BOX_ ANNULUS_ SIZE_ STRING distFormat
	  {FR->markerBoxAnnulusSizeCmd($1, $5, 
	    currentDistSystem, currentDistFormat);}
	| INT BOX_ SIZE_ numeric numeric distFormat
	  {FR->markerBoxSizeCmd($1, Vector($4,$5), 
	    currentDistSystem, currentDistFormat);}
	| INT CALLBACK_ markerCallBack STRING STRING
	  {FR->markerCallBackCmd($1,(Marker::Callback)$3,$4,$5);}
	| INT CIRCLE_ RADIUS_ numeric distFormat
	  {FR->markerCircleRadiusCmd($1, $4, 
	    currentDistSystem, currentDistFormat);}
	| INT COLOR_ STRING {FR->markerColorCmd($1,$3);}
	| INT CREATE_ ANNULUS_ RADIUS_ numeric numeric
	  {FR->markerAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ BOX_ ANNULUS_ SIZE_ numeric numeric
	  {FR->markerBoxAnnulusCreateSizeCmd($1,Vector($6,$7));}
	| INT CREATE_ ELLIPSE_ ANNULUS_ RADIUS_ numeric numeric
	  {FR->markerEllipseAnnulusCreateRadiusCmd($1,Vector($6,$7));}
	| INT CREATE_ PANDA_ ANGLE_ numeric numeric
	  {FR->markerPandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ PANDA_ RADIUS_ numeric numeric
	  {FR->markerPandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ POLYGON_ VERTEX_ INT numeric numeric
	  {FR->markerPolygonCreateVertexCmd($1,$5,Vector($6,$7));}
	| INT DELETE_ {FR->markerDeleteCmd($1);}
	| INT DELETE_ ANNULUS_ RADIUS_ INT 
	  {FR->markerAnnulusDeleteRadiusCmd($1,$5);}
	| INT DELETE_ BOX_ ANNULUS_ SIZE_ INT 
	  {FR->markerBoxAnnulusDeleteSizeCmd($1,$6);}
	| INT DELETE_ ELLIPSE_ ANNULUS_ RADIUS_ INT 
	  {FR->markerEllipseAnnulusDeleteRadiusCmd($1,$6);}
	| INT DELETE_ CALLBACK_ markerCallBack STRING
	  {FR->markerDeleteCallBackCmd($1,(Marker::Callback)$4,$5);}
	| INT DELETE_ PANDA_ INT {FR->markerPandaDeleteCmd($1,$4);}
	| INT DELETE_ POLYGON_ VERTEX_ INT 
	  {FR->markerPolygonDeleteVertexCmd($1,$5);}
	| INT EDIT_ BEGIN_ INT {FR->markerEditBeginCmd($1,$4);}
	| INT ELLIPSE_ RADIUS_ numeric numeric distFormat
	  {FR->markerEllipseRadiusCmd($1, Vector($4, $5),
	    currentDistSystem, currentDistFormat);}
	| INT ELLIPSE_ ANNULUS_ RADIUS_ numeric numeric numeric INT distFormat
	  {FR->markerEllipseAnnulusRadiusCmd($1, Vector($5,$6),
	    Vector($7,$7*$6/$5), $8,
	    currentDistSystem, currentDistFormat);}
	| INT ELLIPSE_ ANNULUS_ RADIUS_ STRING distFormat
	  {FR->markerEllipseAnnulusRadiusCmd($1, $5, 
	    currentDistSystem, currentDistFormat);}
	| INT FONT_ STRING {FR->markerFontCmd($1,$3);}
	| INT LINE_ ARROW_ yesno yesno {FR->markerLineArrowCmd($1,$4,$5);}
	| INT LINE_ POINT_ coordSystem coord coord
	  {FR->markerLineCmd($1, Vector($5), Vector($6), (CoordSystem)$4);}
	| INT MOVE_ {currentID = $1} markerMoveID
	| INT PANDA_ angle angle INT numeric numeric INT distFormat
	  {FR->markerPandaEditCmd($1, $3, $4, $5, $6, $7, $8,
	    currentDistSystem, currentDistFormat);}
	| INT PANDA_ STRING STRING coordSystem distFormat
	  {FR->markerPandaEditCmd($1, $3, $4, (CoordSystem)$5,
	    currentDistSystem, currentDistFormat);}
	| INT PROJECTION_ coordSystem coord coord numeric INT distFormat
	  {FR->markerProjectionCmd($1,Vector($4),Vector($5),(CoordSystem)$3,
	    $6, $7, currentDistSystem, currentDistFormat);}
	| INT PROPERTY_ markerProperty yesno {FR->markerPropertyCmd($1,$3,$4);}
	| INT ROTATE_ BEGIN_ {FR->markerRotateBeginCmd($1);}
	| INT RULER_ POINT_ coordSystem coord coord
	  {FR->markerRulerPointCmd($1,Vector($5),Vector($6),(CoordSystem)$4);}
	| INT RULER_ SYSTEM_ coordSystem distFormat
	  {FR->markerRulerSystemCmd($1, (CoordSystem)$4,
	    currentDistSystem, currentDistFormat);}
	| INT SELECT_ {FR->markerSelectCmd($1);}
	| INT TEXT_ STRING {FR->markerTextCmd($1,$3);}
	| INT UNSELECT_ {FR->markerUnselectCmd($1);}
	| INT WIDTH_ INT {FR->markerLineWidthCmd($1,$3);}
	| LIST_ markerList
	| LOAD_ markerLoad
	| MOVE_ markerMoveSelected
	| PROPERTY_ markerProperty yesno {FR->markerPropertyCmd($2,$3);}
	| PROPERTY_ markerProperty yesno numeric numeric
	  {FR->markerPropertyCmd($2,$3,Vector($4,$5));}
	| ROTATE_ BEGIN_ numeric numeric 
	  {FR->markerRotateBeginCmd(Vector($3,$4));}
	| ROTATE_ MOTION_ numeric numeric INT
	  {FR->markerRotateMotionCmd(Vector($3,$4),$5);}
	| ROTATE_ END_ {FR->markerRotateEndCmd();}
	| SAVE_ STRING markerFormat coordSystem 
	  {FR->markerSaveCmd($2,(Frame::MarkerFormat)$3,
	    (CoordSystem)$4,DEGREES);}
	| SAVE_ STRING markerFormat coordSystem coordFormat
	  {FR->markerSaveCmd($2,(Frame::MarkerFormat)$3,
	    (CoordSystem)$4,(CoordFormat)$5);}
	| SELECT_ numeric numeric {FR->markerSelectCmd(Vector($2,$3));}
	| SELECT_ ALL_ {FR->markerSelectAllCmd();}
	| SELECT_ SHIFT_ numeric numeric 
	  {FR->markerSelectToggleCmd(Vector($3,$4));}
	| PASTE_ {FR->markerPasteCmd();}
	| UNDO_ {FR->markerUndoCmd();}
	| UNSELECT_ ALL_ {FR->markerUnselectAllCmd();}
	| WIDTH_ INT {FR->markerLineWidthCmd($2);}
	;

markerCallBack : SELECT_ {$$ = Marker::SELECTCB;}
	| EDIT_ {$$ = Marker::EDITCB;}
	| MOVE_  {$$ = Marker::MOVECB;}
	| ROTATE_ {$$ = Marker::ROTATECB;}
	| DELETE_ {$$ = Marker::DELETECB;}
	| TEXT_ {$$ = Marker::TEXTCB;}
	| COLOR_ {$$ = Marker::COLORCB;}
	| WIDTH_ {$$ = Marker::LINEWIDTHCB;}
	| PROPERTY_ {$$ = Marker::PROPERTYCB;}
	| FONT_ {$$ = Marker::FONTCB;}
	;

markerList : markerFormat coordSystem coordFormat yesno initQuery markerQuery
	  {FR->markerListCmd((Frame::MarkerFormat)$1, (CoordSystem)$2,
	    (CoordFormat)$3, $4, propQMask, propQValue);}
	;

initQuery: /* empty */ {propQMask=0; propQValue=0;}

markerQuery: /* empty */
	| queries
	;

queries : queries query
	| query
	;

query	: markerProperty '=' yesno 
	  {propQMask |= $1; setProps(&propQValue,$1,$3);}
	;

markerLoad : markerFormat STRING
	  {FR->markerLoadCmd((Frame::MarkerFormat)$1,$2);}
	| markerFormat STRING coordSystem 
	  {FR->markerLoadCmd((Frame::MarkerFormat)$1,$2,(CoordSystem)$3);}
	| markerFormat INT
	  {FR->markerLoadCmd((Frame::MarkerFormat)$1,$2);}
	| markerFormat INT coordSystem
	  {FR->markerLoadCmd((Frame::MarkerFormat)$1,$2,(CoordSystem)$3);}
	| FITS_ STRING {FR->markerLoadFitsCmd($2);}
	;

markerProperty : NONE_ {$$ = Marker::NONE;}
	| SELECT_ {$$ = Marker::SELECT;}
	| EDIT_ {$$ = Marker::EDIT;}
	| MOVE_  {$$ = Marker::MOVE;}
	| ROTATE_ {$$ = Marker::ROTATE;}
	| DELETE_ {$$ = Marker::DELETE;}
	| INCLUDE_ {$$ = Marker::INCLUDE;}
	| SOURCE_ {$$ = Marker::SOURCE;}
	| FIXED_ {$$ = Marker::FIXED;}
	;

markerProperties : /* empty */ markerInitProp
	| markerInitProp markerProps
	;

markerInitProp : {
	  strncpy(currentFont, "helvetica 10 normal", 32);
	  strncpy(currentColor, "green", 16);
	  currentWidth = 1;
	  strncpy(currentText, "", 80);
	  currentProps = defaultProps;
	}
	;

markerProps : markerProps markerProp
	| markerProp
	;

markerProp : markerProperty '=' yesno {setProps(&currentProps,$1,$3);}
	| COLOR_ '=' STRING {strncpy(currentColor,$3,16);}
	| WIDTH_ '=' INT {currentWidth = $3;}
	| FONT_ '=' STRING {strncpy(currentFont,$3,32);}
	| TEXT_ '=' STRING {strncpy(currentText,$3,80);}
	;

markerCreate : ANNULUS_ numeric numeric numeric numeric INT markerProperties
	  {FR->createAnnulusCmd(FITS->map(Vector($2,$3),CANVAS,REF),$4,$5,$6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| PANDA_ numeric numeric angle angle INT
	    numeric numeric INT markerProperties
	  {FR->createPandaCmd(FITS->map(Vector($2,$3),CANVAS,REF),
	    $4,$5,$6,$7,$8,$9,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| CIRCLE_ numeric numeric numeric markerProperties
	  {FR->createCircleCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),
	    FITS->mapLenToRef($4,CANVAS,PIXELS),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}

	| ELLIPSE_ numeric numeric numeric numeric optangle markerProperties
	  {FR->createEllipseCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF), 
	    FITS->mapLenToRef(Vector($4,$5),CANVAS),$6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| ELLIPSE_ ANNULUS_ numeric numeric numeric numeric numeric INT 
	    optangle markerProperties
	  {FR->createEllipseAnnulusCmd(
	    FITS->map(Vector($3,$4),CANVAS,REF),
	    FITS->mapLenToRef(Vector($5,$6),CANVAS,PIXELS), 
	    FITS->mapLenToRef(Vector($7,$7*$5/$6),CANVAS,PIXELS),$8,$9,
	    currentColor,currentWidth,currentFont, 
	    currentText,currentProps,NULL);}

	| BOX_ numeric numeric numeric numeric optangle markerProperties
	  {FR->createBoxCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),
	    FITS->mapLenToRef(Vector($4,$5),CANVAS),$6,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| BOX_ ANNULUS_ numeric numeric numeric numeric numeric INT 
	    optangle markerProperties
	  {FR->createBoxAnnulusCmd(
	    FITS->map(Vector($3,$4),CANVAS,REF),
	    FITS->mapLenToRef(Vector($5,$6),CANVAS,PIXELS), 
	    FITS->mapLenToRef(Vector($7,$7*$5/$6),CANVAS,PIXELS),$8,$9,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}

	| POLYGON_ numeric numeric numeric numeric markerProperties
	  {FR->createPolygonCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF), Vector($4,$5),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| LINE_ numeric numeric numeric numeric  markerProperties
	  {FR->createLineCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),
	    FITS->map(Vector($4,$5),CANVAS,REF), 0, 0,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| TEXT_ numeric numeric optangle markerProperties
	  {FR->createTextCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),$4,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| RULER_ numeric numeric numeric numeric markerProperties
	  {FR->createRulerCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),
	    FITS->map(Vector($4,$5),CANVAS,REF), PHYSICAL, PHYSICAL, PIXELS,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| PROJECTION_ numeric numeric numeric numeric numeric markerProperties
	  {FR->createProjectionCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),
	    FITS->map(Vector($4,$5),CANVAS,REF),
	    FITS->mapLenToRef($6,CANVAS),	    
	    NULL,NULL,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| PROJECTION_ numeric numeric numeric numeric numeric 
	    STRING STRING markerProperties
	  {FR->createProjectionCmd(
	    FITS->map(Vector($2,$3),CANVAS,REF),
	    FITS->map(Vector($4,$5),CANVAS,REF),
	    FITS->mapLenToRef($6,CANVAS),	    
	    $7,$8,
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| CIRCLE_ POINT_ numeric numeric markerProperties
	  {FR->createCirclePointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| BOX_ POINT_ numeric numeric markerProperties
	  {FR->createBoxPointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| DIAMOND_ POINT_ numeric numeric markerProperties
	  {FR->createDiamondPointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| CROSS_ POINT_ numeric numeric markerProperties
	  {FR->createCrossPointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| X_ POINT_ numeric numeric markerProperties
	  {FR->createExPointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| ARROW_ POINT_ numeric numeric markerProperties
	  {FR->createArrowPointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	| BOXCIRCLE_ POINT_ numeric numeric markerProperties
	  {FR->createBoxCirclePointCmd(FITS->map(Vector($3,$4),CANVAS,REF),
	    currentColor,currentWidth,currentFont,
	    currentText,currentProps,NULL);}
	;

markerEdit : BEGIN_ numeric numeric INT 
	  {FR->markerEditBeginCmd(Vector($2,$3),$4);}
	| MOTION_ numeric numeric INT 
	  {FR->markerEditMotionCmd(Vector($2,$3),$4);}
	| END_ {FR->markerEditEndCmd();}
	;

markerFormat : DS9_ {$$ = Frame::DS9;}
	| CIAO_ {$$ = Frame::CIAO;}
	| SAOTNG_ {$$ = Frame::SAOTNG;}
	| SAOIMAGE_ {$$ = Frame::SAOIMAGE;}
	| PROS_ {$$ = Frame::PROS;}
	| XY_ {$$ = Frame::RAWXY;}
	;

markerGet : COLOR_ {FR->getMarkerColorCmd();}
	| FONT_ {FR->getMarkerFontCmd();}
	| HANDLE_ numeric numeric {FR->getMarkerHandleCmd(Vector($2,$3));}
	| ID_ numeric numeric {FR->getMarkerIdCmd(Vector($2,$3));}
	| ID_ ALL_ {FR->getMarkerIdAllCmd();}
	| INT ANGLE_ coordSystem
	  {FR->getMarkerAngleCmd($1,(CoordSystem)$3);}
	| INT ANNULUS_ RADIUS_ distFormat
	  {FR->getMarkerAnnulusRadiusCmd($1,
	    currentDistSystem, currentDistFormat);}
	| INT BOX_ ANNULUS_ SIZE_ distFormat
	  {FR->getMarkerBoxAnnulusSizeCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT BOX_ SIZE_ distFormat
	  {FR->getMarkerBoxSizeCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT CENTER_ coordSystem coordFormat 
	  {FR->getMarkerCenterCmd($1, (CoordSystem)$3, (CoordFormat)$4);}
	| INT CIRCLE_ RADIUS_ distFormat
	  {FR->getMarkerCircleRadiusCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT COLOR_ {FR->getMarkerColorCmd($1);}
	| INT ELLIPSE_ RADIUS_ distFormat
	  {FR->getMarkerEllipseRadiusCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT ELLIPSE_ ANNULUS_ RADIUS_ distFormat
	  {FR->getMarkerEllipseAnnulusRadiusCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT FONT_ {FR->getMarkerFontCmd($1);}
	| INT LINE_ ARROW_ {FR->getMarkerLineArrowCmd($1);}
	| INT LINE_ POINT_ coordSystem coordFormat
	  {FR->getMarkerLineCmd($1, (CoordSystem)$4, (CoordFormat)$5);}
	| INT LINE_ DISTANCE_ distFormat
	  {FR->getMarkerLineDistanceCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT PANDA_ ANGLE_ coordSystem 
	  {FR->getMarkerPandaAnglesCmd($1, (CoordSystem)$4);}
	| INT PANDA_ RADIUS_ distFormat
	  {FR->getMarkerPandaRadiusCmd($1,
	    currentDistSystem, currentDistFormat);}
	| INT PROJECTION_ STRING STRING {FR->getMarkerProjectionCmd($1,$3,$4);}
	| INT PROJECTION_ POINT_ coordSystem coordFormat
	  {FR->getMarkerProjectionPointsCmd($1, 
	    (CoordSystem)$4, (CoordFormat)$5);}
	| INT PROJECTION_ DISTANCE_ distFormat
	  {FR->getMarkerProjectionDistanceCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT PROJECTION_ METHOD_ {FR->getMarkerProjectionMethodCmd($1);} 
	| INT PROJECTION_ THICK_ distFormat
	  {FR->getMarkerProjectionWidthCmd($1, 
	    currentDistSystem, currentDistFormat);}
	| INT PROPERTY_ {FR->getMarkerPropertyCmd($1);}
	| INT PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($1,$3);}
	| INT RULER_ DISTANCE_ distFormat
	  {FR->getMarkerRulerDistanceCmd($1,
	    currentDistSystem, currentDistFormat);}
	| INT RULER_ POINT_ coordSystem coordFormat
	  {FR->getMarkerRulerPointCmd($1, (CoordSystem)$4, (CoordFormat)$5);}
	| INT TEXT_ {FR->getMarkerTextCmd($1);}
	| INT TYPE_ {FR->getMarkerTypeCmd($1);}
	| INT WIDTH_ {FR->getMarkerLineWidthCmd($1);}
	| POLYGON_ SEGMENT_ numeric numeric
	  {FR->getMarkerPolygonSegmentCmd(Vector($3,$4));}
	| PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($2);}
	| SELECT_ markergetselect
	| SELECT_ NUMBER_ {FR->getMarkerSelectedNumberCmd();}
	| WIDTH_ {FR->getMarkerLineWidthCmd();}
	;

markergetselect : /* empty */ {FR->getMarkerSelectedCmd();}
	| numeric numeric {FR->getMarkerSelectedCmd(Vector($1,$2));}
	;

markerMoveID : numeric numeric {FR->markerMoveCmd(currentID, Vector($1,$2));}
	| FRONT_ {FR->markerFrontCmd(currentID);}
	| BACK_ {FR->markerBackCmd(currentID);}
	| BEGIN_ numeric numeric 
	  {FR->markerMoveBeginCmd(currentID, Vector($2,$3));}
	| MOTION_ numeric numeric 
	  {FR->markerMoveMotionCmd(currentID, Vector($2,$3));}
	| END_ {FR->markerMoveEndCmd(currentID);}
	| TO_ coordSystem coord 
	  {FR->markerMoveToCmd(currentID, Vector($3), (CoordSystem)$2);}
	;

markerMoveSelected : numeric numeric {FR->markerMoveCmd(Vector($1,$2));}
	| FRONT_ {FR->markerFrontCmd();}
	| BACK_ {FR->markerBackCmd();}
	| BEGIN_ numeric numeric {FR->markerMoveBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {FR->markerMoveMotionCmd(Vector($2,$3));}
	| END_ {FR->markerMoveEndCmd();}
	| TO_ coordSystem coord 
	  {FR->markerMoveToCmd(Vector($3), (CoordSystem)$2);}
	;

orient	: X_ {FR->orientCmd(XX);}
	| Y_ {FR->orientCmd(YY);}
	| XY_ {FR->orientCmd(XY);}
	| NONE_ {FR->orientCmd(NORMAL);}
	;

pan	: numeric numeric numeric numeric 
	  {FR->panCmd(Vector($1,$2),Vector($3,$4));}
	| coordSystem coord {FR->panCmd(Vector($2), (CoordSystem)$1);}
	| TO_ coordSystem coord {FR->panToCmd(Vector($3), (CoordSystem)$2);}
	| BBOX_ numeric numeric numeric numeric
	  {FR->panBBoxCmd(BBox($2,$3,$4,$5));}
	| MOTION_ panMotion
	;

panMotion : BEGIN_ numeric numeric {FR->panMotionBeginCmd(Vector($2,$3));}
	| numeric numeric {FR->panMotionCmd(Vector($1,$2));}
	| END_ numeric numeric {FR->panMotionEndCmd(Vector($2,$3));}
	;

panner	: yesno {FR->pannerCmd($1);}
	| STRING INT INT {FR->pannerCmd($1, $2, $3);}
	| UPDATE_ {FR->updatePannerCmd();}
	;

postscript : COLORSPACE_ pscolorspace 
	  {FR->psColorSpaceCmd((Frame::PSColorSpace)$2);}
	| INTERPOLATE_ yesno {FR->psInterpolateCmd($2);}
	| LEVEL_ INT {FR->psLevelCmd($2);}
	| RESOLUTION_ INT {FR->psResolutionCmd($2);}
	;

pscolorspace : BW_ {$$ = Frame::BW;}
	| GRAY_ {$$ = Frame::GRAY;}
	| RGB_ {$$ = Frame::RGB;}
	| CMYK_ {$$ = Frame::CMYK;}
	;

region	: SELECT_ regionSelect
	;

regionSelect : BEGIN_ numeric numeric 
	  {FR->regionSelectBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {FR->regionSelectMotionCmd(Vector($2,$3));}
	| END_ {FR->regionSelectEndCmd();}
	| SHIFT_ END_ {FR->regionSelectShiftEndCmd();}
	;

rotate	: numeric {FR->rotateCmd(degToRad($1));}
	| numeric DEGREES_ {FR->rotateCmd(degToRad($1));}
	| MOTION_ rotateMotion
	| TO_ numeric {FR->rotateToCmd(degToRad($2));}
	| TO_ numeric DEGREES_ {FR->rotateToCmd(degToRad($2));}
	;

rotateMotion : BEGIN_ {FR->rotateMotionBeginCmd();}
	| numeric {FR->rotateMotionCmd(degToRad($1));}
	| END_ {FR->rotateMotionEndCmd();}
	;

save	: FITS_ saveFits
	;

saveFits: IMAGE_ saveFitsImage
	| TABLE_ saveFitsTable
	| RESAMPLE_ saveFitsResample
	;

saveFitsImage : FILE_ STRING compressType {FR->saveFitsImageFileCmd($2, $3);}
	| CHANNEL_ STRING compressType {FR->saveFitsImageChannelCmd($2, $3);}
	| SOCKET_ INT compressType {FR->saveFitsImageSocketCmd($2, $3);}
	;

saveFitsTable : FILE_ STRING compressType {FR->saveFitsTableFileCmd($2, $3);}
	| CHANNEL_ STRING compressType {FR->saveFitsTableChannelCmd($2, $3);}
	| SOCKET_ INT compressType {FR->saveFitsTableSocketCmd($2, $3);}
	;

saveFitsResample : FILE_ STRING compressType 
	  {FR->saveFitsResampleFileCmd($2, $3);}
	| CHANNEL_ STRING compressType 
	  {FR->saveFitsResampleChannelCmd($2, $3);}
	| SOCKET_ INT compressType 
	  {FR->saveFitsResampleSocketCmd($2, $3);}
	;

set	: NAME_ STRING {FR->setNameCmd($2);}
	;

update	: /* empty */ {FR->updateFitsCmd(0);}
	| INT numeric numeric numeric numeric
	  {FR->updateFitsCmd($1,BBox($2,$3,$4,$5),0);}
	| NOW_ {FR->updateFitsCmd(1);}
	| NOW_ INT numeric numeric numeric numeric
	  {FR->updateFitsCmd($2,BBox($3,$4,$5,$6),1);}
	| FITS_ SLICE_ INT {FR->sliceCmd($3);}
	;

warp	: /* empty */ numeric numeric {FR->warpCmd(Vector($1,$2));}
	| TO_ numeric numeric {FR->warpToCmd(Vector($2,$3));}
	;

wcs	: ALIGN_ yesno {FR->wcsAlignCmd($2);}
	| ZOOM_ numeric {FR->wcsZoomCmd($2);}
	| RESET_ {FR->wcsResetCmd();}
	| REPLACE_ INT {FR->wcsReplaceCmd($2);}
	| REPLACE_ STRING {FR->wcsReplaceCmd($2);}
	| APPEND_ INT {FR->wcsAppendCmd($2);}
	| APPEND_ STRING {FR->wcsAppendCmd($2);}
	;

zoom	: numeric {FR->zoomCmd($1);}
	| numeric ABOUT_ coordSystem coord 
	  {FR->zoomAboutCmd($1, Vector($4), (CoordSystem)$3);}
	| TO_ zoomTo
	;

zoomTo	: FIT_	{FR->zoomToFitCmd();}
	| numeric {FR->zoomToCmd($1);}
	| numeric ABOUT_ coordSystem coord
	  {FR->zoomToAboutCmd($1, Vector($4), (CoordSystem)$3);}
	;

%%

static void setProps(unsigned short* props, unsigned short prop, int value)
{
  if (value)
    *props |= prop;
  else
    *props &= ~prop;
}
