#ifndef __point_h__
#define __point_h__

#include "marker.h"

class Point : public Marker {
 protected:
  char pointtype[16];

protected:
  virtual void render(Drawable, const Matrix&, double, RenderMode);
  void setLineDash();
  void psLineDash();

public:
  Point(const Point&);
  Point(Frame* p, const Vector& ctr, const char* clr, int w, const char* f,
	const char* t, unsigned short prop, const char* c);

  virtual Marker* dup() =0;
  void updateBBox();
  void rotate(const Vector& v, int h) {}

  virtual void ps(int);

  virtual void list(ostream&, CoordSystem, CoordFormat, char);
  void listProperties(ostream&, char);
  void listCiao(ostream&, CoordSystem, CoordFormat, char);
  void listPros(ostream&, CoordSystem, CoordFormat, char);
  void listSAOtng(ostream&, CoordSystem, CoordFormat, char);
  void listSAOimage(ostream&, CoordSystem, CoordFormat, char);
};

class CirclePoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  CirclePoint(const CirclePoint&);
  CirclePoint(Frame* p, const Vector& ctr, 
	      const char* clr, int w, const char* f,
	      const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new CirclePoint(*this);}
  void ps(int);
};

class BoxPoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  BoxPoint(const BoxPoint&);
  BoxPoint(Frame* p, const Vector& ctr, 
	   const char* clr, int w, const char* f,
	   const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new BoxPoint(*this);}
  void ps(int);
};

class DiamondPoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  DiamondPoint(const DiamondPoint&);
  DiamondPoint(Frame* p, const Vector& ctr, 
	       const char* clr, int w, const char* f,
	       const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new DiamondPoint(*this);}
  void ps(int);
};

class CrossPoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  CrossPoint(const CrossPoint&);
  CrossPoint(Frame* p, const Vector& ctr, 
	     const char* clr, int w, const char* f,
	     const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new CrossPoint(*this);}
  void ps(int);
};

class ExPoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  ExPoint(const ExPoint&);
  ExPoint(Frame* p, const Vector& ctr, 
	  const char* clr, int w, const char* f,
	  const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new ExPoint(*this);}
  void ps(int);
};

class ArrowPoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  ArrowPoint(const ArrowPoint&);
  ArrowPoint(Frame* p, const Vector& ctr,
	     const char* clr, int w, const char* f,
	     const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new ArrowPoint(*this);}
  void ps(int);
};

class BoxCirclePoint : public Point {
private:
  void render(Drawable, const Matrix&, double, RenderMode);

public:
  BoxCirclePoint(const BoxCirclePoint&);
  BoxCirclePoint(Frame* p, const Vector& ctr, 
		 const char* clr, int w, const char* f,
		 const char* t, unsigned short prop, const char* c);

  Marker* dup() {return new BoxCirclePoint(*this);}
  void ps(int);
};

#endif
